#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : Oct 28 2016
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib '/usr/local/esia/plugins/lib/perl';
use JSON;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;

my $PROGNAME = 'check_snmp_ap_ubiquity.pl';
my $PROGDESC = 'Check SNMP Ubiquity access point CPU, ram and clients';
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS = %ESIA::PLUGIN::ERRORS;

my $return=0;
my ($opt_h,$opt_V,$json);
my $type = 'cpu';
my $warning = 80;
my $critical = 90;

my $ssid_name='';
my $oid_sysobject='.1.3.6.1.2.1.1.2.0';
my $oid_cpu_name  ='.1.3.6.1.4.1.10002.1.1.1.4.2.1.2';
my $oid_cpu_load  ='.1.3.6.1.4.1.10002.1.1.1.4.2.1.3';
my $oid_mem_total ='1.3.6.1.4.1.10002.1.1.1.1.1.0';
my $oid_mem_free  ='1.3.6.1.4.1.10002.1.1.1.1.2.0';
my $oid_mem_cache ='1.3.6.1.4.1.10002.1.1.1.1.4.0'; # Real memory cached
my $oid_ssid_name ='1.3.6.1.4.1.41112.1.4.5.1.2';
my $oid_ssid_station_count='1.3.6.1.4.1.41112.1.4.5.1.15';
#UNIFY
my $oid_default_cpu = "1.3.6.1.2.1.25.3.3.1.2";
my $oid_default_mem_total ='1.3.6.1.4.1.2021.4.5.0';
my $oid_default_mem_free  ='1.3.6.1.4.1.2021.4.6.0';
my $oid_default_mem_cache ='1.3.6.1.4.1.2021.4.15.0'; # Real memory cached
my $oid_unify_ssid_name ='1.3.6.1.4.1.41112.1.6.1.2.1.6';
my $oid_unify_ssid_station_count='1.3.6.1.4.1.41112.1.6.1.2.1.8';
my $oid_unify_ssid_station_type='1.3.6.1.4.1.41112.1.6.1.2.1.9';
my $oid_unify_model = '1.3.6.1.4.1.41112.1.6.3.3.0';
my $oid_unify_firmware = '1.3.6.1.4.1.41112.1.6.3.6.0';

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"Type=s" => \$type,
	"n=s" => \$ssid_name,"name=s" => \$ssid_name,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
    "J" => \$json, "json" => \$json,
);
if($opt_h)
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T (cpu|mem|client|inventory)');
	my @arr;
	push(@arr,["-w","--warning","warning threshold (default: $warning %)" ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical %)"]);
	push(@arr,["-T","--type","Type of test [cpu|mem|client|inventory] (default: $type )"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if(!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $result;
my $sysobject = ESIA::SNMP::get($snmp, "$oid_sysobject");
$result = $snmp->get_table(Baseoid => $oid_default_cpu);
if(defined $result)
{
	$sysobject = "0";
}
if($sysobject =~/4\.1\.41112/)
{
    if($type eq 'cpu')
    {
        $result = $snmp->get_table(Baseoid => $oid_cpu_name);
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $name =$$result{"$key"};
                my $message ="< $warning%";
                if($name =~/^5\s+Minute/)
                {
                    my $cpu = ESIA::SNMP::get($snmp,"$oid_cpu_load.$index");
                    my $state='OK';
                    if($cpu > $critical)
                    {
                        $state='CRITICAL';
                        $message ="< $critical%";
                    }
                    elsif($cpu > $warning)
                    {
                        $state='WARNING';
                        $message ="> $warning%";
                    }
                    print "$state : $name is used $cpu % $message|cpu=$cpu\n";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                    last;
                }
            }
        }
    }
    elsif($type eq 'mem')
    {
        my $total = ESIA::SNMP::get($snmp,"$oid_mem_total");
        my $free = ESIA::SNMP::get($snmp,"$oid_mem_free");
        # my $cache = ESIA::SNMP::get($snmp,"$oid_mem_cache");
        my $used = $total - $free;

        $used = sprintf("%.0f", (($used/$total)*100));
        my $message ="< $warning%";
        my $state='OK';
        if($used > $critical)
        {
            $state='CRITICAL';
            $message ="> $critical%";
        }
        elsif($used > $warning)
        {
            $state='WARNING';
            $message ="> $warning%";
        }
        print "$state : memory is used $used % $message|mem=$used\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'client')
    {
        $result = $snmp->get_table(Baseoid => $oid_unify_ssid_name);
        my $total_client=0;
        my $found=0;
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $name =$$result{"$key"};
                if($ssid_name eq '' || $ssid_name=~/$name/)
                {
                    my $client = ESIA::SNMP::get($snmp,"$oid_unify_ssid_station_count.$index");
                    $total_client = $total_client + $client;
                    my $message ="< $warning";
                    my $state='OK';
                    if($client > $critical)
                    {
                        $state='CRITICAL';
                        $message ="< $critical";
                    }
                    elsif($client > $warning)
                    {
                        $state='WARNING';
                        $message ="> $warning";
                    }
                    print "$state : SSID $name: number of client: $client $message\n";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                    $found=1;
                }
            }
        }
        if($ssid_name eq '')
        {
            my $message ="< $warning";
            my $state='OK';
            if($total_client > $critical)
            {
                $state='CRITICAL';
                $message ="< $critical";
            }
            elsif($total_client > $warning)
            {
                $state='WARNING';
                $message ="> $warning";
            }
            print "$state : total of client: $total_client $message\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
        else
        {
            if($found==0)
            {
                print "UNKNOWN : SSID $ssid_name not found\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
            }
        }
        print"|totalclient=$total_client";
    }
    elsif($type eq 'inventory')
    {
        my $version = ESIA::SNMP::get($snmp,"$oid_unify_firmware");
		my $model = ESIA::SNMP::get($snmp,"$oid_unify_model");
		my @output;
		my %hash = (
			'name' => "Ubiquiti $model",
			'description'=>'',
			'manufacturer'=>'Ubiquiti',
			'model' => $model,
			'type'=>'CHASSIS',
			'serial' =>'',
			'hardwareVersion' => '',
			'firmwareVersion' => $version,
		);
		push(@output, \%hash);
		if($json)
		{
			my $out = encode_json(\@output);
			print $out;
		}
		else
		{
		    my $size = @output;
		    if($size != 0)
		    {
		        foreach my $line ( @output)
		        {
		            my $hash = \%$line;
		            print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
		            print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		        }
		        exit 0;
		    }
		}
    }
}
else
{
    if($type eq 'cpu')
    {
        $result = $snmp->get_table(Baseoid => $oid_default_cpu);
        my $nbr=0;
        my $totalcpu=0;
        foreach my $key ( keys %$result)
        {
            $totalcpu=$totalcpu+$$result{"$key"};
            $nbr++;
        }
        my $cpu = int($totalcpu/$nbr);
        my $state='OK';
        my $message ="< $warning";
        if($cpu > $critical)
        {
            $state='CRITICAL';
            $message ="< $critical%";
        }
        elsif($cpu > $warning)
        {
            $state='WARNING';
            $message ="> $warning%";
        }
        print "$state : CPU is used $cpu % $message|cpu=$cpu\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'mem')
    {
        my $total = ESIA::SNMP::get($snmp,"$oid_default_mem_total");
        my $free = ESIA::SNMP::get($snmp,"$oid_default_mem_free");
        my $cache = ESIA::SNMP::get($snmp,"$oid_default_mem_cache");
        my $used = $total - $free-$cache;
        $used = sprintf("%.0f", (($used/$total)*100));
        my $message ="< $warning%";
        my $state='OK';
        if($used > $critical)
        {
            $state='CRITICAL';
            $message ="> $critical%";
        }
        elsif($used > $warning)
        {
            $state='WARNING';
            $message ="> $warning%";
        }
        print "$state : memory is used $used % $message|mem=$used\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'client')
    {
        $result = $snmp->get_table(Baseoid => $oid_unify_ssid_name);
        my $total_client=0;
        my $found=0;
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $name =$$result{"$key"};
                if($ssid_name eq '' || $ssid_name=~/$name/)
                {
                    my $client = ESIA::SNMP::get($snmp,"$oid_unify_ssid_station_count.$index");
                    my $stationtype = ESIA::SNMP::get($snmp,"$oid_unify_ssid_station_type.$index");
                    $total_client = $total_client + $client;
                    my $message ="< $warning";
                    my $state='OK';
                    if($client > $critical)
                    {
                        $state='CRITICAL';
                        $message ="< $critical";
                    }
                    elsif($client > $warning)
                    {
                        $state='WARNING';
                        $message ="> $warning";
                    }
                    print "$state : SSID $name ($stationtype): number of client: $client $message\n";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                    $found=1;
                }
            }
        }
        if($ssid_name eq '')
        {
            my $message ="< $warning";
            my $state='OK';
            if($total_client > $critical)
            {
                $state='CRITICAL';
                $message ="< $critical";
            }
            elsif($total_client > $warning)
            {
                $state='WARNING';
                $message ="> $warning";
            }
            print "$state : total of client: $total_client $message\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
        else
        {
            if($found==0)
            {
                print "UNKNOWN : SSID $ssid_name not found\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
            }
        }
        print"|totalclient=$total_client";
    }
    elsif($type eq 'inventory')
    {
        my $version = '';
		my $model = '';
		my @output;
		my %hash = (
			'name' => "Ubiquiti $model",
			'description'=>'',
			'manufacturer'=>'Ubiquiti',
			'model' => $model,
			'type'=>'CHASSIS',
			'serial' =>'',
			'hardwareVersion' => '',
			'firmwareVersion' => $version,
		);
		push(@output, \%hash);
		if($json)
		{
			my $out = encode_json(\@output);
			print $out;
		}
		else
		{
		    my $size = @output;
		    if($size != 0)
		    {
		        foreach my $line ( @output)
		        {
		            my $hash = \%$line;
		            print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
		            print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		        }
		        exit 0;
		    }
		}
    }
}

exit($return);
