#!/usr/bin/perl
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
#use lib "/root/libperl/"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;
use POSIX qw( strftime );
use JSON;
#nom du programme & description
my $PROGNAME = "check_snmp_checkpoint.pl";
my $PROGDESC = "CheckPoint firewall by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libmath-round-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);
my $oid_sysObjectOID ='1.3.6.1.2.1.1.2.0';
my $oid_licence_name ='1.3.6.1.4.1.2620.1.6.18.1.1.4';
my $oid_licence_state='1.3.6.1.4.1.2620.1.6.18.1.1.5';
my $oid_licence_exp  ='1.3.6.1.4.1.2620.1.6.18.1.1.6';

my $oid_sensors_name        ='1.3.6.1.4.1.2620.1.6.7.8.1.1.2';
my $oid_sensors_temperature ='1.3.6.1.4.1.2620.1.6.7.8.1.1.3';
my $oid_sensors_type        ='1.3.6.1.4.1.2620.1.6.7.8.1.1.5';
my $oid_sensors_val         ='1.3.6.1.4.1.2620.1.6.7.8.1.1.4';

my $oid_product_model         ='1.3.6.1.4.1.2620.1.6.16.7';
my $oid_product_serial        ='1.3.6.1.4.1.2620.1.6.16.3';
my $oid_product_manufacturer  ='1.3.6.1.4.1.2620.1.6.16.9';
my $oid_product_serie         ='1.3.6.1.4.1.2620.1.6.16.1';
my $oid_product_firmware      ='1.3.6.1.4.1.2620.1.6.4.1';

Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V,$json);
my $community = "public";
my $snmp_version = 1;
my ($hostname);
my $warning=80;
my $critical=90;
my $return =0;
my $type;
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::Session(@ARGV);
my $match;
GetOptions(
	"V"   => \$opt_V,	 "version"    => \$opt_V,
	"h"   => \$opt_h,	 "help"       => \$opt_h,
	"T=s" => \$type,	 "Type=s" => \$type,
	"m=s" => \$match,	 "match=s" => \$match,
	"w=i" => \$warning,	 "warning=i" => \$warning,
	"c=i" => \$critical, "critical=i" => \$critical,
	"J" => \$json,	 "json" => \$json,
);

# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [fan|psu|temp] [options]');
    my @arr;
    push(@arr,["-T","--type","type of test [fan,cpu,unit]" ]);
    push(@arr,["-w","--warning","warning threshold max in (degrees for temp, percent for cpu)" ]);
    push(@arr,["-c","--critical","critical threshold max in (degrees for temp, percent for cpu)"]);
    push(@arr,["","--show-ignore","Print also components/values ignored"]);
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
    exit $ERRORS{'OK'};
}

# -V & --version print version
if ($opt_V)
{
    print "$PROGNAME version $PROGVERSION\n";
    exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "";

my $sysOid_value = ESIA::SNMP::get($snmp,$oid_sysObjectOID);
if( !defined( $sysOid_value ) )
{
	print ("UNKNOWN: SNMP no response. $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

if($type eq 'licence')
{
	my $result = $snmp->get_table( $oid_licence_state);
	foreach my $key (sort keys %$result)
	{
		if($key=~/\.([0-9]*)\.0$/)
		{
			 my $id = $1;
			 my $licstate = $$result{"$key"};
			 if($licstate eq "Entitled")
			 {
			 	my $state='OK';
			 	my $name = ESIA::SNMP::get($snmp,"$oid_licence_name.$id.0");
			 	my $expire = ESIA::SNMP::get($snmp,"$oid_licence_exp.$id.0");
			 	my $message ='never';
			 	if($expire !=0)
			 	{
					my $dt =strftime("%d/%m/%Y", localtime($expire));
					$message = "expire on $dt";
					my $now = time();
					my $delta = $expire - $now;
					my $cmp_warn=$warning*24*60*60;
					my $cmp_cri=$critical*24*60*60;
					if($delta < $cmp_cri)
					{
						$state='CRITICAL';
						$message.= "(less than $critical days)";
					}
					elsif($delta < $cmp_warn)
					{
						$state='WARNING';
						$message.= "(less than $warning days)";
					}
				}
				else
				{
					$message = "never expire";
				}
				print "$state : the $name license $message\n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			 }
		}
	}
}
elsif($type eq 'temp')
{
	my $graph ='';
	my $result = $snmp->get_table( $oid_sensors_type);
	foreach my $key (sort keys %$result)
	{
		if($key=~/\.([0-9]*)\.0$/)
		{
			 my $id = $1;
			 my $sensorType = $$result{"$key"};
			 if($sensorType eq "temperature")
			 {
			 	my $state='OK';
			 	my $name = ESIA::SNMP::get($snmp,"$oid_sensors_name.$id.0");
			 	my $temp = ESIA::SNMP::get($snmp,"$oid_sensors_temperature.$id.0");
			 	my $val = ESIA::SNMP::get($snmp,"$oid_sensors_val.$id.0");
				$temp = sprintf("%.1f", $temp);
				my $message = "< $warning";
				if($temp > $critical)
				{
					$message = "> $critical";
					$state='CRITICAL';
				}
				elsif($temp > $warning)
				{
					$message = "> $warning";
					$state='WARNING';
				}
				if($name eq 'CPU Temperature')
				{
					$graph.="cpu=$temp;";
				}
				elsif($name eq 'CPU Temperature(internal)')
				{
					$graph.="int=$temp;";
				}
				print "$state : the $name is $temp $val $message\n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			 }
		}
	}
	print "|$graph";
}
elsif($type eq 'inventory')
{
	my @array;
	my $series = ESIA::SNMP::get($snmp,"$oid_product_serie.0");
	my $model = ESIA::SNMP::get($snmp,"$oid_product_model.0");
	$model=~ s/^\s+|\s+$//g;
	my $firmware = ESIA::SNMP::get($snmp,"$oid_product_firmware.0");
	$firmware=~ s/^\s+|\s+$//g;
	my $serial = ESIA::SNMP::get($snmp,"$oid_product_serial.0");
	$serial=~ s/^\s+|\s+$//g;
	my %hash = ('name' => "$series",'manufacturer'=>'CheckPoint', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@array, \%hash);

	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
else
{
	print ("UNKNOWN: Incorrect or missed option -T, --type\n");
	$return = $ERRORS{'UNKNOWN'};
}
exit $return;
