#!/usr/bin/perl -w
############################## check_snmp_syno_diskhealth.pl ########
my $Version='1.0';
# Date : Jul 03 2024
# Author  : GEORGE Julien
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;
use JSON;
my $PROGNAME='check_snmp_fortinet.pl';
my $PROGDESC = "Check fortigate (fortinet manufacturer) sensors by SNMP (CPU, RAM,Session,VPN) ";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my %ENTITY_PHYSICAL_CLASS= %ESIA::INVENTORY::ENTITY_PHYSICAL_CLASS;

my $return=0;
my ($opt_h,$opt_V,$json);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical = 90;
my $match_name;
my $type = "CPU";
## OIDs ##
my $oid_sysObjectOID 	 = ".1.3.6.1.2.1.1.2.0";
my $oid_unitdesc         = ".1.3.6.1.2.1.1.5.0";                   # Location of Fortinet device description... (String)
my $oid_serial           = ".1.3.6.1.4.1.12356.100.1.1.1.0";       # Location of Fortinet serial number (String)

## FortiMail OIDs ##
my $oid_fe_model    	 = ".1.3.6.1.4.1.12356.105.1.1.0";		   # System model string        
my $oid_fe_serial   	 = ".1.3.6.1.4.1.12356.105.1.2.0";         # Device serial number
my $oid_fe_version  	 = ".1.3.6.1.4.1.12356.105.1.3.0";         # Device firmware version
my $oid_fe_versionAv	 = ".1.3.6.1.4.1.12356.105.1.4.0";         # Antivirus DB version

my $oid_fe_cpu           = ".1.3.6.1.4.1.12356.105.1.6.0";         # Location of CPU for FortiMail (%)
my $oid_fe_mem           = ".1.3.6.1.4.1.12356.105.1.7.0";         # Location of Memory used for FortiMail (%)
my $oid_fe_ldisk         = ".1.3.6.1.4.1.12356.105.1.8.0";         # Location of Log Disk used for FortiMail (%)
my $oid_fe_mdisk         = ".1.3.6.1.4.1.12356.105.1.9.0";         # Location of Mail Disk used for FortiMail (%)
my $oid_fe_load          = ".1.3.6.1.4.1.12356.105.1.30.0";        # Location of Load used for FortiMail (%)
my $oid_fe_ses           = ".1.3.6.1.4.1.12356.105.1.10.0";        # Location of cluster member Sessions for FortiMail (int)

my $oid_fe_mqueue_idx	 = ".1.3.6.1.4.1.12356.105.1.103.2.1.1";
my $oid_fe_mqueue_name	 = ".1.3.6.1.4.1.12356.105.1.103.2.1.2";
my $oid_fe_mqueue_value	 = ".1.3.6.1.4.1.12356.105.1.103.2.1.3";

ESIA::PLUGIN::arm_end_signal($TIMEOUT);
#
#MAIN
#
&Getopt::Long::config('bundling');
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"n=s" => \$match_name,"name=s" => \$match_name,
	"J" => \$json,	 "json" => \$json
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T <type>');
	my @arr;
	push(@arr,["-T","--type","type of check [CPU|MEM|maildisk|kogdisk|mailload|mailqueue]" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	push(@arr,["-n","--name","name or key of a value to match (mailqueue name, ...)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on interroge en SNMP pour récupérer les informations

my $result = ESIA::SNMP::get($snmp,"$oid_serial");

#si l'host ne répond pas on blocke le script
if (!defined($result))
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}

my $sysOid_value = ESIA::SNMP::get($snmp,"$oid_sysObjectOID");

my $curr_serial =  $result;
my $curr_desc = ESIA::SNMP::get($snmp,"$oid_unitdesc");
my $message ="";
my $return_state = "OK";


if($type eq 'CPU' || $type eq 'cpu')
{
	$message = get_cpu();
}
elsif($type eq 'MEM'|| $type eq 'mem')
{
	$message = get_mem();
}
elsif($type eq 'logdisk' )
{
	$message = get_fortimailPerc('Log Disk',$oid_fe_ldisk);
}
elsif($type eq 'maildisk' )
{
	$message = get_fortimailPerc('Mail Disk',$oid_fe_mdisk);
}
elsif($type eq 'mailload' )
{
	$message = get_fortimailPerc('Load',$oid_fe_load);
}
elsif($type eq 'mailqueue' )
{
	$message = get_mailqueue();
}
elsif($type eq 'inventory')
{
    my @output;
	my $type = 'os';
	my $model = ESIA::SNMP::get($snmp,"$oid_fe_model");
	my $name = $model;
	my $manufacturer = 'Fortinet';
	my $serial = ESIA::SNMP::get($snmp,"$oid_fe_serial");
	my $version = ESIA::SNMP::get($snmp,"$oid_fe_version");
	my %hash = ('name' => $name,'manufacturer'=>$manufacturer, 'model' => $model, 'type'=>$type,'serial'=>$serial,'version' => $version);
	push(@output, \%hash);	
	
	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
	    my $size = @output;
	    if($size != 0)
	    {
	        foreach my $line ( @output)
	        {
	            my $hash = \%$line;
	            print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				if($hash->{type} eq 'os' )
				{
					print ", version: ".$hash->{version}."\n";
				}
				else
				{
	            	print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
				}
	        }
	        exit 0;
	    }
	}
}
print "$message";
exit($return);

sub get_cpu
{
	my $message="";
    my $cpu_val = ESIA::SNMP::get($snmp,"$oid_fe_cpu");

	if (!defined($cpu_val))
	{
		print "CPU na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $status = "OK";
		my $endline="< ".$warning."%;";
		if ( $cpu_val >= $critical )
		{
			$status = "CRITICAL";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		}
		elsif ( $cpu_val >= $warning )
		{
			$status = "WARNING";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		}
		else
		{
			$status = "OK";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		}
		$message = "$status: $curr_desc(Current device: $curr_serial) load ".$cpu_val."% $endline |cpu=".$cpu_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_mem
{
	my $message="";
	my $mem_val = ESIA::SNMP::get($snmp,"$oid_fe_mem");

	if (!defined($mem_val))
	{
		print "Memory na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $status = "OK";
		my $endline="< ".$warning."%;";
		if ( $mem_val >= $critical )
		{
			$status = "CRITICAL";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		}
		elsif ( $mem_val >= $warning )
		{
			$status = "WARNING";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		}
		else
		{
			$status = "OK";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		}
		$message = "$status: $curr_desc(Current device: $curr_serial) memory is ".$mem_val."% $endline |memory=".$mem_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_fortimailPerc
{
	my($type_name,$oid) = @_;
	my $message="";
	my $disk_val;

	$disk_val = ESIA::SNMP::get($snmp,"$oid");

	if (!defined($disk_val))
	{
		print "$type_name na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $status = "OK";
		my $endline="< ".$warning."%;";
		if ( $disk_val >= $critical )
		{
			$status = "CRITICAL";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		}
		elsif ( $disk_val >= $warning )
		{
			$status = "WARNING";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		}
		else
		{
			$status = "OK";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		}
		$message = "$status: $curr_desc(Current device: $curr_serial) $type_name is ".$disk_val."% $endline |value=".$disk_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_mailqueue
{
	my @list;
	my $message="";
	my $result = $snmp->get_table( $oid_fe_mqueue_name);
	if (!defined($result))
	{
		print "mailqueue na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		foreach my $key ( keys %$result)
		{
			if($key=~ /([0-9]*)$/ )
		   	{
				my $id = $1;
				my $name = $$result{$key};
				push(@list,$name);
				my $value = ESIA::SNMP::get($snmp,"$oid_fe_mqueue_value.$1");
				if($value =~ m/noSuch/i)
				{
					print "SNMP ERROR: bad mailqueue value returned : $value\n";
					$snmp->close;
					exit $ERRORS{"UNKNOWN"};
				}
				if( defined($match_name) &&  $name eq $match_name )
				{
					my $status = "OK";
					my $endline="< ".$warning.";";
					if ( $value >= $critical )
					{
						$status = "CRITICAL";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
						$endline="> ".$critical.";";
					}
					elsif ( $value >= $warning )
					{
						$status = "WARNING";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
						$endline="> ".$warning.";";
					}
					else
					{
						$status = "OK";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
					}
					$message = "$status: $curr_desc(Current device: $curr_serial) $name: ".$value." $endline |mailqueue=".$value.";".$warning.";".$critical."\n";
					return $message;					
				}
			}
		}
		
		my $list_str = join("\n",@list);
		my @list;
		if( !defined($match_name) || $match_name eq "")
		{
			$message = "missing -n, --name opt to match mailqueue, must be one of : \n$list_str\n";
		}
		else
		{
			$message = "mailqueue $match_name not found, must be one of : \n$list_str\n";
		}
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
		return $message;
	}
}
