#!/usr/bin/perl -w
############################## check_snmp_syno_diskhealth.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;
use JSON;
my $PROGNAME='check_snmp_fortinet_ap.pl';
my $PROGDESC = "Check forti AP(fortinet manufacturer) sensors by SNMP (CPU, RAM,Session,VPN) ";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my %ENTITY_PHYSICAL_CLASS= %ESIA::INVENTORY::ENTITY_PHYSICAL_CLASS;
my $return=0;
my ($opt_h,$opt_V,$json);
my $warning = 80;
my $critical = 90;
my $type = "CPU";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my $oid_forti_version = "1.3.6.1.4.1.12356.120.1.1.0";
my $oid_forti_serial = "1.3.6.1.4.1.12356.120.1.2.0";
my $oid_forti_part = "1.3.6.1.4.1.12356.120.1.8.0";

my $oid_forti_client = "1.3.6.1.4.1.12356.120.8.1.1.1.1";
my $oid_forti_cawap_state = "1.3.6.1.4.1.12356.120.3.23.0";
#
#MAIN
#
&Getopt::Long::config('bundling');
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"J" => \$json,	 "json" => \$json
);
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-T","--type","type of check [CPU|MEM|Ses|VPN] (default: CPU)" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on interroge en SNMP pour récupérer les informations
if($type eq 'cpu' || $type eq 'mem')
{
		print "OK : CPU and memory OID are not implemented on this model\n";
}
elsif($type eq 'state')
{
	my $state = ESIA::SNMP::get($snmp,"$oid_forti_cawap_state");
	if($state eq 'RUN')
	{
		print "OK : CAWAP is $state state";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
	}
	else
	{
		print "CRITICAL : CAWAP is $state state";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	}
}
elsif( $type eq 'client')
{

 	my $result = $snmp->get_table( $oid_forti_client);
	my $nbr = 0;
    foreach my $key ( keys %$result)
    {
		$nbr++;
    }
    my $state = "OK";
    my $message = "";
    if($nbr < $warning)
    {
		$message = "< $warning";
    }
    elsif($nbr > $critical)
    {
		$message = "> $critical";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
    }
    elsif($nbr > $warning)
    {
		$message = "< $warning";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
    }
    print "$state : There are $nbr client connected $message|totalclient=$nbr";
}
elsif($type eq 'inventory')
{
	my @output;
	my $firmware = ESIA::SNMP::get($snmp,"$oid_forti_version");
	my $serial = ESIA::SNMP::get($snmp,"$oid_forti_serial");
    my $model = ESIA::SNMP::get($snmp,"$oid_forti_part");
	my %hash = ('name' => 'AP '.$model,'manufacturer'=>'Fortinet', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@output, \%hash);
	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
	    my $size = @output;
	    if($size != 0)
	    {
	        foreach my $line ( @output)
	        {
	            my $hash = \%$line;
	            print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
	            print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
	        }
	        exit 0;
	    }
	}
}
