#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use JSON;
use ESIA::SNMP;
use ESIA::PLUGIN;
use Data::Dumper;
use ESIA::CacheFile;

#nom du programme & description
my $PROGNAME = "check_snmp_fortinet_switch.pl";
my $PROGDESC = "Check fortinet SWITCH by SNMP";
my $PROGVERSION = "1.0";
my $path= "/tmp/fortiswitch/";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my $oid_switch_sn         ="1.3.6.1.4.1.12356.101.24.1.1.1.3";
my $oid_switch_name       ="1.3.6.1.4.1.12356.101.24.1.1.1.4";
my $oid_switch_version    ="1.3.6.1.4.1.12356.101.24.1.1.1.5";
my $oid_switch_authorized ="1.3.6.1.4.1.12356.101.24.1.1.1.6"; #discovered(0), disabled(1), authorized(2)
my $oid_switch_status     ="1.3.6.1.4.1.12356.101.24.1.1.1.7"; #1= ONLINE #0= DOWN
my $oid_switch_ip_address ="1.3.6.1.4.1.12356.101.24.1.1.1.10";

my $oid_switch_cpu_percent ="1.3.6.1.4.1.12356.101.24.1.1.1.11";
my $oid_switch_mem_percent ="1.3.6.1.4.1.12356.101.24.1.1.1.12";
my $warning = 80;
my $critical = 90;
my $type = "state";
my ($json,$match);
Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"T=s" => \$type,	 "Type=s" => \$type,
		"m=s" => \$match,	 "match=s" => \$match,
		"w=i" => \$warning,	 "warning=i" => \$warning,
		"J" => \$json,	 "json" => \$json,
		"c=i" => \$critical, "critical=i" => \$critical,
	  );


# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [state,cpu,mem] (default: state)" ]);
	push(@arr,["-w","--warning","warning threshold in percent for the line power (default: 80) or warning threshold time remaining in minute for battery test (default: 30 min)" ]);
	push(@arr,["-c","--critical","critical threshold in percent for the line power (default: 90) or critical threshold time remaining in minute for battery test (default: 10 min) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $hostname = $snmp->hostname();
my $filename = 'fsw_'.$hostname.'.json';
my $cachefile = ESIA::CacheFile->new($path,$filename);
my $switchs;
my $return = $ERRORS{'OK'};
if($cachefile->isValid() == 1)
{
    my $data = $cachefile->getData();
    $switchs = decode_json($data);
}
else
{
    my $result = $snmp->get_table( $oid_switch_name);
    my %hash;
    foreach my $key ( sort keys %$result)
    {
        if($key=~/\.([0-9]*\.[0-9]*\.[0-9]*)$/)
        {
            my $id = $1;
            print "oid: $id\n";
            my $name = $$result{"$key"};
            $hash{$name}=$id;
        }
    }
    my $json = encode_json \%hash;
    $cachefile->writeData($json);
    $switchs=\%hash;  
}
if(exists $switchs->{$match})
{
    my $id = $switchs->{$match};
    if($type eq 'state')
    {
        my $sn=  ESIA::SNMP::get($snmp,"$oid_switch_sn.$id");
        my $version=  ESIA::SNMP::get($snmp,"$oid_switch_sn.$id");
        my $status=  ESIA::SNMP::get($snmp,"$oid_switch_status.$id");
        my $message=" is online"; 
        my $state="OK";
        if($status !=1)
        {
            $state="CRITICAL";
            $message=" is offline"; 
        }
        else
        {
            my $authorize=  ESIA::SNMP::get($snmp,"$oid_switch_authorized.$id");
            my $ip=  ESIA::SNMP::get($snmp,"$oid_switch_ip_address.$id");
            if($authorize == 0)
            {  
                $message.=" and discovered (fixed IP: $ip)";
            }
            elsif($authorize == 1)
            {  
                $message.=" and disabled (fixed IP: $ip)";
            }  
            elsif($authorize == 2)
            {  
                $message.=" and authorized (fixed IP: $ip)";
            }  
        }
        print "$state: $match (sn: $sn and version: $version) $message\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'cpu') 
    {
        my $value=  ESIA::SNMP::get($snmp,"$oid_switch_cpu_percent.$id");
        my $state="OK";
        my $message=""; 
        if($value < $warning)
        {
            $state="OK";
            $message="< $warning"; 
        }
        elsif($value > $critical)
        {
            $state="CRITICAL";
            $message="> $warning"; 
        }
        elsif($value > $warning)
        {
            $state="WARNING";
            $message="> $warning"; 
        }
        print "$state: $match cpu used $value % $message |cpu=$value\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'mem') 
    {
        my $value=  ESIA::SNMP::get($snmp,"$oid_switch_mem_percent.$id");
        my $state="OK";
        my $message=""; 
        if($value < $warning)
        {
            $state="OK";
            $message="< $warning"; 
        }
        elsif($value > $critical)
        {
            $state="CRITICAL";
            $message="> $warning"; 
        }
        elsif($value > $warning)
        {
            $state="WARNING";
            $message="> $warning"; 
        }
        print "$state: $match mem used $value % $message |mem=$value\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
}
else
{
    print "UNKNOWN: $match switch not found. Switch available:\n";
    for(keys %$switchs)
    {
	    print " $_\n";
    }
    $return = $ERRORS{UNKNOWN};
}


exit $return;

