#!/usr/bin/perl -w
############################## check_snmp_fujitsu #################

# Date : Juillet 06 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use lib "/usr/local/esia/plugins/lib/perl"  ;
use strict;
use JSON;
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;

#nom du programme & description
my $PROGNAME = "check_snmp_fujitsu.pl";
my $PROGDESC = "Check fujitsu server by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $warning=80;
my $critical=90;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
#GLOBAL#
my $fcs_model_name ="1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.5";
my $fcs_model_id ="1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.4";
my $fcs_model_manufacturer ="1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.6";
my $fcs_model_serial ="1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.7";
my $fcs_agent_version ="1.3.6.1.4.1.231.2.10.2.2.10.1.3";
my $fcs_motherboard_name ="1.3.6.1.4.1.231.2.10.2.2.10.6.1.1.3";
my $fcs_motherboard_product ="1.3.6.1.4.1.231.2.10.2.2.10.6.1.1.4";
my $fcs_motherboard_serial ="1.3.6.1.4.1.231.2.10.2.2.10.6.1.1.6";
my $fcs_motherboard_revision ="1.3.6.1.4.1.231.2.10.2.2.10.6.1.1.5";
my $fcs_cpu_name ="1.3.6.1.4.1.231.2.10.2.2.10.6.4.1.5";
my $fcs_cpu_vendor ="1.3.6.1.4.1.231.2.10.2.2.10.6.4.1.6";

#FAN
my $fujitsu_fan_status	   				="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.3";
my $fujitsu_fan_CurrentSpeed	        ="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.8"; # measured in revolutions per minute
my $fujitsu_fan_name 					="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.16";
my $fujitsu_fan_text= { 1 => 'unknown',2=>'disabled',3=>'ok',4=>'fail',5=>'prefailure-predicted',6=>'redundant-fan-failed',7=>'not-manageable',8=>'not-present'};
my $fujitsu_fan_text_state = { 1 => 'UNKNOWN',2=>'UNKNOWN',3=>'OK',4=>'CRITICAL',5=>'WARNING',6=>'WARNING',7=>'OK',8=>'OK'};
my $fcs_fan_name    ="1.3.6.1.4.1.231.2.10.2.2.10.5.2.1.3";
my $fcs_fan_state    ="1.3.6.1.4.1.231.2.10.2.2.10.5.2.1.5";
my $fcs_fan_speed    ="1.3.6.1.4.1.231.2.10.2.2.10.5.2.1.6";

####POWER SUPPLY####
my $fujitsu_power_supply_status         ="1.3.6.1.4.1.231.2.10.2.2.5.11.1.1.2.0"; # (GET) 1=unknown, 2=ok, 3=degraded, 4=cri
my $fcs_power_supply_bay            ="1.3.6.1.4.1.231.2.10.2.2.10.6.2.1.2";
my $fcs_power_supply_name           ="1.3.6.1.4.1.231.2.10.2.2.10.6.2.1.3";
my $fcs_power_supply_status         ="1.3.6.1.4.1.231.2.10.2.2.10.6.2.1.5";
my $fcs_supply_text= { 1 => 'unknown',2=>'not-present',3=>'ok',4=>'failed',5=>'ac-fail',6=>'dc-fail',7=>'critical-temperature',8=>'not-manageable',
                        9=>'fan-failure-predicted',10=>'fan-failure',11=>'power-safe-mode',12=>'non-redundant-dc-fail',13=>'non-redundant-ac-fail'};
my $fcs_supply_text_state={ 1 => 'UNKNOWN',2=>'OK',3=>'OK',4=>'CRITICAL',5=>'WARNING',6=>'WARNING',7=>'CRITICAL',8=>'OK',9=>'WARNING',10=>'CRITICAL',11=>'WARNING',12=>'CRITICAL',13=>'CRITICAL'};
####MEMORY####
my $fcs_mem_name           ="1.3.6.1.4.1.231.2.10.2.2.10.6.5.1.3";
my $fcs_mem_status         ="1.3.6.1.4.1.231.2.10.2.2.10.6.5.1.4";
my $fcs_mem_size           ="1.3.6.1.4.1.231.2.10.2.2.10.6.5.1.6";
my $fcs_mem_text= { 1 => 'unknown',2=>'not-present',3=>'ok',4=>'disabled',5=>'error',6=>'failed',7=>'prefailure-predicted',8=>'hot-spare',
                        9=>'mirror',10=>'raid',11=>'hidden'};
my $fcs_mem_text_state={ 1 => 'UNKNOWN',2=>'OK',3=>'OK',4=>'WARNING',5=>'CRITICAL',6=>'CRITICAL',7=>'WARNING',8=>'OK',9=>'OK',10=>'OK',11=>'OK'};

####DISK####
my $fcs_dsk_name           ="1.3.6.1.4.1.231.2.49.1.5.2.1.24";
my $fcs_dsk_status         ="1.3.6.1.4.1.231.2.49.1.5.2.1.15";
my $fcs_dsk_smart_status   ="1.3.6.1.4.1.231.2.49.1.5.2.1.14";
my $fcs_dsk_size           ="1.3.6.1.4.1.231.2.49.1.5.2.1.21";
my $fcs_dsk_model          ="1.3.6.1.4.1.231.2.49.1.5.2.1.5";
my $fcs_dsk_firmware       ="1.3.6.1.4.1.231.2.49.1.5.2.1.16";
my $fcs_dsk_serial         ="1.3.6.1.4.1.231.2.49.1.5.2.1.17";
my $fcs_dsk_vendor         ="1.3.6.1.4.1.231.2.49.1.5.2.1.6";
my $fcs_dsk_text= { 1 => 'unknown',2=>'noDisk',3=>'online',4=>'ready',5=>'failed',6=>'rebuilding',7=>'hotspareGlobal',8=>'hotspareDedicated',
                        9=>'offline',10=>'unconfiguredFailed',11=>'formatting',12=>'dead'};
my $fcs_dsk_text_state={ 1 => 'UNKNOWN',2=>'OK',3=>'OK',4=>'OK',5=>'CRITICAL',6=>'WARNING',7=>'OK',8=>'OK',9=>'CRITICAL',10=>'WARNING',11=>'WARNING',12=>'CRITICAL'};

my $fcs_dsk_smart_text= { 1 => 'ok',2=>'failurePredicted',3=>'smartNotAvailable',4=>'smartMonitoringDisabled'};
my $fcs_dsk_smart_text_state={ 1 => 'OK',2=>'WARNING',3=>'OK',4=>'OK'};
####RAID####
my $fcs_raid_name          ="1.3.6.1.4.1.231.2.49.1.6.2.1.11";
my $fcs_raid_status        ="1.3.6.1.4.1.231.2.49.1.6.2.1.19";
my $fcs_raid_size          ="1.3.6.1.4.1.231.2.49.1.6.2.1.3";
my $fcs_raid_lvl           ="1.3.6.1.4.1.231.2.49.1.6.2.1.6";

my $fcs_raid_text= { 1 => 'unknown',2=>'operational',3=>'partiallyDegraded',4=>'degraded',5=>'failed',6=>'rebuilding',7=>'checking',8=>'mdcing',
                        9=>'initializing',10=>'backgroundInitializing',11=>'migrating',12=>'copying',13=>'offline',14=>'spareInUse',15=>'erasing'};
my $fcs_raid_text_state={ 1 => 'UNKNOWN',2=>'OK',3=>'WARNING',4=>'CRITICAL',5=>'CRITICAL',6=>'WARNING',7=>'OK',8=>'OK',9=>'OK',10=>'OK',11=>'OK',12=>'OK',13=>'CRITICAL',14=>'WARNING',15=>'CRITICAL'};
####CNTRL####
my $fcs_cntrl_model         ="1.3.6.1.4.1.231.2.49.1.4.2.1.2";
my $fcs_cntrl_dsk_number    ="1.3.6.1.4.1.231.2.49.1.4.2.1.11";
my $fcs_cntrl_status        ="1.3.6.1.4.1.231.2.49.1.4.2.1.15";
my $fcs_cntrl_firmware      ="1.3.6.1.4.1.231.2.49.1.4.2.1.7";
my $fcs_cntrl_manufacturer  ="1.3.6.1.4.1.231.2.49.1.4.2.1.3";
my $fcs_cntrl_serial        ="1.3.6.1.4.1.231.2.49.1.4.2.1.21";
my $fcs_cntrl_hardwarevers  ="1.3.6.1.4.1.231.2.49.1.4.2.1.6";

my $fcs_cntrl_text= { 1 => 'ok',2=>'prefailure',3=>'failure'};
my $fcs_cntrl_text_state={ 1 => 'OK',2=>'WARNING',3=>'CRITICAL'};

####TEMPERATURE####
my $fujitsu_temp_name                   ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.4";
my $fujitsu_status_temperature          ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.5";
my $fujitsu_current_temperature         ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.6"; # in degrees Celsius
my $fujitsu_temp_text= { 1 => 'unknown',2=>'not-available',3=>'ok',4=>'sensor-failed',5=>'failed',6=>'warning-toohot',7=>'critical-toohot',
				8=>'normal',9=>'warning'};
my $fujitsu_temp_text_state = { 1 => 'UNKNOWN',2=>'UNKNOWN',3=>'OK',4=>'CRITICAL',5=>'WARNING',6=>'WARNING',7=>'CRITICAL',8=>'OK',9=>'WARNING'};


my $return=$ERRORS{OK};
my $type='health';
my ($opt_h,$opt_V, $json);
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"T=s" => \$type,	 "type=s" => \$type,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
		"J" => \$json,	 "json" => \$json,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'');
	my @arr;
	push(@arr,["-T","--Type","type of test [health|raid|cntrl|disk|temp|inventory]. Default value 'health'" ]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $fujitsu_descr = "1.3.6.1.2.1.1.1.0";
########## MAIN ##########
if( $type eq 'health' )
{
	my $result = $snmp->get_table(Baseoid => $fujitsu_fan_status);
	if(defined($result))
	{
		foreach my $key ( keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.16.(.*)/ )
			{
				my $id =$1;
				my $fan_status = $$result{$key};
				my $fan_name =  ESIA::SNMP::get($snmp,"$fujitsu_fan_name.$id");
				my $fan_CurrentSpeed =  ESIA::SNMP::get($snmp,"$fujitsu_fan_CurrentSpeed.$id");
                my $state = 'OK';
                $state = $fujitsu_fan_text_state->{ $fan_status };
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				print "$state : FAN $fan_name (speed: $fan_CurrentSpeed rpm) is ".$fujitsu_fan_text->{ $fan_status }." state\n";
            }
        }
        my $state = 'OK';
        my $message = "OK state";
        my $status_power_supply = ESIA::SNMP::get($fujitsu_power_supply_status);
        if($status_power_supply==2)
        {
            $state = "OK";
        }
        elsif($status_power_supply==3)
        {
            $state = "WARNING";
            $message = "Degraded state";
        }
        elsif($status_power_supply==4)
        {
            $state = "CRITICAL";
            $message = "error state";
        }
        else
        {
            $state = "UNKNOWN";
            $message = "unknown state";
        }
        print "$state : Power supply is $message";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    else
    {
	    my $result = $snmp->get_table(Baseoid => $fcs_power_supply_status  );
	    if(defined($result))
	    {
		    foreach my $key ( sort keys %$result)
		    {
			    if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.6.2.1.5.([0-9\.]*)/ )
			    {
                    my $id =$1;
				    my $status = $$result{$key};
				    my $name =  ESIA::SNMP::get($snmp,"$fcs_power_supply_name.$id");
                    my $bay =  ESIA::SNMP::get($snmp,"$fcs_power_supply_bay.$id");
                    my $state = 'OK';
                    if($status > 13)
                    {
                        $state = 'UNKNOWN';
                        $fcs_supply_text->{ $status }="unknown value contact ESIA";
                    }
                    else
                    {
                        $state = $fcs_supply_text_state->{ $status };
                    }
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				    print "$state : Power supply $name (bay: $bay) is ".$fcs_supply_text->{ $status }." state\n";

                }
            }
            $result = $snmp->get_table(Baseoid => $fcs_fan_state  );
		    foreach my $key (sort keys %$result)
		    {
			    if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.5.2.1.5.(.*)/ )
			    {
				    my $id =$1;
				    my $fan_status = $$result{$key};
				    my $fan_name =  ESIA::SNMP::get($snmp,"$fcs_fan_name.$id");
				    my $fan_CurrentSpeed =  ESIA::SNMP::get($snmp,"$fcs_fan_speed.$id");
                    my $state = 'OK';
                    $state = $fujitsu_fan_text_state->{ $fan_status };
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				    print "$state : Fan $fan_name (speed: $fan_CurrentSpeed rpm) is ".$fujitsu_fan_text->{ $fan_status }." state\n";
                }
            }

            $result = $snmp->get_table(Baseoid => $fcs_mem_status  );
		    foreach my $key ( sort keys %$result)
		    {
			    if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.6.5.1.4.([0-9\.]*)/ )
			    {
                    my $id =$1;
				    my $status = $$result{$key};
				    my $name =  ESIA::SNMP::get($snmp,"$fcs_mem_name.$id");
                    my $size =  ESIA::SNMP::get($snmp,"$fcs_mem_size.$id");
                    $size = ($size/1024).'GB';
                    my $state = 'OK';
                    if($status > 11)
                    {
                        $state = 'UNKNOWN';
                        $fcs_mem_text->{ $status }="unknown value contact ESIA";
                    }
                    else
                    {
                        $state = $fcs_mem_text_state->{ $status };
                    }
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				    print "$state : RAM $name (size: $size) is ".$fcs_mem_text->{ $status }." state\n";

                }
            }
        }
        else
        {
            print "UNKNOWN: Fujitsu server contact ESIA support\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{'UNKNOWN'});
        }
    }
}
elsif( $type eq 'raid' )
{
    my $result = $snmp->get_table(Baseoid => $fcs_raid_status  );
	if(defined($result))
	{
        foreach my $key ( sort keys %$result)
		{
		    if($key=~ /1.3.6.1.4.1.231.2.49.1.6.2.1.19.([0-9\.]*)/ )
		    {
                my $id =$1;
                my $status = $$result{$key};
                my $name   =  ESIA::SNMP::get($snmp,"$fcs_raid_name.$id");
                my $level  =  ESIA::SNMP::get($snmp,"$fcs_raid_lvl.$id");
                my $size   =  ESIA::SNMP::get($snmp,"$fcs_raid_size.$id");
                my $state = 'OK';
                if($status > 15)
                {
                    $state = 'UNKNOWN';
                    $fcs_raid_text_state->{ $status }="unknown value contact ESIA";
                }
                else
                {
                    $state = $fcs_raid_text_state->{ $status };
                }

                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                print "$state : raid $name (size: $size GB, $level) is ".$fcs_raid_text->{ $status }."\n";

            }
        }
    }
    else
	{
		print ("UNKNOWN: SNMP Error raid table is missing");
		$return = $ERRORS{'UNKNOWN'};
	}
}
elsif( $type eq 'cntrl' )
{
    my $result = $snmp->get_table(Baseoid => $fcs_cntrl_status  );
	if(defined($result))
	{
        foreach my $key ( sort keys %$result)
		{
		    if($key=~ /1.3.6.1.4.1.231.2.49.1.4.2.1.15.([0-9\.]*)/ )
		    {
                my $id =$1;
                my $status = $$result{$key};
                my $manufacturer =  ESIA::SNMP::get($snmp,"$fcs_cntrl_manufacturer.$id");
                my $model   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_model.$id");
                my $dsknumber   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_dsk_number.$id");
                my $firmware   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_firmware.$id");
                my $serial   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_serial.$id");
                my $state = 'OK';
                if($status > 3)
                {
                    $state = 'UNKNOWN';
                    $fcs_cntrl_text->{ $status }="unknown value contact ESIA";
                }
                else
                {
                    $state = $fcs_cntrl_text_state->{ $status };
                }
		        my $name = $manufacturer.' '.$model;
		        $name=~ s/^\s+|\s+$//g;
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                print "$state : controler $name (firmware: $firmware, serial: $serial, disk: $dsknumber conntected) is ".$fcs_cntrl_text->{ $status }."\n";

            }
        }
    }
    else
	{
		print ("UNKNOWN: SNMP Error controller table is missing");
		$return = $ERRORS{'UNKNOWN'};
	}
}
elsif( $type eq 'disk' )
{
    my $result = $snmp->get_table(Baseoid => $fcs_dsk_status  );
	if(defined($result))
	{
        foreach my $key ( sort keys %$result)
		{
		    if($key=~ /1.3.6.1.4.1.231.2.49.1.5.2.1.15.([0-9\.]*)/ )
		    {
                my $id =$1;
                my $status = $$result{$key};
                my $name   =  ESIA::SNMP::get($snmp,"$fcs_dsk_name.$id");
                #my $model  =  ESIA::SNMP::get($snmp,"$fcs_dsk_model.$id");
                my $sn     =  ESIA::SNMP::get($snmp,"$fcs_dsk_serial.$id");
                #my $vendor =  ESIA::SNMP::get($snmp,"$fcs_dsk_vendor.$id");
                my $size   =  ESIA::SNMP::get($snmp,"$fcs_dsk_size.$id");
                my $smart   =  ESIA::SNMP::get($snmp,"$fcs_dsk_smart_status.$id");
                my $state = 'OK';
                $size = sprintf("%.0f",$size/1024);
                if($status > 12)
                {
                    $state = 'UNKNOWN';
                    $fcs_dsk_text_state->{ $status }="unknown value contact ESIA";
                }
                else
                {
                    $state = $fcs_dsk_text_state->{ $status };
                }
                if($smart > 4)
                {
                    $state = 'UNKNOWN';
                    $fcs_dsk_smart_text_state->{ $smart }="unknown value contact ESIA";
                }
                else
                {
                    $state = $fcs_dsk_smart_text_state->{ $smart };
                }
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                print "$state : disk $name (size: $size GB, serial: $sn) is ".$fcs_dsk_text->{ $status }." state, SMART is ".$fcs_dsk_smart_text->{ $smart }."\n";
            }
        }
    }
    else
	{
		print ("UNKNOWN: SNMP Error disk table is missing");
		$return = $ERRORS{'UNKNOWN'};
	}
}
elsif( $type eq 'temp' )
{
	my $result = $snmp->get_table(Baseoid => $fujitsu_status_temperature);
	if(defined($result))
	{
		my $ambientTemp=0;
		foreach my $key ( sort keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.5.(.*)/ )
			{
				my $id =$1;
				my $status = $$result{$key};
				#Si une températuree est disponnible pour le capteur
				if($status !=2)
				{
					my $name    =  ESIA::SNMP::get($snmp,"$fujitsu_temp_name.$id");
					my $current =  ESIA::SNMP::get($snmp,"$fujitsu_current_temperature.$id");
					if($name eq 'Ambient' || $name eq 'ambient')
					{
						$ambientTemp=$current;
					}
		            my $state = 'OK';
		            if($status > 9)
		            {
		                $state = 'UNKNOWN';
		                $fujitsu_temp_text->{ $status }="unknown value contact ESIA";
		            }
		            else
		            {
		                $state = $fujitsu_temp_text_state->{ $status };
		            }
		            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		            print "$state : temperature of $name is ".$fujitsu_temp_text->{ $status }." state( $current Celsius)\n";
				}
			}
		}
		print "|tempAmb=$ambientTemp";
	}
	else
	{
		print ("UNKNOWN: SNMP Error temperature table is missing");
		$return = $ERRORS{'UNKNOWN'};
	}
}
elsif( $type eq 'inventory' )
{
	my @output;
    my $result = $snmp->get_table(Baseoid => $fcs_model_id  );
	if(defined($result))
	{
		my $version   =  ESIA::SNMP::get($snmp,"$fcs_agent_version.0");
        foreach my $key (keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.2.3.1.4.([0-9\.]*)/ )
		    {
		    	my $id = $1;
				my $model = $$result{$key};
				my $name =  ESIA::SNMP::get($snmp,"$fcs_model_name.$id");
				my $manufacturer =  ESIA::SNMP::get($snmp,"$fcs_model_manufacturer.$id");
				my $serial =  ESIA::SNMP::get($snmp,"$fcs_model_serial.$id");
				my %hash = (
				'name' => $name,
				'description'=>'',
				'manufacturer'=>$manufacturer,
				'model' => "$model",
				'type'=>'CHASSIS',
				'serial' => $serial,
				'hardwareVersion' => '',
				'firmwareVersion' => $version,
				);
				push(@output, \%hash);
			}
		}
		$result = $snmp->get_table(Baseoid => $fcs_motherboard_name  );
		if(defined($result))
		{
		    foreach my $key ( sort keys %$result)
			{
				if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.6.1.1.3.([0-9\.]*)/ )
				{
		            my $id =$1;
		            my $name = $$result{$key};
		            my $model  =  ESIA::SNMP::get($snmp,"$fcs_motherboard_product.$id");
		            my $serial =  ESIA::SNMP::get($snmp,"$fcs_motherboard_serial.$id");
		            my $vendor =  'FUJITSU';
		            my $revision =  ESIA::SNMP::get($snmp,"$fcs_motherboard_revision.$id");
					my %hash = (
						'name' => $name,
						'description'=>"",
						'manufacturer'=>$vendor,
						'model' => $model,
						'type'=>'MOTHERBOARD',
						'serial' => $serial,
						'hardwareVersion' => $revision,
						'firmwareVersion' => '',
					);
					push(@output, \%hash);
				}
			}
		}
		$result = $snmp->get_table(Baseoid => $fcs_cpu_name  );
		if(defined($result))
		{
		    foreach my $key ( sort keys %$result)
			{
				if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.6.4.1.5.([0-9\.]*)/ )
				{
		            my $id =$1;
		            my $name = $$result{$key};
		            my $vendor  =  ESIA::SNMP::get($snmp,"$fcs_cpu_vendor.$id");
					my %hash = (
						'name' => $name,
						'description'=>"",
						'manufacturer'=>$vendor,
						'model' => $name,
						'type'=>'CPU',
						'serial' => '',
						'hardwareVersion' => '',
						'firmwareVersion' => '',
					);
					push(@output, \%hash);
				}
			}
		}
		$result = $snmp->get_table(Baseoid => $fcs_dsk_name  );
		if(defined($result))
		{
		    foreach my $key ( sort keys %$result)
			{
				if($key=~ /1.3.6.1.4.1.231.2.49.1.5.2.1.24.([0-9\.]*)/ )
				{
		            my $id =$1;
		            my $name = $$result{$key};
		            my $model  =  ESIA::SNMP::get($snmp,"$fcs_dsk_model.$id");
		            my $serial =  ESIA::SNMP::get($snmp,"$fcs_dsk_serial.$id");
		            my $vendor =  ESIA::SNMP::get($snmp,"$fcs_dsk_vendor.$id");
		            my $size   =  ESIA::SNMP::get($snmp,"$fcs_dsk_size.$id");
		            my $firmware =  ESIA::SNMP::get($snmp,"$fcs_dsk_firmware.$id");
		            $size = sprintf("%.0f",$size/1024);
					my %hash = (
						'name' => $name,
						'description'=>"Size:$size GB",
						'manufacturer'=>$vendor,
						'model' => $model,
						'type'=>'DISK',
						'serial' => $serial,
						'hardwareVersion' => '',
						'firmwareVersion' => $firmware,
					);
					push(@output, \%hash);
				}
			}
		}
		$result = $snmp->get_table(Baseoid => $fcs_cntrl_manufacturer  );
		if(defined($result))
		{
        	foreach my $key ( sort keys %$result)
			{
		    	if($key=~ /1.3.6.1.4.1.231.2.49.1.4.2.1.3.([0-9\.]*)/ )
				{
		            my $id =$1;
		            my $vendor =  $$result{$key};
		            my $model   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_model.$id");
		            my $firmware   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_firmware.$id");
		            my $serial   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_serial.$id");
		            my $hard   =  ESIA::SNMP::get($snmp,"$fcs_cntrl_hardwarevers.$id");
		            my $name = $vendor.' '.$model;
		            $name=~ s/^\s+|\s+$//g;
		      		my %hash = (
						'name' => $name,
						'description'=>"",
						'manufacturer'=>$vendor,
						'model' => $model,
						'type'=>'CONTROLLER',
						'serial' => $serial,
						'hardwareVersion' => $hard,
						'firmwareVersion' => $firmware,
					);
					push(@output, \%hash);
				}
			}
		}
	}

	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
		my $size = @output;
		if($size != 0)
		{
			foreach my $line ( @output)
			{

				my $hash = \%$line;
				print "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
			}
			exit(0);
		}
	}

}
else
{
	print ("UNKNOWN: Incorrect or missed option -T, --type\n");
	$return = $ERRORS{'UNKNOWN'};
}
exit $return;
