#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
my $PROGNAME='check_snmp_hp_switch.pl';
my $PROGDESC = "Check HP, Aruba switch CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;


my $return=0;
my ($opt_h,$opt_V);
my $warning='';
my $critical='';
my $type = 'env';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
#procurve switchs
my $oid_procurve_operating_descr    	= "1.3.6.1.4.1.11.2.14.11.1.2.6.1.7"; 	# Component description
my $oid_procurve_operating_state    	= "1.3.6.1.4.1.11.2.14.11.1.2.6.1.4"; 	# Operating status of component - Unknown(1), Bad(2), Warning(3), Good(4), NotPresent(5)
my %procurve_operating_status_text  = ( "0"  => "--Invalid--","1" => "Unknown","2"  => "Bad","3"  => "Warning","4"  => "Good","5"  => "NotPresent");
my %procurve_operating_status = ( "0"  => "UNKNOWN","1"  => "UNKNOWN","2" => "WARNING","3"  => "CRITICAL","4"  => "OK","5"  => "OK");
#CPU
my $oid_procurve_cpu = "1.3.6.1.4.1.11.2.14.11.5.1.9.6.1.0"; # Procurve CPU Counter
# MEM
my $oid_procurve_mem_index	    = "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.1"; # memory slot index
my $oid_procurve_mem_total	    = "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.5"; # Total Bytes
my $oid_procurve_mem_free		= "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.6"; # Free Bytes
my $oid_procurve_mem_free_seg	= "1.3.6.1.4.1.11.2.14.11.5.1.1.2.2.1.1.3"; # Free segments
#ARUBA
my $oid_hpSystemAirName = "1.3.6.1.4.1.11.2.14.11.1.2.8.1.1.2";
my $oid_hpCurrentTemp = "1.3.6.1.4.1.11.2.14.11.1.2.8.1.1.3";
my $oid_hpThresholdTemp = "1.3.6.1.4.1.11.2.14.11.1.2.8.1.1.7";
my $oid_aruba_cpu = "1.3.6.1.2.1.25.3.3.1.2";
my $oid_aruba_storage_descr = '1.3.6.1.2.1.25.2.3.1.3';
my $oid_aruba_storage_size = '1.3.6.1.2.1.25.2.3.1.5';
my $oid_aruba_storage_used = '1.3.6.1.2.1.25.2.3.1.6';
my $oid_aruba_storage_alloc_units= '1.3.6.1.2.1.25.2.3.1.4';

my $oid_aruba_sensors_type= '1.3.6.1.2.1.99.1.1.1.1';
my $oid_aruba_sensors_state= '1.3.6.1.2.1.99.1.1.1.5';
my %oid_aruba_sensors_state_text  = ( "0"  => "Unknown","1" => "OK","2"  => "unavailable","3"  => "nonoperational");
my %oid_aruba_sensors_state_status = ( "0"  => "UNKNOWN","1"  => "OK","2" => "OK","3"  => "CRITICAL","4"  => "CRITICAL");

my $oid_aruba_sensors_val = '1.3.6.1.2.1.99.1.1.1.4';
my %oid_aruba_sensors_type_text  = ( "1"  => "other","2" => "unknown","3"  => "voltsAC","4"  => "voltsDC",
									 "5"  => "amperes","6"  => "watts","7" => "hertz","8"  => "celsius",
									 "9"  => "percentRH","10"  => "rpm","11" => "cmm","12"  => "truthvalue");
#HPE 5130
my $oid_sensorType = "1.3.6.1.2.1.99.1.1.1.1";	#8 Celcius
my $oid_sensorValue = "1.3.6.1.2.1.99.1.1.1.4";	#Value
my $oid_sensorUnit = "1.3.6.1.2.1.99.1.1.1.2"; 	#9 Unit
my $oid_hpe_sensorName = "1.3.6.1.2.1.47.1.1.1.1.7";

#Sensors
my $oid_hpe_sensorStatus="1.3.6.1.4.1.25506.2.6.1.1.1.1.3";
my %hpe_sensorStatus_text  = ( "0"  => "--Invalid--","1" => "notSupported","2"  => "disabled","3"  => "enabled","4"  => "dangerous");
my %hpe_sensorStatus_status = ( "0"  => "UNKNOWN","1"  => "UNKNOWN","2" => "OK","3"  => "OK","4"  => "CRITICAL");
my $oid_hpe_sensorTemperature="1.3.6.1.4.1.25506.2.6.1.1.1.1.12";
my $oid_hpe_sensorTemperatureThreshold="1.3.6.1.4.1.25506.2.6.1.1.1.1.13";
my $oid_hpe_sensorTemperatureLowerThreshold="1.3.6.1.4.1.25506.2.6.1.1.1.1.21";
#CPU&mem
my $oid_hpe_cpu_usage = "1.3.6.1.4.1.25506.2.6.1.1.1.1.6";
my $oid_hpe_mem_usage = "1.3.6.1.4.1.25506.2.6.1.1.1.1.8";

#Power
my $oid_aruba_power_state = ".1.3.6.1.4.1.11.2.14.11.5.1.55.1.1.1.2";
my $oid_aruba_power_temp = ".1.3.6.1.4.1.11.2.14.11.5.1.55.1.1.1.4";
my $oid_aruba_power_model= ".1.3.6.1.4.1.11.2.14.11.5.1.55.1.1.1.9";

my %oid_aruba_power_state_text  = ( "1"  => "psNotPresent","2" => "psNotPlugged","3"  => "psPowered","4"  => "psFailed","5"  => "psPermFailure","6"  => "psMax");
my %oid_aruba_power_state_status = ( "1"  => "OK","2" => "UNKNOWN","3"  => "OK","4"  => "CRITICAL","5"=>"WARNING","6"=>"WARNING");

#fan
my $oid_aruba_fan_state = ".1.3.6.1.4.1.11.2.14.11.5.1.54.2.1.1.4";
my $oid_aruba_fan_tray  = ".1.3.6.1.4.1.11.2.14.11.5.1.54.2.1.1.2";
my %oid_aruba_fan_state_text  = ( "0"=>"failed", "1"  => "removed","2" => "off","3"  => "underspeed","4"  => "overspeed","5"  => "ok","6"  => "maxstate");
my %oid_aruba_fan_state_status = ( "0"=>"CRITICAL","1"  => "UNKNOWN","2" => "UNKNOWN","3"  => "WARNING","4"  => "WARNING","5"=>"OK","6"=>"WARNING");

#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,env] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
}
#
#HP procurve switch
#
if($Sysoid_value == 11)
{
    if($type eq 'env')
    {
        my $result = $snmp->get_table(Baseoid => $oid_procurve_operating_descr);
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                my $status = ESIA::SNMP::get($snmp,"$oid_procurve_operating_state.$index");
                my $state = $procurve_operating_status{$status};
                print "$state : $descr is ".$procurve_operating_status_text{$status}."\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
        $result = $snmp->get_table(Baseoid => $oid_hpSystemAirName);
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                my $temps = ESIA::SNMP::get($snmp,"$oid_hpCurrentTemp.$index");
                $temps =~s/C//g;
                my $thresh = ESIA::SNMP::get($snmp,"$oid_hpThresholdTemp.$index");
                $thresh =~s/C//g;
                my $state='OK';
                my $msg = "< $thresh C";
                if( $temps >= $thresh)
                {
                     $state='CRITICAL';
                    $msg = "> $thresh C";
                }
                print "$state : $descr temperature is $temps $msg\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
        $result = $snmp->get_table(Baseoid => $oid_aruba_power_state);
        foreach my $key (sort keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $powerstate =$$result{"$key"};
                my $temps = ESIA::SNMP::get($snmp,"$oid_aruba_power_temp.$index");
                my $model = ESIA::SNMP::get($snmp,"$oid_aruba_power_model.$index");
                my $state = $oid_aruba_power_state_status{$powerstate};
                print "$state : power id $index ($model) is ".$oid_aruba_power_state_text{$powerstate}.", temperature is $temps degree\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
        $result = $snmp->get_table(Baseoid => $oid_aruba_fan_state);
        foreach my $key (sort keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $powerstate =$$result{"$key"};
                my $tray = ESIA::SNMP::get($snmp,"$oid_aruba_fan_tray.$index");
                #my $model = ESIA::SNMP::get($snmp,"$oid_aruba_power_model.$index");
                my $state = $oid_aruba_fan_state_status{$powerstate};
                print "$state : fan id $index on tray $tray is ".$oid_aruba_fan_state_text{$powerstate}."\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
    }
    elsif($type eq 'cpu')
    {
        my $state = 'OK';
        my $threshold = ESIA::SNMP::get($snmp,"$oid_procurve_cpu");
        my $message="(< $warning)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        printf("%s : CPU used %.1f%% %s",$state,$threshold,$message);
    }
    elsif($type eq 'mem')
    {
       my $result = $snmp->get_table(Baseoid => $oid_procurve_mem_total);
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $total =$$result{"$key"};
		        my $memory = ESIA::SNMP::get($snmp,"$oid_procurve_mem_free.$index");
                my $state = 'OK';
                my $threshold = 100 - ($memory / $total)*100;
                my $message="(< $warning)";
                if($threshold >= $critical)
                {
                    $state='CRITICAL';
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
                    $message="(> $critical)";
                }
                elsif($threshold >= $warning)
                {
                    $state='WARNING';
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
                    $message="(> $warning)";
                }
                printf("%s :Slot %s:%.0f%% %s",$state,$index,$threshold,$message);
               # print "$state : Slot $index: is $threshold\n";
               # $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }

    }
}
#
#HPE 5130 & co
#
elsif($Sysoid_value == 25506)
{
    if($type eq 'env')
    {
        my $nbr_sensors=0;
        my $result = $snmp->get_table(Baseoid => $oid_hpe_sensorName);
        foreach my $key ( keys %$result)
        {
            my $state = 'OK';
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                if($descr =~ /FAN/  ||$descr =~ /PSU/)
                {

		            my $status = ESIA::SNMP::get($snmp,"$oid_hpe_sensorStatus.$index");
                    $state = $hpe_sensorStatus_status{$status};
                    print "$state : $descr is ".$hpe_sensorStatus_text{$status}."\n";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
                elsif($descr =~ /SENSOR/)
                {
                    $nbr_sensors++;
                    my $temp = ESIA::SNMP::get($snmp,"$oid_hpe_sensorTemperature.$index");
                    my $tempthreshold = ESIA::SNMP::get($snmp,"$oid_hpe_sensorTemperatureThreshold.$index");
                    my $templower =  ESIA::SNMP::get($snmp,"$oid_hpe_sensorTemperatureLowerThreshold.$index");
                    my $message="";
                    if($temp >= $tempthreshold)
                    {
                        $state="CRITICAL";
                        $message="high temperature $temp > $tempthreshold";
                    }
                    elsif($temp <= $templower)
                    {
                        $state="CRITICAL";
                        $message="lower temperature $temp < $templower";
                    }
                    else
                    {
                        $state="OK";
                        $message="normal $temp < $tempthreshold";
                    }
                    print "$state : $descr is $message\n";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
            }
        }
        if($nbr_sensors == 0)
        {
            print "OK : No sensors found\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
        }
    }
    elsif($type eq 'cpu')
    {
        my $result = $snmp->get_table(Baseoid => $oid_hpe_sensorName);
        my $cpuusage =-1;
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                if($descr =~ /Board/ || $descr =~ /Level 1 Virtual Module/)
                {
                    $cpuusage = ESIA::SNMP::get($snmp,"$oid_hpe_cpu_usage.$index");
                    last;
                }
            }
        }
        if($cpuusage != -1 )
        {
            my $state = "OK";
            my $message="(< $warning)";
            if($cpuusage >= $critical)
            {
                $state='CRITICAL';
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
                $message="(> $critical)";
            }
            elsif($cpuusage >= $warning)
            {
                $state='WARNING';
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
                $message="(> $warning)";
            }
            printf("%s : CPU used %.1f%% %s",$state,$cpuusage,$message);
        }
        else
        {
            print "UNKNWON: CPU not found";
            $snmp->close;
            exit $ERRORS{"UNKNOWN"};
        }
    }
    elsif($type eq 'mem')
    {
        my $result = $snmp->get_table(Baseoid => $oid_hpe_sensorName);
        my $memusage =-1;
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
                if($descr =~ /Board/ || $descr =~ /Level 1 Virtual Module/)
                {
                    $memusage = ESIA::SNMP::get($snmp,"$oid_hpe_mem_usage.$index");
                    last;
                }
            }
        }
        if($memusage != -1 )
        {
            my $state = "OK";
            my $message="(< $warning)";
            if($memusage >= $critical)
            {
                $state='CRITICAL';
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
                $message="(> $critical)";
            }
            elsif($memusage >= $warning)
            {
                $state='WARNING';
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
                $message="(> $warning)";
            }
            printf("%s :Slot %s:%.0f%% %s",$state,1,$memusage,$message);
        }
        else
        {
            print "UNKNWON: CPU not found";
            $snmp->close;
            exit $ERRORS{"UNKNOWN"};
        }
    }
}
elsif($Sysoid_value == 47196)
{
    if($type eq 'cpu')
    {
		my $result = $snmp->get_table( $oid_aruba_cpu);
		my $load=0;
		my $nbrcpu = 0;
		foreach my $key ( keys %$result)
		{
			$load=$load + $$result{$key};
			$nbrcpu++;
		}
		if($nbrcpu == 0)
		{
			print "UNKNOWN: No CPU information found\n";
			exit $ERRORS{'UNKNOWN'};
		}
		else
		{
			$load = $load / $nbrcpu;
			$load = sprintf("%.1f",$load);
			my $state = "OK";
			my $message = "< $warning%";
			if($load < $warning)
			{
				$state = "OK";
			}
			elsif($load < $critical)
			{
				$state = "WARNING";
				$message = "> $warning%";
			}
			else
			{
				$state = "CRITICAL";
				$message = "> $critical%";
			}
			printf("%s : CPU used %.1f%% %s",$state,$load,$message);
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
	}
	elsif($type eq 'mem')
    {
        my $result = $snmp->get_table(Baseoid => $oid_aruba_storage_descr);
        my $descr ='';
        my $index ='';
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                $index =$1;
                if($$result{"$key"} =~ /Physical memory/)
                {
                	$descr =$$result{"$key"};
                    last;
                }
            }
        }
        if($descr ne '' )
        {
            my $used = ESIA::SNMP::get($snmp,"$oid_aruba_storage_used.$index");
            my $size = ESIA::SNMP::get($snmp,"$oid_aruba_storage_size.$index");
            my $unit = ESIA::SNMP::get($snmp,"$oid_aruba_storage_alloc_units.$index");
            my $percent_used = ($used/$size)*100;
            $percent_used = sprintf("%.0f",$percent_used);
            my $used_byte = $used * $unit;
            my $size_byte = $size * $unit;
            my $used_MB =sprintf("%.0f",$used_byte/1024/1024);
            my $size_MB =sprintf("%.0f",$size_byte/1024/1024);
		    my $state = "OK";
		    my $message = "< $warning%";
		    if($percent_used < $warning)
		    {
			    $state = "OK";
		    }
		    elsif($percent_used < $critical)
		    {
			    $state = "WARNING";
			    $message = "> $warning%";
		    }
		    else
		    {
			    $state = "CRITICAL";
			    $message = "> $critical%";
		    }
            printf("%s :Slot %s:%.0f%% %s",$state,$descr,$percent_used,$message);
        }
        else
        {
            print "UNKNWON: CPU not found";
            $snmp->close;
            exit $ERRORS{"UNKNOWN"};
        }
    }
    elsif($type eq 'env')
    {
        my $nbr_sensors=0;
        my $result = $snmp->get_table(Baseoid => $oid_aruba_sensors_type);
        foreach my $key ( sort keys %$result)
        {
            my $state = 'OK';
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
	        	$nbr_sensors++;
                my $index =$1;
		        my $type =$$result{"$key"};
		        my $descr =  ESIA::SNMP::get($snmp,"$oid_hpe_sensorName.$index");
				#my $value = ESIA::SNMP::get($snmp,"$oid_aruba_sensors_val.$index");
				my $status = ESIA::SNMP::get($snmp,"$oid_aruba_sensors_state.$index");
				$state =$oid_aruba_sensors_state_status{$status};

				print "$state: $descr is ".$oid_aruba_sensors_state_text{$status}." state.\n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
        if($nbr_sensors == 0)
        {
            print "OK : No sensors found\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
        }
    }
}
else
{
    print "OID is unknown. Model not support contact your support\@esia-sa.com\n";
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
exit($return);
