#!/usr/bin/perl -w
############################## check_snmp_huawei.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;
use JSON;

my $PROGNAME='check_snmp_ubiquity_switch.pl';
my $PROGDESC = "Check Ubiquity switch CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my %ENTITY = %ESIA::INVENTORY::ENTITY_PHYSICAL_CLASS;

my $return=0;
my ($opt_h,$opt_V,$json);
my $warning=80;
my $critical=90;
my $type = 'cpu';
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
my $oid_proc_load = "1.3.6.1.2.1.25.3.3.1.2";
my $oid_phy_type = "1.3.6.1.2.1.47.1.1.1.1.5";
my $oid_phy_serial= "1.3.6.1.2.1.47.1.1.1.1.11";
my $oid_phy_name= "1.3.6.1.2.1.47.1.1.1.1.7";
my $oid_phy_firmware_version= "1.3.6.1.2.1.47.1.1.1.1.9";
my $oid_phy_manufacturer = "1.3.6.1.2.1.47.1.1.1.1.12";
my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"J" => \$json,"--json" => \$json,
);
# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,env] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
if($type eq 'cpu')
{
    my $result = $snmp->get_table( $oid_proc_load);
	my $load=0;
	my $nbrcpu = 0;
	foreach my $key ( keys %$result)
	{
		$load=$load + $$result{$key};
		$nbrcpu++;
	}
	if($nbrcpu == 0)
	{
		print "UNKNOWN: No CPU information found\n";
		exit $ERRORS{'UNKNOWN'};
	}
	else
	{
		$load = $load / $nbrcpu;
		$load = sprintf("%.1f",$load);
		my $state = "OK";
		my $message = "< $warning%";
		if($load < $warning)
		{
			$state = "OK";
		}
		elsif($load < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}
		printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$load,$message,$load);
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
}
elsif($type eq 'inventory')
{
    my @array;

    my $sysdescr =  ESIA::SNMP::get($snmp,$oid_sysDescr);
    my $result = $snmp->get_table( $oid_phy_type);
	foreach my $key ( keys %$result)
	{
        my $type = $ENTITY{$$result{"$key"}};
        if($key=~ /([0-9]*)$/ )
		{
            my $index = $1;
            my $name = ESIA::SNMP::get($snmp,"$oid_phy_name.$index");
            my $serial = ESIA::SNMP::get($snmp,"$oid_phy_serial.$index");
            my $manufacturer = ESIA::SNMP::get($snmp,"$oid_phy_manufacturer.$index");
            my $firmware = ESIA::SNMP::get($snmp,"$oid_phy_firmware_version.$index");
            my $model ='';
            if($type eq 'CHASSIS')
            {
                $model = $name;
                $name = $manufacturer.' '.$name;
		        if($sysdescr=~/Version\s+([0-9]*\.[0-9]*)/)
		        {
			        $firmware=$1;
		        }
            }

	        my %hash = ('name' => "$name",'manufacturer'=>$manufacturer, 'model' =>$model , 'type'=>$type,'serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' =>$firmware);
	        push(@array, \%hash);

        }
	}

	my $output;
	if($json)
	{
	   $output = encode_json(\@array);
	}
	else
	{
        foreach my $line(@array)
        {
		    my $hash = \%$line;
		    $output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
		    $output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
        }
    }
	print $output;
}
exit($return);
