#!/usr/bin/perl -w
############################## check_snmp_sonicwall.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;


my $PROGNAME='check_snmp_proges.pl';
my $PROGDESC = "Check Proges Plug & track sensors by SNMP";

my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my $return=0;
my ($opt_h,$opt_V,$json);
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
my $oid_name =".1.3.6.1.4.1.31440.10.3.1.3";
my $oid_type =".1.3.6.1.4.1.31440.10.3.1.2";
my $oid_ds18b20 =".1.3.6.1.4.1.31440.10.5.1.1";

my $warning = 80;
my $critical = 90;
my $match = "all";

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP

my ($snmp, $snmperror) = ESIA::SNMP::Session(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"m=s" => \$match,"match=s" => \$match,
	"w=s" => \$warning,"warning=s" => \$warning,
	"c=s" => \$critical,"critical=s" => \$critical,
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-m","--match","select sensors (default: $match)" ]);
	push(@arr,["-w","--warning","warning threshold (default: $warning) in celsius" ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical) in celsius"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
if($match eq 'all')
{
    $match='.*';
}
#si l'host ne répond pas on blocke le script
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if ($Sysoid_value ==-1)
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
my $result = $snmp->get_table( $oid_name);
my $oid_sensors = "";
my $numbersensors=0;
my $total=0;
foreach my $key ( sort keys %$result)
{
	if($key=~ /([0-9]*)$/ )
    {
    	my $index =$1;
		my $descr = $$result{"$key"};
		#Il renvoit un caractère ASCI NULL
		if(ord($descr) !=0)
		{
			if($descr =~ /$match/)
			{
				#AGENT SNMP BUGGE, il ne renvoit pas les OID correctement.
				my $res = $snmp->get_request("$oid_type.$index");
				#si index est égal à 0 on n'ajoute pas le 0 finale
				my $type = -1;
				if($index == 0)
				{
					$type =  $$res{"$oid_type.$index"};
				}
				else
				{
					$type =  $$res{"$oid_type.$index.0"};
				}
				if($type == 3)
				{
					$oid_sensors = $oid_ds18b20;
				}
				if($oid_sensors ne '')
				{
					my $state = "OK";
					$res = $snmp->get_request("$oid_sensors.$index");
					my $temperature =  $$res{"$oid_sensors.$index.0"};
					$temperature = sprintf("%.2f", $temperature);
					my $message="good";
                    my $alrt_msg="< $warning";
					if($temperature < $warning)
					{
						$state = "OK";

					}
					elsif($temperature < $critical)
					{
						$state = "WARNING";
						$message="high";
                        $alrt_msg="> $warning";
					}
					else
					{
						$state = "CRITICAL";
						$message="very high";
                        $alrt_msg="> $critical";
					}
					$total = $total + $temperature;
					print "$state : $descr temperature is $message ($temperature C°) $alrt_msg\n";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
					$numbersensors++;
					$oid_sensors="";
				}
			}
		}
	}
}
if($numbersensors != 0)
{
	$total = sprintf("%.2f", $total/$numbersensors);
	print "|temperature=$total\n";
}
else
{
	print "UNKNOWN: No sensors found\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
}
exit($return);
