#!/usr/bin/perl -w
############################## check_snmp_sonicwall.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use JSON;

my $PROGNAME='check_snmp_sonicwall.pl';
my $PROGDESC = "Check Sonicwall Firewall by SNMP";

my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my $return=0;
my ($opt_h,$opt_V,$json);
my $type = 'cpu';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

my $warning = 80;
my $critical = 90;

#CPU
my $oid_old_sonicwall_cpu = ".1.3.6.1.4.1.4331.2.3.0";
my $oid_old_sonicwall_mem = "1.3.6.1.4.1.4331.2.4.1.0";
my $oid_old_sonicwall_max_connection =  "1.3.6.1.4.1.4331.2.2.2";
my $oid_old_sonicwall_current_connection =  "1.3.6.1.4.1.4331.2.2.1";
my $oid_sonicwall_cpu =  "1.3.6.1.4.1.8741.1.3.1.3.0";
my $oid_sonicwall_mem =  "1.3.6.1.4.1.8741.1.3.1.4.0";
my $oid_sonicwall_max_connection =  "1.3.6.1.4.1.8741.1.3.1.1.0";
my $oid_sonicwall_current_connection =  "1.3.6.1.4.1.8741.1.3.1.2.0";

my $oid_sonicwall_model = ".1.3.6.1.4.1.8741.2.1.1.1.0";
my $oid_sonicwall_serial= ".1.3.6.1.4.1.8741.2.1.1.2.0";
my $oid_sonicwall_firmware = ".1.3.6.1.4.1.8741.2.1.1.3.0";

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP

my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=s" => \$warning,"warning=s" => \$warning,
	"c=s" => \$critical,"critical=s" => \$critical,
	"J" => \$json,	 "json" => \$json,
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-w","--warning","warning threshold (default: $warning %)" ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical %) "]);
	push(@arr,["-T","--type","Type of test (default: cpu)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}

#si l'host ne répond pas on blocke le script
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if ($Sysoid_value ==-1)
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
#VEILLE OID SONIC WALL
if($Sysoid_value == 4331)
{
    $oid_sonicwall_cpu = $oid_old_sonicwall_cpu;
    $oid_sonicwall_mem = $oid_old_sonicwall_mem;
    $oid_sonicwall_max_connection = $oid_old_sonicwall_max_connection;
    $oid_sonicwall_current_connection = $oid_old_sonicwall_current_connection;
}
if($type eq 'cpu')
{
    my $state = 'OK';
    my $threshold = ESIA::SNMP::get($snmp,"$oid_sonicwall_cpu");
    if(defined($threshold) && $threshold ne 'noSuchObject')
    {
        my $message="(< $warning%)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $message="(> $critical%)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $message="(> $warning%)";
        }
        print "$state : CPU used $threshold% $message |cpu=$threshold\n";
        $return =  ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

    }
    else
    {
        print "UNKNOWN: can't get CPU information\n";
        $return =  ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
    }
}
elsif($type eq 'mem')
{
    my $state = 'OK';
    my $threshold = ESIA::SNMP::get($snmp,"$oid_sonicwall_mem");
    if(defined($threshold) && $threshold ne 'noSuchObject')
    {
        my $message="(< $warning%)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $message="(> $critical%)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $message="(> $warning%)";
        }
        print "$state : Memory used $threshold% $message |mem=$threshold\n";
        $return =  ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

    }
    else
    {
        print "UNKNOWN: can't get Memory information\n";
        $return =  ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
    }
}
elsif($type eq 'conn')
{
    my $state = 'OK';
    my $actualconn = ESIA::SNMP::get($snmp,"$oid_sonicwall_current_connection");
    if(defined($actualconn) && $actualconn ne 'noSuchObject')
    {
        my $maxconn = ESIA::SNMP::get($snmp,"$oid_sonicwall_max_connection");
        my $threshold = ($actualconn / $maxconn)*100;
        my $message="(< $warning%)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $message="(> $critical%)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $message="(> $warning%)";
        }
        $threshold = sprintf("%.1f", $threshold);
        print "$state : connections used $threshold% $message |max=$maxconn;actual=$actualconn;percent=$threshold\n";
        $return =  ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

    }
    else
    {
        print "UNKNOWN: can't get connections information\n";
        $return =  ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
    }
}
elsif( $type eq 'inventory' )
{
	my @array;
	my $model = ESIA::SNMP::get($snmp,"$oid_sonicwall_model");
	my $serial = ESIA::SNMP::get($snmp,"$oid_sonicwall_serial");
	my $firmware = ESIA::SNMP::get($snmp,"$oid_sonicwall_firmware");
	if($firmware=~/SonicOS\s+Enhanced\s+([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/)
	{
		$firmware=$1;
	}
	my %hash = ('name' => "SonicWall $model",'manufacturer'=>'	SonicWALL, Inc.', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@array, \%hash);
		my $output;
		if($json)
		{
		   $output = encode_json(\@array);
		}
		else
		{
			foreach my $line(@array)
		    {
				my $hash = \%$line;
				$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
			}
		}
		print $output;
}


exit($return);
