#!/usr/bin/perl -w
############################## check_snmp_sophos.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use JSON;
my $PROGNAME='check_snmp_sophos.pl';
my $PROGDESC = "Check Sophos firewall CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $json;
my $return=0;
my ($opt_h,$opt_V);
my $warning='';
my $critical='';
my $type = 'cpu';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#CPU
my $oid_sophos_cpu_core_load 	= "1.3.6.1.2.1.25.3.3.1.2"; #
# MEM
my $oid_sophos_mem_percent	    = "1.3.6.1.4.1.2604.5.1.2.5.2.0";
my $oid_sophos_disk_percent		= "1.3.6.1.4.1.2604.5.1.2.4.2.0";
#HA
my $oid_sophos_ha_enable		='1.3.6.1.4.1.2604.5.1.4.1.0'; #0= disabled 1=enable
my $oid_sophos_ha_master_mode	='1.3.6.1.4.1.2604.5.1.4.4.0';
my $oid_sophos_ha_peer_mode		='1.3.6.1.4.1.2604.5.1.4.5.0';
my %oid_sophos_ha_text = ( "0"  => "notapplicable","1" => "auxiliary","2"  => "standAlone","3"  => "primary","4"  => "faulty","5"  => "ready");

#inventory
my $oid_sophos_type		='1.3.6.1.4.1.2604.5.1.1.2.0';
my $oid_sophos_firmware	='1.3.6.1.4.1.2604.5.1.1.3.0';
my $oid_sophos_serial	='1.3.6.1.4.1.2604.5.1.1.4.0';
my $oid_sophos_peer_serial	='1.3.6.1.4.1.2604.5.1.4.3.0';

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"J" => \$json,	 "json" => \$json,
);
# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,disk,env,inventory] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	push(@arr,["-j","--json","json output for inventory -T inventory required"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem' || $type eq 'disk')
    {
        $warning=80;
    }
    else
    {
    	$warning=55;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
    else
    {
    	$critical=65;
    }
}
#
#Ubiquity switch
#

if($type eq 'cpu')
{
	my $state = 'OK';
	my $result = $snmp->get_table(Baseoid => $oid_sophos_cpu_core_load);
	my $number=0;
	my $cpu_load=0;
	foreach my $key ( keys %$result)
	{
		$cpu_load = $$result{"$key"} + $cpu_load;
		$number++;
	}
	if($number !=0)
	{
		$cpu_load=$cpu_load/$number;
	}
    my $message="(< $warning)";
    if($cpu_load >= $critical)
    {
    	$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($cpu_load >= $warning)
	{
		$state='WARNING';
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
	printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$cpu_load,$message,$cpu_load);
}
elsif($type eq 'mem')
{
	my $state = 'OK';
	my $used = ESIA::SNMP::get($snmp,"$oid_sophos_mem_percent");
	my $message="(< $warning)";
	if($used >= $critical)
	{
		$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($used >= $warning)
	{
		$state='WARNING';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
    printf("%s : memory used %.0f%% %s|mem=%.0f",$state,$used,$message,$used);
}
elsif($type eq 'disk')
{
	my $state = 'OK';
	my $used = ESIA::SNMP::get($snmp,"$oid_sophos_disk_percent");
	my $message="(< $warning)";
	if($used >= $critical)
	{
		$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($used >= $warning)
	{
		$state='WARNING';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
    printf("%s : disk used %.0f%% %s|disk=%.0f",$state,$used,$message,$used);
}
elsif($type eq 'ha')
{
	my $state = 'OK';
	my $ha_enable = ESIA::SNMP::get($snmp,"$oid_sophos_ha_enable");
	if($ha_enable == 1)
	{
		print "$state : HA is  enable\n";
		my $ha_master = ESIA::SNMP::get($snmp,"$oid_sophos_ha_master_mode");
		my $ha_peer = ESIA::SNMP::get($snmp,"$oid_sophos_ha_peer_mode");
		my $message = $oid_sophos_ha_text{$ha_master};
		if( ($ha_master == 3 || $ha_master == 5 ) && ($ha_peer==1 || $ha_peer==5))
		{
			$state='OK';
		}
		elsif( $ha_master == 4 || $ha_peer == 4 )
		{
			$state='CRITICAL';
		}
		else
		{
			$state='WARNING';
		}
		print "$state : Device current HA state is ".$oid_sophos_ha_text{$ha_master}." and peer HA is ".$oid_sophos_ha_text{$ha_peer}."\n";
	}
	else
	{
		print "$state : HA is not configured";
	}
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
}
elsif($type eq 'inventory')
{
	my @array;

	my $model = ESIA::SNMP::get($snmp,"$oid_sophos_type");
	my $firmware = ESIA::SNMP::get($snmp,"$oid_sophos_firmware");
	my $serial = ESIA::SNMP::get($snmp,"$oid_sophos_serial");
	my %hash = ('name' => 'Sophos Plc','manufacturer'=>'Sophos Plc', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@array, \%hash);
	my $ha_enable = ESIA::SNMP::get($snmp,"$oid_sophos_ha_enable");
	if($ha_enable == 1)
	{
		$serial = ESIA::SNMP::get($snmp,"$oid_sophos_peer_serial");
		my %hash = ('name' => 'Sophos Plc','manufacturer'=>'Sophos Plc', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
		push(@array, \%hash);
	}
	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
exit($return);
