#!/usr/bin/perl -w
############################## check_snmp_fujitsu #################

# Date : Juillet 06 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use lib "/usr/local/esia/plugins/lib/perl"  ;
use strict;
use JSON;
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;

#nom du programme & description
my $PROGNAME = "check_snmp_stormshield.pl";
my $PROGDESC = "Check Stormshield firewall by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $warning=80;
my $critical=90;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
#GLOBAL#
my $oid_proc_load = "1.3.6.1.2.1.25.3.3.1.2"; # %time the proc was not idle over last minute
my $oid_ram_free	= "1.3.6.1.4.1.2021.4.6.0";  # Real memory free
my $oid_ram_total	= "1.3.6.1.4.1.2021.4.5.0";  # Real memory total
my $oid_ram_cache   = "1.3.6.1.4.1.2021.4.15.0"; # Real memory cached
#disque
my $oid_dsk_descr = '1.3.6.1.2.1.25.2.3.1.3';
my $oid_dsk_size = '1.3.6.1.2.1.25.2.3.1.5';
my $oid_dsk_used = '1.3.6.1.2.1.25.2.3.1.6';
my $oid_dsk_alloc_units= '1.3.6.1.2.1.25.2.3.1.4';
#ha
my $oid_ha_name="1.3.6.1.4.1.11256.1.11.7.1.2";
my $oid_ha_online="1.3.6.1.4.1.11256.1.11.7.1.3";
my $oid_ha_model="1.3.6.1.4.1.11256.1.11.7.1.4";
my $oid_ha_licence="1.3.6.1.4.1.11256.1.11.7.1.6";
my $oid_ha_mode="1.3.6.1.4.1.11256.1.11.7.1.10";
my $oid_ha_sync= "1.3.6.1.4.1.11256.1.11.8.0";
#env
my $oid_power_state   = "1.3.6.1.4.1.11256.1.16.2.1.5.0";
my $oid_power_ha_state = "1.3.6.1.4.1.11256.1.11.10.1.3";
my $oid_fan_state     = "1.3.6.1.4.1.11256.1.16.2.1.6.0";
my $oid_cpu_state     = "1.3.6.1.4.1.11256.1.16.2.1.7.0";
my $oid_mem_state     = "1.3.6.1.4.1.11256.1.16.2.1.8.0";
my $oid_disk_state     = "1.3.6.1.4.1.11256.1.16.2.1.9.0";
my $oid_raid_state     = "1.3.6.1.4.1.11256.1.16.2.1.10.0";
my $oid_cert_state     = "1.3.6.1.4.1.11256.1.16.2.1.11.0";
my $oid_crl_state     = "1.3.6.1.4.1.11256.1.16.2.1.12.0";
my $oid_tpm_state     = "1.3.6.1.4.1.11256.1.16.2.1.13.0";
my $oid_password_state     = "1.3.6.1.4.1.11256.1.16.2.1.14.0";
my $oid_temp_state     = "1.3.6.1.4.1.11256.1.16.2.1.15.0";
my $oid_gateway_state     = "1.3.6.1.4.1.11256.1.16.2.1.16.0";
my $oid_NTP_state     = "1.3.6.1.4.1.11256.1.16.2.1.17.0";

my %oid_health_state = ( "Good"  => "OK","Minor" => "WARNING","Major"  => "CRITICAL","N/A"  => "OK");
my %oid_health_text = ( "Good"  => "is in good state","Minor" => "have a minor alarm ","Major"  => "have a major alarm","N/A"  => "in a non-applicable state (VM?)");

#vpn
my $oid_vpn_name="1.3.6.1.4.1.11256.1.1.1.1.4";
my $oid_vpn_state="1.3.6.1.4.1.11256.1.1.1.1.18";
my $oid_vpn_src="1.3.6.1.4.1.11256.1.1.1.1.5";
my $oid_vpn_dst="1.3.6.1.4.1.11256.1.1.1.1.6";

#VPn user
my $oid_vpn_user   = '1.3.6.1.4.1.11256.1.17.1.1.3';
my $oid_vpn_realIP = '1.3.6.1.4.1.11256.1.17.1.1.5';
my $oid_vpn_ovpnip = '1.3.6.1.4.1.11256.1.17.1.1.6';

#inventory
my $oid_model     = "1.3.6.1.4.1.11256.1.18.1.0";
my $oid_version   = "1.3.6.1.4.1.11256.1.18.2.0";
my $oid_serial    = "1.3.6.1.4.1.11256.1.18.3.0";

my $return=$ERRORS{OK};
my $type='cpu';
my $exclude='none';
my ($opt_h,$opt_V, $json);
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"T=s" => \$type,	 "type=s" => \$type,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
		"e=s" => \$exclude,	 "exclude=s" => \$exclude,
		"J" => \$json,	 "json" => \$json,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'');
	my @arr;
	push(@arr,["-T","--Type","type of test [cpu|mem|health|security|temp|inventory]. Default value 'cpu'" ]);
	push(@arr,["-w","--warning","warning threshold in percent  (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent (default: 90)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
$warning =~ s/\%//g;
$critical =~ s/\%//g;
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
########## MAIN ##########
my $graphdata="";
if( $type eq 'cpu' )
{
	my $result = $snmp->get_table( $oid_proc_load);
	my $load=0;
	my $nbrcpu = 0;
    foreach my $key ( keys %$result)
    {
    	$load=$load + $$result{$key};
    	$nbrcpu++;
	}
	if($nbrcpu == 0)
	{
		print "UNKNOWN: No CPU information found\n";
		exit $ERRORS{'UNKNOWN'};
	}
	else
	{
		$load = $load / $nbrcpu;
		$load = sprintf("%.1f",$load);
		my $state = "OK";
		my $message = "< $warning%";
		if($load < $warning)
		{
			$state = "OK";
		}
		elsif($load < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}

		print "$state: $nbrcpu CPU, average load $load% $message\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		$graphdata="cpu=$load";
	}
}
elsif( $type eq 'mem' )
{
	my $ram_free = ESIA::SNMP::get($snmp,"$oid_ram_free");
	if(!defined($ram_free))
	{
		print "UNKNOWN: Host not SNMP response $snmperror\n";
		exit $ERRORS{'UNKNOWN'};
	}
	my $ram_cache = ESIA::SNMP::get($snmp,"$oid_ram_cache");
	my $ram_total = ESIA::SNMP::get($snmp,"$oid_ram_total");
	my $ram_used = undef;
	if(defined($ram_cache))
	{
		$ram_used = $ram_total-$ram_free - $ram_cache;
	}
	else
	{
		$ram_used = $ram_total-$ram_free;
	}
	my $percent_used = ($ram_used/$ram_total)*100;
	$percent_used = sprintf("%.0f",$percent_used);
	my $ram_used_mb = sprintf("%.0f",$ram_used/1024);
	my $ram_total_mb = sprintf("%.0f",$ram_total/1024);
	my $state = "OK";
	my $message = "< ".$warning;
	if($percent_used < $warning)
	{
		$state = "OK";
	}
	elsif($percent_used < $critical)
	{
		$state = "WARNING";
		$message = ">".$warning;
	}
	else
	{
		$state = "CRITICAL";
		$message = "> ".$critical;
	}
	print "$state: Ram : $percent_used% (".$ram_used_mb."MB/".$ram_total_mb."MB) $message%\n";
	$graphdata="ram=$percent_used;ramByte=$ram_used;ramTotal=$ram_total;";
}
elsif( $type eq 'health' )
{
	my $model = ESIA::SNMP::get($snmp,"$oid_model");
	my $firmware = ESIA::SNMP::get($snmp,"$oid_version");
	my $serial = ESIA::SNMP::get($snmp,"$oid_serial");
	print "Stormshield Firewall $model (serial: $serial), version: $firmware\n";

	my $power = ESIA::SNMP::get($snmp,"$oid_power_state");
	my $state = $oid_health_state{$power};
	if($power eq 'Major')
	{
		my $result = $snmp->get_table( $oid_ha_name);
		my $size=0;
		my %hash;
		foreach my $key ( keys %$result)
		{
			$state = $$result{$key};
            if($key=~ /([0-9]*)$/ )
            {
            	$hash{$1}= $$result{$key};
				$size++;
			}

		}
		if($size > 0)
		{
			my $result = $snmp->get_table( $oid_power_ha_state);
			foreach my $key (sort keys %$result)
			{
			    if($key=~ /([0-9]*)\.([0-9]*)$/ )
		        {
		        	my $status = 'OK';
		        	my $idfw = $1;
		        	my $idpower = $2;
		        	if($exclude eq 'none' || $exclude ne $idpower)
		        	{
				    	my $res =  $$result{$key};
				    	if($res ne 'OK')
				    	{
							$status = 'CRITICAL';

						}
						print "$status: Power supply $idpower on ".$hash{$idfw}." is $res state\n";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$status});
					}
				}
				$state = $$result{$key};

			}
		}
		else
		{
			print "$state: Power supply ".$oid_health_text{$power}."\n";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
	}
	else
	{
		print "$state: Power supply ".$oid_health_text{$power}."\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}

	my $fan = ESIA::SNMP::get($snmp,"$oid_fan_state");
	$state = $oid_health_state{$fan};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: Fans ".$oid_health_text{$fan}."\n";

	my $cpu = ESIA::SNMP::get($snmp,"$oid_cpu_state");;
	$state = $oid_health_state{$cpu};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: CPU ".$oid_health_text{$cpu}."\n";

	my $mem = ESIA::SNMP::get($snmp,"$oid_mem_state");
	$state = $oid_health_state{$mem};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: Memory ".$oid_health_text{$mem}."\n";

	my $dsk = ESIA::SNMP::get($snmp,"$oid_disk_state");
	$state = $oid_health_state{$dsk};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: Disk ".$oid_health_text{$dsk}."\n";

	my $raid = ESIA::SNMP::get($snmp,"$oid_raid_state");
	$state = $oid_health_state{$raid};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: RAID ".$oid_health_text{$raid}."\n";
}
elsif( $type eq 'security' )
{
	my $cert = ESIA::SNMP::get($snmp,"$oid_cert_state");
	my $state = $oid_health_state{$cert};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: Certificate ".$oid_health_text{$cert}."\n";

	my $crl = ESIA::SNMP::get($snmp,"$oid_crl_state");
	$state = $oid_health_state{$crl};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: Certificate revocation list ".$oid_health_text{$crl}."\n";

	my $tpm = ESIA::SNMP::get($snmp,"$oid_tpm_state");
	$state = $oid_health_state{$tpm};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: TPM ".$oid_health_text{$tpm}."\n";

	my $pass = ESIA::SNMP::get($snmp,"$oid_password_state");
	$state = $oid_health_state{$pass};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: Administrator password ".$oid_health_text{$pass}."\n";

}
elsif( $type eq 'gateway' )
{
	my $gate = ESIA::SNMP::get($snmp,"$oid_gateway_state");
	my $state = $oid_health_state{$gate};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: All gateway ".$oid_health_text{$gate}."\n";
}
elsif( $type eq 'ha' )
{
	my $result = $snmp->get_table( $oid_ha_name);
	foreach my $key ( sort keys %$result)
	{
		if($key=~ /([0-9]*)$/ )
		{
			my $id = $1;
            my $sn = $$result{$key};
            my $state='OK';
			my $model = ESIA::SNMP::get($snmp,"$oid_ha_model.$id");
			my $online = ESIA::SNMP::get($snmp,"$oid_ha_online.$id");
			if($online ==1)
			{
				$online='online';
			}
			else
			{
				$online='offline';
				$state='CRITICAL';
			}
			my $licence = ESIA::SNMP::get($snmp,"$oid_ha_licence.$id");
			my $mode = ESIA::SNMP::get($snmp,"$oid_ha_mode.$id");
			if($mode ==1)
			{
				$mode='active';
			}
			else
			{
				$mode='passive';
				if( $licence eq 'Master')
				{
					$state='OK';
				}
			}
			print "$state: $sn ($model $licence) is $online and $mode\n";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
	}
	my $state='OK';
	my $sync = ESIA::SNMP::get($snmp,"$oid_ha_sync");
	if($sync == 1 )
	{
		print "OK: all nodes are synchronized\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'OK'});
	}
	elsif($sync == 0 )
	{
		print "WARNING: nodes are not synchronized\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'WARNING'});
	}
	else
	{
		print "CRITICAL: synchronisation has failed or unknown state. Check your firewall \n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'CRITICAL'});
	}
}
elsif( $type eq 'disk' )
{
	my $result = $snmp->get_table( $oid_dsk_descr);
	my $graphdata="";
	foreach my $key (sort keys %$result)
	{
		my $name = $$result{$key};
		if($name =~/^\/data$/ || $name =~/^\/log/ || $name =~/^\/$/)
    	{
			if($key=~/\.([0-9]*)$/)
			{
            	my $ind = $1;
				my $used = ESIA::SNMP::get($snmp,"$oid_dsk_used.$ind");
				my $size = ESIA::SNMP::get($snmp,"$oid_dsk_size.$ind");
				my $unit = ESIA::SNMP::get($snmp,"$oid_dsk_alloc_units.$ind");
				my $percent_used = 0;
				my $used_byte = $used * $unit;
				my $size_byte = $size * $unit;
				if($size != 0)
				{
					$percent_used = ($used_byte/$size_byte)*100;
				}
				$percent_used = sprintf("%.0f",$percent_used);
				my $message = "";
				my $state = "OK";
				$message = "< $warning%";
		        my $used_MB =sprintf("%.0f",$used_byte/1024/1024);
		        my $size_MB =sprintf("%.0f",$size_byte/1024/1024);
		        my $unitB ='MB';
		        if(int($used_byte/1099511627776) > 0 )
		        {
					$used_MB =sprintf("%.1f",$used_byte/1099511627776);
					$size_MB =sprintf("%.1f",$size_byte/1099511627776);
					$unitB ='TB';
		        }
		        elsif(int($used_byte/1073741824) > 0 )
				{
					$used_MB =sprintf("%.1f",$used_byte/1073741824);
		        	$size_MB =sprintf("%.1f",$size_byte/1073741824);
		        	$unitB ='GB';
		        }
				if($percent_used >= $critical)
				{
					$state = "CRITICAL";
					$message = "> $critical%";
				}
				elsif($percent_used >= $warning)
				{
					$state = "WARNING";
					$message = "> $warning%";
				}
				print "$state: $name: ".$percent_used."%"."used($used_MB"."$unitB/$size_MB"."$unitB) $message\n";
				my $graphname ='/';
				if($name eq '/')
				{
					$graphname='/';
				}
				elsif($name eq '/data')
				{
					$graphname='data';
				}
				elsif($name eq '/log')
				{
					$graphname='log';
				}
				$graphdata.="$graphname=$percent_used;";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
    	}
	}
	print "|$graphdata";
}
elsif( $type eq 'vpn' )
{
	my $result = $snmp->get_table( $oid_vpn_name);
	my $ok=0;
	my $nok=0;
	my @arr;
	foreach my $key ( keys %$result)
	{
		if($key=~ /([0-9]*)$/ )
		{
			my $id = $1;
            my $name = $$result{$key};
			my $status = ESIA::SNMP::get($snmp,"$oid_vpn_state.$id");
			if($status eq 'installed')
			{
				$ok++;
			}
			else
			{
				$nok++;
				my $src = ESIA::SNMP::get($snmp,"$oid_vpn_src.$id");
				my $dest = ESIA::SNMP::get($snmp,"$oid_vpn_dst.$id");
				my %hash;
				$hash{'name'}=$name;
				$hash{'state'}=$status;
				$hash{'src'}=$src;
				$hash{'dst'}=$dest;
				push(@arr,\%hash);
			}
		}
	}
	if($nok ==0)
	{
		print "OK: All VPN IPSec are OK ($ok / $ok)|installed=$ok;down=0\n";
	}
	else
	{
		my $state ='WARNING';
		if($nok > $critical)
		{
			$state ='CRITICAL';
		}
		my $total = $ok + $nok;
		print "$state : There are $ok VPN in installed state ($ok/$total).\n";
		foreach my $key (@arr)
		{
			print $key->{'name'}.' is in '.$key->{'state'}.' state (src:'.$key->{'src'}.', dst:'.$key->{'dst'}.")\n";
		}
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		print"|installed=$ok;down=$nok\n";
	}

}
elsif( $type eq 'vpn-user' )
{
	my $isdata=0;
	my $ismatch=0;
	if($warning eq '80')
	{
		$warning='zz_esia_zz2024';
	}
	if($critical eq '90')
	{
		$critical='zz_esia_zz2024';
	}
	my $result = $snmp->get_table( $oid_vpn_user);
    foreach my $key ( keys %$result)
    {
    	#on récupère l'index
	   	if($key=~ /([0-9]*)$/ )
	   	{
	   		my $id = $1;
	   		my $state = 'OK';
	   		my $user =  $$result{$key};
	   		$isdata=1;
	   		if($user =~/$critical/)
	   		{
	   			$state = 'CRITICAL';
	   			$ismatch=1;
	   		}
	   		elsif($user =~/$warning/)
	   		{
	   			$state = 'WARNING';
	   			$ismatch=1;
	   		}
	   		if($state ne 'OK')
	   		{
	   			my $localip = ESIA::SNMP::get($snmp,"$oid_vpn_realIP.$id");
				my $vpnIP = ESIA::SNMP::get($snmp,"$oid_vpn_ovpnip.$id");
				print "$state : $user is connected, local IP: $localip, VPN IP: $vpnIP \n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	   		}
	   	}
    }
    if($isdata == 0)
    {
    	print "OK: no one is connected\n";
    	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'OK'});
    }
	else
	{
		if($ismatch == 0)
		{
			if($warning eq 'zz_esia_zz2024' )
			{
				$warning='';
			}
			if($critical eq 'zz_esia_zz2024' )
			{
				$critical='';
			}
    		print "OK: rule does not match the user (warning: $warning or critical: $critical)\n";
    		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'OK'});
		}
	}

}
elsif( $type eq 'temp' )
{
	my $temp = ESIA::SNMP::get($snmp,"$oid_temp_state");
	my $state = $oid_health_state{$temp};
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	print "$state: CPU temperature ".$oid_health_text{$temp}."\n";
}
elsif( $type eq 'inventory' )
{
	my @array;

	my $model = ESIA::SNMP::get($snmp,"$oid_model");
	my $firmware = ESIA::SNMP::get($snmp,"$oid_version");
	my $serial = ESIA::SNMP::get($snmp,"$oid_serial");
	my %hash = ('name' => "Stormshield $model",'manufacturer'=>'Stormshield', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@array, \%hash);

	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
else
{
	print ("UNKNOWN: Incorrect or missed option -T, --type\n");
	$return = $ERRORS{'UNKNOWN'};
}
print "|$graphdata";
exit $return;
