#!/usr/bin/perl -w
############################## check_snmp_load #################
my $Version='1.0';
# Date : Oct 20 2014
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use JSON;
my $PROGNAME='check_snmp_sw_interface.pl';
my $PROGDESC = "Check switch, router and firewall bandwith by SNMP";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my ($opt_h,$opt_V,$debug);
my $ifindex;
my $path='/tmp/swinter';
my $warning='70';
my $critical='80';
my $error_crc;
my $down = 'ok';
my $delta_interval = 300;
my $speed=1000000000; # default 1000000000 = 1Gb/s
my $detectspeed="auto";
my $oid_outoctet='1.3.6.1.2.1.2.2.1.16';
my $oid_outHCoctet='1.3.6.1.2.1.31.1.1.1.10';
my $oid_inoctet='1.3.6.1.2.1.2.2.1.10';
my $oid_inHCoctet='1.3.6.1.2.1.31.1.1.1.6';
my $oid_speed='1.3.6.1.2.1.2.2.1.5';
my $oid_highspeed='1.3.6.1.2.1.31.1.1.1.15';
my $oid_descr = "1.3.6.1.2.1.2.2.1.2";
my $oid_name = ".1.3.6.1.2.1.31.1.1.1.1";
my $oid_alias = ".1.3.6.1.2.1.31.1.1.1.18";
my $oid_type = ".1.3.6.1.2.1.2.2.1.3";
my $oid_operstatus = "1.3.6.1.2.1.2.2.1.8";

my $oid_inUnicast ="1.3.6.1.2.1.2.2.1.11";
my $oid_inBroadcast ="1.3.6.1.2.1.2.2.1.12";
my $oid_inErrors = "1.3.6.1.2.1.2.2.1.14";

my $oid_outUnicast ="1.3.6.1.2.1.2.2.1.17";
my $oid_outBroadcast ="1.3.6.1.2.1.2.2.1.18";
my $oid_outErrors = "1.3.6.1.2.1.2.2.1.20";
my $oid_lastChange="1.3.6.1.2.1.2.2.1.9";
my $oid_sysUptime="1.3.6.1.2.1.1.3.0";
my $oid_ipifindex=".1.3.6.1.2.1.4.20.1.2";

my $oid_counter_type='32bit';
my $no_autodetect_counter;
my $threshold_type='percent';
sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -i ifindex [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
sub Trace
{
	 my($debug,$message) = @_;
	 if($debug)
	 {
	 	print "DEBUG: $message\n";
	 }
}
sub CongestionCal
{
	my($value,$speed) = @_;
	if($value < 0)
    {
    	#Changement de type de counter entre le 32 et le 64 bit
    	$value=$speed*10;
    }
    elsif($value > $speed)
    {
        #print "CONGESTION\n";
        my $maxspeed=0;
        if($speed < 100000000)#100Mbs
        {
            $maxspeed = $speed + ($speed/2);
        }
        elsif($speed < 1000000000)#1Gbs
        {
            $maxspeed = $speed + ($speed/10);
        }
        elsif($speed < 10000000000)#10Gbs
        {
            $maxspeed = $speed + ($speed/100);
        }
        elsif($speed < 10000000000)#10000Mbs
        {
            $maxspeed = $speed + ($speed/100);
        }
        # print "CONGESTION: $in < $maxspeed $speed?\n";
        if($value <= $maxspeed)
        {
        	#print "CONGESTION: mise au maximun\n";
            $value = $speed;
        }
    }
    return $value;
}
sub getExtendInterfaceData
{
	my($snmp,$oid) = @_;
	my $val = ESIA::SNMP::get($snmp,$oid);
	#corrige un bug sur cisco
    if($val eq 'noSuchInstance')
    {
        $val=0;

    }
    return $val;
}
sub TriggerSpeedToHuman
{
	my ($triggerspeed,$speed,$type) = @_;
	my ($human,$unity);
	if($type eq 'percent')
	{
		$triggerspeed = ($speed/100)*$triggerspeed;
		($human,$unity) = bitToHumanReadable($triggerspeed);
	}
	else
	{
		$human = $triggerspeed;
		$unity=ucfirst($type);
	}
	return "$human $unity";
}
sub bitToHumanReadable
{
	my ($bandwith) = @_;
	my $unity="b/s";
	# on regarde l'unité en Gb/S
	if( int($bandwith/1000000000)>0)
	{
		$bandwith = sprintf("%.1f", $bandwith/1000000000);
		$unity ="Gb/s";
	}
	# on regarde l'unité en Mb/S
	elsif( int($bandwith/1000000)>0)
	{
		$bandwith = sprintf("%.1f", $bandwith/1000000);
		$unity ="Mb/s";
	}
	elsif( int($bandwith/1000)>0)
	{
		$bandwith = sprintf("%.1f", $bandwith/1000);
		$unity ="Kb/s";
	}
	return ($bandwith,$unity);

}
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"D"   => \$debug,"debug"     => \$debug,
	"i=s" => \$ifindex,"ifindex=s" => \$ifindex,
	"w=s" => \$warning,"warning=s" => \$warning,
	"c=s" => \$critical,"critical=s" => \$critical,
	"s=s" => \$detectspeed,"speed=s"=>\$detectspeed,
	"d=s" => \$path,"directory=s"=>\$path,
	"T=s" => \$threshold_type,"Threshold=s"=>\$threshold_type,
	"a=s" => \$down,"arg=s"=>\$down,
    "e=s" => \$error_crc,"crc=s",\$error_crc,
	"32"=>\$no_autodetect_counter

);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-i","--ifindex","ifindex of the interface" ]);
	push(@arr,["-d","--directory","directory for tmp file (default: $path)" ]);
	push(@arr,["-D","--debug","enable Debug mode" ]);
	push(@arr,["-s","--speed","speed (default: $detectspeed)" ]);
	push(@arr,["-w","--warning","warning threshold (default: $warning %), you can specify an interval with ',' separator (example: -w 5,10) " ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical %) "]);
	push(@arr,["-T","--type","threshold unit [percent|kbs|mbs|gbs] (default: $threshold_type )"]);
	push(@arr,["-e","--crc","check crc error on interface warning and critical threshold are separete by ','. " ]);
	push(@arr,["","--32","disable detect counter 64" ]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#si l'host ne répond pas on blocke le script
Trace($debug,"CHECK SNMP respond");
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if ($Sysoid_value ==-1)
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
my $hostname = $snmp->hostname();
my $interInfo;
#préparation fichier cache
my $filecache = $hostname."_inter.json";
Trace($debug,"LOAD cache IFMIB");
my $cache = ESIA::CacheFile->new($path,$filecache,15);
if($cache->isValid() == 1)
{
    my $data = $cache->getData();
    $interInfo = decode_json($data);
}
else
{
	my $filelock = $hostname."_inter.lock";
	my $lock = ESIA::CacheFile->new($path,$filelock,20);
	my $locktime =0;
	if($lock->isValid() == 1)
	{
		$locktime = $lock->getData();
		Trace($debug,"file $filelock locktime: $locktime");
	}
	my $now = time();
	my $delta = $now - $locktime;
	if($delta > 1200)
	{
		# reload demandé
		$lock->writeData($now);
		Trace($debug,"create cache ifmib");
		my $res = $snmp->get_table(Baseoid =>$oid_descr);
		my @arr;
		my %hash_inter;
		foreach my $key ( keys %$res)
		{
		    if($key =~ /\.([0-9]*)$/)
			{
		        my %hash;
		        my $ind = $1;
		        $hash{'ifindex'} = $ind;
		        my $desc = $$res{"$key"};
		        my $la =chop($desc);
		        if($la =~ m/[0-9a-zA-Z]$/)
		        {
			        chomp($la);
			        $desc = $desc.$la;
		        }
		        $hash{'ifDescr'}=$desc;
		        $hash{'ifName'} = ESIA::SNMP::get($snmp,"$oid_name.$ind");
		        $hash{'ifAlias'} = ESIA::SNMP::get($snmp,"$oid_alias.$ind");
		        $hash{'ifType'} = ESIA::SNMP::get($snmp,"$oid_type.$ind");
		        push(@arr,\%hash);
		    }
		}
		$hash_inter{'ifmib'}=\@arr;
		my %hash;
		Trace($debug,"create cache ipmib");
		my $resultat = $snmp->get_table(Baseoid =>$oid_ipifindex);
		if(defined($resultat))
		{
		    foreach my $key ( keys %$resultat)
			{
				if($key =~ /\.([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)$/)
				{
		            $hash{$1} = $$resultat{"$key"};
		        }
		    }
		}
		$hash_inter{'ipmib'}=\%hash;
		my $json = encode_json \%hash_inter;
		$cache->writeData($json);
		$interInfo =  \%hash_inter;
	}
	else
	{
		Trace($debug,"There is another process that reload data.");
    	my $data = $cache->getData();
    	$interInfo = decode_json($data);
	}
}
my $name;
# On vérifie si on a un ifindex
if( !$ifindex)
{
    Trace($debug,"ifindex found by ip : $hostname ");
    if(exists $interInfo->{'ipmib'}->{$hostname})
    {
        $ifindex  = $interInfo->{'ipmib'}->{$hostname};
    }
    Trace($debug,"ifindex found: $ifindex ");
}
else
{
    Trace($debug,"search ifindex: $ifindex ");
    my $arr = $interInfo->{'ifmib'};
    foreach my $key ( @$arr )
	{
        my $found = 0;
        if ( $ifindex=~/^[0-9]*$/ )
        {
            if($key->{'ifindex'} == $ifindex)
            {
                $found = 1;
            }
        }
        elsif($key->{'ifDescr'} eq $ifindex)
        {
            $found = 1;
        }
        if($found == 1)
        {
            $name= $key->{'ifDescr'};
            $ifindex=$key->{'ifindex'};
            Trace($debug,"ifindex found: $ifindex = $name");
            if($key->{'ifAlias'} ne '' && $key->{'ifAlias'} ne 'noSuchInstance')
            {
                $name=$name.' ('.$key->{'ifAlias'}.')';
            }
            last;
        }
    }
}
if(!$ifindex)
{
    print "UNKNOWN: ifindex $ifindex not found on $hostname\n";
    exit $ERRORS{"UNKNOWN"};
}
#on interroge en SNMP pour récupérer les informations
my $result;
my $return = $ERRORS{'OK'};
my $outoctet = 0;
my $inoctet = 0;
my $operstatus = ESIA::SNMP::get($snmp,"$oid_operstatus.$ifindex");
#Sur certainz zyxels, les interfaces passe parfois en down (890 == sysOID de zyxel)-.
if( $operstatus == 2 && $Sysoid_value == 890 )
{
	sleep(10);
	$operstatus = ESIA::SNMP::get($snmp,"$oid_operstatus.$ifindex");
	if( $operstatus == 2 )
	{
		sleep(15);
		$operstatus = ESIA::SNMP::get($snmp,"$oid_operstatus.$ifindex");
	}
}

if($operstatus == 1) # Etat UP
{
	my $defaultspeed='';
	Trace($debug,"Interface $oid_operstatus.$ifindex is UP");
	#récupération de la vitesse de l'interface
	if($detectspeed eq 'auto')
	{
		my $result =ESIA::SNMP::get($snmp,"$oid_speed.$ifindex");
		if($result !=0)
		{
			$speed =  $result;
			if($speed > 4294900000)
			{
				Trace($debug,"speed is bad, check if highspeed");
				$speed =  ESIA::SNMP::get($snmp,"$oid_highspeed.$ifindex");
				$speed = $speed * 1000000;
			}
		}
		else
		{
			$defaultspeed='(speed recovery failed, use default speed)';
		}
	}
	else
	{
		$speed = $detectspeed*1000*1000 ;
	}
	Trace($debug,"SPEED : $speed $defaultspeed");
	#récupération du nombre d'octet en sortie avec le counter64 (ifHCOutOctets) si échoué utilise le counter32 (1.3.6.1.2.1.2.2.1.16)
	#récupère le nombre d'octet en entré suivant la valeur du dessus
	if($no_autodetect_counter)
	{
		Trace($debug,"No check counter");
		$outoctet =  ESIA::SNMP::get($snmp,"$oid_outoctet.$ifindex");
		$inoctet =  ESIA::SNMP::get($snmp,"$oid_inoctet.$ifindex");
	}
	else
	{
		$outoctet = ESIA::SNMP::get($snmp,"$oid_outHCoctet.$ifindex");
		if (!defined($outoctet))
		{
			$outoctet =  ESIA::SNMP::get($snmp,"$oid_outoctet.$ifindex");
			$inoctet =  ESIA::SNMP::get($snmp,"$oid_inoctet.$ifindex");
		}
		#certain implémente un counter64 mais le laisse à 0 (ex: certain switch TP-LINK)
		elsif($outoctet==0 || $outoctet eq 'noSuchObject' || $outoctet eq 'noSuchInstance')
		{
			$outoctet =  ESIA::SNMP::get($snmp,"$oid_outoctet.$ifindex");
			$inoctet =  ESIA::SNMP::get($snmp,"$oid_inoctet.$ifindex");
		}
		else
		{
			#OK le noeud est full compatible avec les compteurs 64 bit
			$inoctet =  ESIA::SNMP::get($snmp,"$oid_inHCoctet.$ifindex");
			$oid_counter_type='64bit';
		}
	}
	my $timestamp = time();
	#
	#récupération des informations sur les packet en erreur,broadcast,etc
	#
	my $inUnicast = getExtendInterfaceData($snmp,"$oid_inUnicast.$ifindex");
	my $inBroadcast = getExtendInterfaceData($snmp,"$oid_inBroadcast.$ifindex");
	my $inErrors = getExtendInterfaceData($snmp,"$oid_inErrors.$ifindex");
	#les infos en sortie
	my $outUnicast =  getExtendInterfaceData($snmp,"$oid_outUnicast.$ifindex");
	my $outBroadcast =  getExtendInterfaceData($snmp,"$oid_outBroadcast.$ifindex");
	my $outErrors =  getExtendInterfaceData($snmp,"$oid_outErrors.$ifindex");

	my $filename = $hostname.'_'.$ifindex.".json";

	Trace($debug,"Interface $ifindex counter type: $oid_counter_type and outoctet: $outoctet, inoctet $inoctet");
	#calcul de la bande passante utilisée
	my $outbit=$outoctet*8;
	my $intbit=$inoctet*8;
	my $cache_value;
	#préparation fichier cache
	my %hash;
	$hash{'time'}=$timestamp;
	$hash{'in'}=$intbit;
	$hash{'out'}=$outbit;
	$hash{'in_error'}=$inErrors;
	$hash{'out_error'}=$outErrors;
	$hash{'in_broadcast'}=$inBroadcast;
	$hash{'out_broadcast'}=$outBroadcast;
	$hash{'in_unicast'}=$inUnicast;
	$hash{'out_unicast'}=$outUnicast;

	Trace($debug,"CACHE FILE $path$filename");
	my $cachefile = ESIA::CacheFile->new($path,$filename,10);
	if($cachefile->isValid() == 1)
	{
		my $data = $cachefile->getData();
		$cache_value = decode_json($data);
	}
	else
	{
		$cache_value=\%hash;
	}
	#Ecriture fichier cache
	my $json = encode_json \%hash;
	$cachefile->writeData($json);
	Trace($debug,"NOW VALUE  time:".$timestamp.', out:'.$outbit.', in: '.$intbit );
	Trace($debug,"CACHE FILE time:".$cache_value->{'time'}.', out:'.$cache_value->{'out'}.', in: '.$cache_value->{'in'} );



	$outbit =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$outbit,$cache_value->{'time'},$cache_value->{'out'});
	$intbit =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$intbit,$cache_value->{'time'},$cache_value->{'in'});

	Trace($debug,"Interface $ifindex out bandwith usage: $outbit b/s speed: $speed");
	Trace($debug,"Interface $ifindex  in bandwith usage: $intbit b/s speed: $speed");

	$outbit = CongestionCal($outbit,$speed);
	$intbit = CongestionCal($intbit,$speed);

	Trace($debug,"Interface $ifindex out after congestion bandwith usage: $outbit b/s speed: $speed");
	Trace($debug,"Interface $ifindex  in after congestion bandwith usage: $intbit b/s speed: $speed");

	#on cacule les types de paquets unicast
	$inUnicast =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$inUnicast,$cache_value->{'time'},$cache_value->{'in_unicast'});
	$outUnicast =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$outUnicast,$cache_value->{'time'},$cache_value->{'out_unicast'});
	#on cacule les types de paquets broadcast
	$inBroadcast =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$inBroadcast,$cache_value->{'time'},$cache_value->{'in_broadcast'});
	$outBroadcast =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$outBroadcast,$cache_value->{'time'},$cache_value->{'out_broadcast'});

	#on cacule les types de paquets en erreur
	$inErrors =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$inErrors,$cache_value->{'time'},$cache_value->{'in_error'});
	$outErrors =  ESIA::PLUGIN::IntegerValueByTime($timestamp,$outErrors,$cache_value->{'time'},$cache_value->{'out_error'});

	#on test les valeur sur le warning et le critique
	my $state = 'OK';
	my ($speedHuman,$speedUnity) = bitToHumanReadable($speed);
	Trace($debug,"Interface $ifindex check warning or critical state");
    my $graphdata;
	if($outbit <= $speed && $intbit <= $speed)
	{
		my $inMessage = "";
		my $outMessage = "";

		my $percent_in = ($intbit/$speed)*100;
		my $percent_out = ($outbit/$speed)*100;
		#on unitialise la valeur à tester
		my $checked_in = $percent_in;
		my $checked_out = $percent_out;
		my $triggerspeed="";
		if($threshold_type ne 'percent')
		{
			Trace($debug,"Interface $ifindex not percent threshold");
			if($threshold_type eq 'mbs')
			{
				$checked_in = $intbit/1000000;
				$checked_out = $outbit/1000000;
			}
			elsif($threshold_type eq 'gbs')
			{
				$checked_in = $intbit/1000000000;
				$checked_out = $outbit/1000000000;
			}
			elsif($threshold_type eq 'kbs')
			{
				$checked_in = $intbit/1000;
				$checked_out = $outbit/1000;
			}
			else
			{
				$checked_in = $intbit;
				$checked_out = $outbit;
			}
			Trace($debug,"Interface $ifindex type value: ".$threshold_type.", out: ".$checked_out.", in: ".$checked_in);
		}

		#test si les valeur en entrée est en warning
		my ($valState,$oper,$triggers) = ESIA::PLUGIN::valueIsInError($checked_in,$warning);
		Trace($debug,"Interface $ifindex state: ".$valState." oper ".$oper." trigger ".$triggers);
		if($valState != 0)
		{
			$state = 'WARNING';
			my $msg = TriggerSpeedToHuman($triggers,$speed,$threshold_type);
			$inMessage = $oper.$msg;
		}
		#test si les valeur en sortie est en warning
		($valState,$oper,$triggers) = ESIA::PLUGIN::valueIsInError($checked_out,$warning);
		if($valState != 0)
		{
			$state = 'WARNING';
			my $msg = TriggerSpeedToHuman($triggers,$speed,$threshold_type);
			$outMessage = $oper.$msg;
		}
		#test si les valeur en entrée est en critique
		($valState,$oper,$triggers) = ESIA::PLUGIN::valueIsInError($checked_in,$critical);
		if($valState != 0)
		{
			$state = 'CRITICAL';
			my $msg = TriggerSpeedToHuman($triggers,$speed,$threshold_type);
			$inMessage = $oper.$msg;
		}
		#test si les valeur en sortie est en critique
		($valState,$oper,$triggers) = ESIA::PLUGIN::valueIsInError($checked_out,$critical);
		if($valState != 0)
		{
			$state = 'CRITICAL';
			my $msg = TriggerSpeedToHuman($triggers,$speed,$threshold_type);
			$outMessage = $oper.$msg;
		}
		my ($inHuman,$inUnity) = bitToHumanReadable($intbit);
		my ($outHuman,$outUnity) = bitToHumanReadable($outbit);
		$percent_in = sprintf("%.2f", $percent_in);
		$percent_out = sprintf("%.2f",$percent_out);
		$graphdata = "|data:in=".$intbit.":data:out=$outbit:speed:$speed:percent_in:$percent_in:percent_out:$percent_out";
		$graphdata .=":inUnicast:$inUnicast:inBroadcast:$inBroadcast:inErrors:$inErrors:outUnicast:$outUnicast:outBroadcast:$outBroadcast:outErrors:$outErrors";
		print "$state: $name in=$inHuman $inUnity".$inMessage.", out=$outHuman $outUnity".$outMessage.", speed: $speedHuman $speedUnity $defaultspeed Counter: $oid_counter_type";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
	else
	{
		#Si $speed est plus petit que 1kb/s c'est qu'il y a une erreur
		if($speed < 1000)
		{
			print "UNKNOWN $name: Bad speed return by device. speed: $speedHuman $speedUnity\n";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'UNKNOWN'});
		}
		else
		{
			print "OK $name: RESET COUNTER OR SPEED MODIFY speed: $speedHuman $speedUnity. Counter: $oid_counter_type\n";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'OK'});
		}
	}
    if($error_crc)
    {
        Trace($debug,"Interface $ifindex check crc error $error_crc");
        my ($crc_warning,$crc_critical);
        if($error_crc =~/([0-9]*)\:([0-9]*)/)
        {
            $crc_warning=$1;
            $crc_critical=$2;
        }
        elsif($error_crc =~/([0-9]*),([0-9]*)/)
        {
            $crc_warning=$1;
            $crc_critical=$2;
        }
        if($inErrors > $crc_critical || $outErrors > $crc_critical)
        {
            print "\nCRITICAL: $name has to many crc error, InError: $inErrors, outError: $outErrors > $crc_critical\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{'CRITICAL'});
        }
        elsif($inErrors > $crc_warning || $outErrors > $crc_warning)
        {
            print "\nWARNING: $name has crc error, InError: $inErrors, outError: $outErrors > $crc_warning\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{'WARNING'});
        }
    }
    print $graphdata;
}
elsif(($operstatus == 2 || $operstatus == 6) && $down eq 'critical')  # Etat DOWN
{
	print "CRITICAL: $name is down\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'CRITICAL'});
}
elsif(($operstatus == 2 || $operstatus == 6) && $down eq 'warning')  # Etat DOWN
{
	print "WARNING: $name is down\n";
    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{'WARNING'});
}
else
{
	print "OK $name: DOWN\n";
	exit $ERRORS{"OK"};
}
exit($return);
