#!/usr/bin/perl -w 
############################## check_snmp_synology.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Support Esia
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use JSON;

my $PROGNAME='check_snmp_synology.pl';
my $PROGDESC = "Check synology NAS by SNMP";
my $PROGVERSION = "1.0";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $return=0;
my ($opt_h,$opt_V,$json);
my $warning = 50;
my $critical = 60;
my $type="diskhealth";


ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );


my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";

my $oid_diskID="1.3.6.1.4.1.6574.2.1.1.2";
my $oid_diskModel="1.3.6.1.4.1.6574.2.1.1.3";
my $oid_diskSerial="1.3.6.1.4.1.6574.101.1.1.14";
my $oid_diskStatus="1.3.6.1.4.1.6574.2.1.1.5";
my $oid_diskHealthStatus="1.3.6.1.4.1.6574.2.1.1.13";
my $oid_diskTemp="1.3.6.1.4.1.6574.2.1.1.6";

my %disks_status_layout= (
        1  => 'The hard disk functions normally.',
        2  => 'Initialized : The hard disk has system partition but no data.',
        3  => 'NotInitialized : The hard disk does not have system in system partition.',
        4  => 'SystemPartitionFailed : The system partitions on the hard disks are damaged.',
        5  => 'Crashed : The hard disk has damaged.'
	);
my $disks_status_health_text = {
        1  => 'Normal',
        2  => 'Warning',
        3  => 'Critical',
        4  => 'Failing'
	};
my $disks_status_health_text_state = {
        1  => 'OK',
        2  => 'WARNING',
        3  => 'CRITICAL',
        4  => 'CRITICAL'
	};

my $oid_RAIDName="1.3.6.1.4.1.6574.3.1.1.2";
my $oid_RAIDStatus="1.3.6.1.4.1.6574.3.1.1.3";

my %raids_status_layout= (
        1  => 'The raid functions normally.',
        2  => 'Repairing',
        3  => 'Migrating',
        4  => 'Expanding',
        5  => 'Deleting',
        6  => 'Creating',
        7  => 'RaidSyncing',
        8  => 'RaidParityChecking',
        9  => 'RaidAssembling',
        10  => 'Canceling',
        11  => 'Degrade : Degrade happens when a tolerable failure of disk(s) occurs.',
        12  => 'Crashed : Raid has crashed and just uses for read-only operation.',
	);
my $oid_smart_test_name="1.3.6.1.4.1.6574.5.1.1.3";
my $oid_smart_test_status="1.3.6.1.4.1.6574.5.1.1.9";
my $oid_smart_test_raw="1.3.6.1.4.1.6574.5.1.1.8";
my $oid_smart_dev_name="1.3.6.1.4.1.6574.5.1.1.2";

my $oid_smart_current="1.3.6.1.4.1.6574.5.1.1.5";
my $oid_smart_thresh="1.3.6.1.4.1.6574.5.1.1.7";

my $oid_sys_status="1.3.6.1.4.1.6574.1.1.0";
my $oid_model="1.3.6.1.4.1.6574.1.5.1.0";
my $oid_sn="1.3.6.1.4.1.6574.1.5.2.0";
my $oid_DSMVersion="1.3.6.1.4.1.6574.1.5.3.0";
my $oid_DSMUpgradeAvailable="1.3.6.1.4.1.6574.1.5.4.0";
my $oid_power_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_sysfan_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_cpufan_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_temperature="1.3.6.1.4.1.6574.1.2.0";

my $oid_storage_io_name="1.3.6.1.4.1.6574.101.1.1.2";
my $oid_storage_io_5min="1.3.6.1.4.1.6574.101.1.1.10";
my $oid_storage_io_15min="1.3.6.1.4.1.6574.101.1.1.11";

my $oid_service_name="1.3.6.1.4.1.6574.6.1.1.2";
my $oid_service_user="1.3.6.1.4.1.6574.6.1.1.3";

my $oid_cluster_name        ="1.3.6.1.4.1.6574.106.4.0";
my $oid_cluster_status      ="1.3.6.1.4.1.6574.106.5.0";
my $oid_cluster_activeNode  ="1.3.6.1.4.1.6574.106.1.0";
my $oid_cluster_passiveNode ="1.3.6.1.4.1.6574.106.2.0";
my $oid_cluster_autofailover="1.3.6.1.4.1.6574.106.3.0";
my $oid_cluster_heartbeat   ="1.3.6.1.4.1.6574.106.6.0";
my %cluster_status_layout= (
        0  => 'normal',
        1  => 'warning',
        2  => 'critical',
        3  => 'upgrading',
        4  => 'processing'
	);
my %cluster_heartbeatstatus_layout= (
        0  => 'normal',
        1  => 'abnormal',
        2  => 'disconnected',
        3  => 'empty',
	);
sub print_help() {
	printf "check l'état de santé des disques d'un NAS Synology.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning disk temperature (default: 40)\n";
	print " -c                  critical disk temperature (default: 50)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}


#
#MAIN
#
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
        "T=s" => \$type,     "type=s" => \$type,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
		"J" => \$json,	 "json" => \$json,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'');
	my @arr;
	push(@arr,["-T","--Type","type of test [diskhealth|syshealth|service|io|cluster|inventory|update]. Default value 'diskhealth'" ]);
	push(@arr,["-w","--warning","warning threshold in percent or celsius. Depent of type test, io=percent, diskhealth and syshealth is Celsius value and service is a user number" ]);
	push(@arr,["-c","--critical","critical threshold in percent or celsius. Depent of type test, io=percent, diskhealth and syshealth is Celsius value and service is a user number"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

#on interroge en SNMP pour récupérer les informations 
my $result = ESIA::SNMP::get($snmp,"$oid_sysDescr");
#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{   printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	$snmp->close;
	exit $ERRORS{"UNKNOWN"};
}
my $status = "OK";
my $snmpresult;
############### DISK ###############
$status = "OK";
if($type eq 'diskhealth')
{
    ############### RAID ###############
    my $raids = $snmp->get_table(Baseoid => $oid_RAIDName);
    if (!defined($raids)) 
    {
       print "RAIDS na\n";
    }
    else
    {
	    foreach my $key ( keys %$raids) 
	    {	
		    if($key =~/6574\.3\.1\.1\.2\.([0-9]*)/)
		    {
			    my $id = "$1";

			    #RAID NAME
			    my $raid_name = $$raids{$key};		

			    #RAID STATUS
			    my $raid_status =ESIA::SNMP::get($snmp,"$oid_RAIDStatus.$id");
			    if($raid_status == 1)
			    {
				    $status = "OK";
			    }
			    elsif($raid_status == 11 || $raid_status == 12)
			    {
				    $status = "CRITICAL";
			    }
                else
                {
                    $status = "WARNING";
                }
			    print "$status : $raid_name ; $raids_status_layout{$raid_status}\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$status});
		    }
	    }
    }
    my $disks = $snmp->get_table(Baseoid => $oid_diskID);
    if (!defined($disks)) 
    {
       print "DISKS na\n";
    }
    else
    {
	    foreach my $key ( keys %$disks) 
	    {	
		    if($key =~/6574\.2\.1\.1\.2\.([0-9]*)/)
		    {
			    my $id = "$1";

			    #DISK NAME
			    my $disk_name = $$disks{$key};		
			    
			    #DISK MODEL
			    my $disk_model = ESIA::SNMP::get($snmp,"$oid_diskModel.$id");
			    $disk_model =~ s/\s+//g;

			    #DISK STATUS
			    my $disk_health = ESIA::SNMP::get($snmp,"$oid_diskHealthStatus.$id");
                if($disk_health>4)
                {
                    $status = "UNKNOWN";
                    $disks_status_health_text->{ $disk_health }="unknown value contact ESIA";  
                }
                else
                {
                    $status = $disks_status_health_text_state->{$disk_health};
                }
                my $message = $disks_status_health_text->{$disk_health};
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$status});

			    my $disk_status = ESIA::SNMP::get($snmp,"$oid_diskStatus.$id");
			    if($disk_status == 1 || $disk_status==2 || $disk_status==3)
			    {
				    $status = "OK";
			    }
			    else
			    {
				    $status = "CRITICAL";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			    }
			    my $disk_temp = ESIA::SNMP::get($snmp,"$oid_diskTemp.$id");
			    if($disk_temp < $warning)
			    {
				    $disk_temp = "Temperature : OK : $disk_temp < $warning Celsius";
			    }
			    else
			    {
				    if($disk_temp >= $critical)
				    {
					    $disk_temp = "Temperature : $disk_temp >= $critical Celsius";
                        $status="CRITICAL";
				    }
				    else # >= warning
				    {
                        $status="WARNING";
					    $disk_temp = "Temperature : $disk_temp >= $warning Celsius";
				    }	
			    }

			    print "$status : $disk_name ($disk_model) is $message state; $disks_status_layout{$disk_status} $disk_temp \n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$status});
		    }
	    }
    }

    ############### SMART ###############
    my $smart = $snmp->get_table(Baseoid => $oid_smart_test_name);
    if (!defined($smart)) 
    {
       print "SMART na\n";
    }
    else
    {
        my $smartmessage="";
        my %hash_disk;
	    foreach my $key ( keys %$smart) 
	    {	
		    if($key =~/6574\.5\.1\.1\.3\.([0-9]*)/)
		    {
			    my $id = "$1";

			    #DISK NAME
			    my $test_name = $$smart{$key};		
			    if( $test_name ne 'Airflow_Temperature_Cel')
                {
			        #DISK MODEL
			        my $test_status =ESIA::SNMP::get($snmp,"$oid_smart_test_status.$id");
			        if($test_status eq 'OK' )
			        {
				        $status = "OK";
                        my $raw_value="";
                        if($test_name eq 'Reallocated_Sector_Ct' || $test_name eq 'Re-allocated_Sector_Count')
                        {
			                $raw_value = ESIA::SNMP::get($snmp,"$oid_smart_test_raw.$id");
                            if($raw_value > 200)
                            {
			                    my $raw_current = ESIA::SNMP::get($snmp,"$oid_smart_current.$id");
			                    my $thresh = ESIA::SNMP::get($snmp,"$oid_smart_thresh.$id");
                                if($raw_current <= $thresh)
                                {
				                    $status = "WARNING";
                                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			                        my $dev = ESIA::SNMP::get($snmp,"$oid_smart_dev_name.$id");
                                    $smartmessage.= "$status : Error on $dev hight $test_name\n";
                                }
                            }
                        }
			        }
			        else
			        {
				        $status = "WARNING";
                        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			            my $dev = ESIA::SNMP::get($snmp,"$oid_smart_dev_name.$id");
                        $smartmessage.= "$status : $dev smart status ($test_name) is bad\n";
			        }
                }
		    }
	    }
        if($smartmessage eq "")
        {
            print "OK : All SMART test is good";      
        }
        else
        {
            print "WARNING: SMART test failed:\n$smartmessage";
        }
    }
}
############### SYSTEM ###############
elsif($type eq 'update')
{
    my $state = 'OK';
    my $version = ESIA::SNMP::get($snmp,"$oid_DSMVersion");
    my $upgradeAvail = ESIA::SNMP::get($snmp,"$oid_DSMUpgradeAvailable");
    my $message="";
    if($upgradeAvail == 1)
    {
	    $message = "An update is available";
        $state = 'WARNING';
    }
    elsif($upgradeAvail == 4)
    {
	    $message = "Update mirror is disconnected";
        $state = 'WARNING';
    }
    elsif($upgradeAvail == 2)
    {
	    $message = "Your NAS is up to date";
    }
    else
    {
	    $message = "Your update system is unknown state";
        $state = 'UNKNWON';
    }
    print "$state : version installed is $version. $message\n";
    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state}); 
}
############### SYSTEM ###############
elsif($type eq 'syshealth')
{
    my $model = ESIA::SNMP::get($snmp,"$oid_model");
    my $sys_status = ESIA::SNMP::get($snmp,"$oid_sys_status");
    if($sys_status == 1)
    {
	    $sys_status = "OK : System functionals normally";
    }
    else
    {
	    $sys_status = "CRITICAL : Volume has crashed.";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
    }
    my $sn = ESIA::SNMP::get($snmp,"$oid_sn");
    my $version = ESIA::SNMP::get($snmp,"$oid_DSMVersion");
    my $power_status =  ESIA::SNMP::get($snmp,"$oid_power_status");
    if($power_status == 1)
    {
	    $power_status = "OK : All power supplies functional normally."
    }
    else
    {
	    $power_status = "CRITICAL : One of power supply has failed.";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
    }
    my $sysfan_status = ESIA::SNMP::get($snmp,"$oid_sysfan_status");
    if($sysfan_status == 1)
    {
	    $sysfan_status = "OK : All Internal fans functional normally."
    }
    else
    {
	    $sysfan_status = "CRITICAL : One of internal fan stopped.";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
    }
    my $cpufan_status = ESIA::SNMP::get($snmp,"$oid_cpufan_status");
    if($cpufan_status == 1)
    {
	    $cpufan_status = "OK : All CPU fans functional normally."
    }
    else
    {
	    $cpufan_status = "CRITICAL : One of CPU fan stopped.";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
    }
    my $temperature = ESIA::SNMP::get($snmp,"$oid_temperature");
    if($temperature < $warning)
    {
	    $temperature = "OK : Temperature value is $temperature < $warning Celsius |temp=$temperature,warning=$warning,critical=$critical";
    }
    else
    {
	    if($temperature >= $critical)
	    {
		    $temperature = "CRITICAL : Temperature is very high $temperature >= $critical Celsius |temp=$temperature,warning=$warning,critical=$critical";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	    }
	    else # >= warning
	    {
		    $temperature = "WARNING : Temperature is high $temperature >= $warning Celsius |temp=$temperature,warning=$warning,critical=$critical";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
	    }	
    }

    print "Model : $model - SN : $sn - Version : $version\n";
    print "$sys_status\n";
    print "$power_status\n";
    print "$sysfan_status\n";
    print "$cpufan_status\n";
    print "$temperature\n";
}
elsif($type eq 'io')
{
    my $io = $snmp->get_table(Baseoid => $oid_storage_io_name);

    if (!defined($io)) 
    {
       print "IO na\n";
    }
    else
    {
        my $i = 0;
        my $total = 0;
	    foreach my $key ( keys %$io) 
	    {	
		    if($key =~/\.([0-9]*)$/)
		    {
                my $state="OK";
			    my $id = "$1";
                my $disk_name = $$io{$key};
                my $io_5min = ESIA::SNMP::get($snmp,"$oid_storage_io_5min.$id");
                my $message = "$disk_name IO load average (5min) is $io_5min%";   
                if($io_5min >=$critical)
                {
                    $state="CRITICAL";
                    $message.=" > $critical%";
                }
                elsif($io_5min >=$warning)
                {
                    $state="WARNING";
                    $message.=" > $warning%";
                }
                else
                {
                    $state="OK";
                    $message.=" < $warning%";
                }
                print "$state : $message\n";
		        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});	
                $total=$total+$io_5min;
                $i++;
            }
        }
        my $state="OK";
        my $load= sprintf("%.0f",($total/$i));      
        my $message = " < $warning%";       
        if($load >=$critical)
        {
            $state="CRITICAL";
            $message=" > $critical%";
        }
        elsif($load >=$warning)
        {
            $state="WARNING";
            $message=" > $warning%";
        }        
        print "$state : Average load : ".$load."$message|average=$load\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
}
elsif($type eq 'service')
{
    my $service = $snmp->get_table(Baseoid => $oid_service_name);
    if (!defined($service)) 
    {
       print "Service na\n";
    }
    else
    {
        my $graph_data="|";
	    foreach my $key ( keys %$service) 
	    {	
		    if($key =~/\.([0-9]*)$/)
		    {
                my $state="OK";
			    my $id = "$1";
                my $name = $$service{$key};
                my $user =  ESIA::SNMP::get($snmp,"$oid_service_user.$id");
                my $message ="< $warning";
                if($user >=$critical)
                {
                    $state="CRITICAL";
                    $message=" > $critical";
                }
                elsif($user >=$warning)
                {
                    $state="WARNING";
                    $message=" > $warning";
                }
                $graph_data.=$name.'='.$user.',';
                print"$state : service $name have $user user connected $message\n";
		        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
        print $graph_data;
    }
}
elsif($type eq 'cluster')
{
    my $name = ESIA::SNMP::get($snmp,"$oid_cluster_name");
    if (!defined($name) || $name eq "noSuchInstance") 
    {
       print "Cluster na\n";
    }
    else
    {
        my $state = 'OK';
        my $status = ESIA::SNMP::get($snmp,"$oid_cluster_status");
        my $active = ESIA::SNMP::get($snmp,"$oid_cluster_activeNode");
        my $pasive = ESIA::SNMP::get($snmp,"$oid_cluster_passiveNode");
        if (defined($status) && $status ne "noSuchInstance") 
        {
            if($status == 0 || $status == 3)
            {
                $state='OK';
            }
            elsif($status == 1 || $status == 4)
            {
                $state='WARNING';
            }
            else
            {
                $state='CRITICAL';
            }
        }
        else
        {
            $state='UNKNOWN';
        }
        print "$state : cluster $name (active node: $active passive: $pasive) is  $cluster_status_layout{$status} state.\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        $state = 'OK';
        my $heartbeat = ESIA::SNMP::get($snmp,"$oid_cluster_heartbeat");
        if($heartbeat == 0)
        {
            $state = 'OK';
        }
        elsif($heartbeat == 1 || $heartbeat == 2)
        {
            $state = 'CRITICAL';
        }
        else
        {
            $state = 'WARNING';
        }
        print "$state : heartbeat is $cluster_heartbeatstatus_layout{$heartbeat} state.\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        $state='OK';
        my $autofail = ESIA::SNMP::get($snmp,"$oid_cluster_autofailover");
        my $message = 'not activated';
        if($autofail ==1)
        {
            $message = 'activated'
        }
        print "$state : autofailover is $message\n";
    }
}
elsif($type eq 'inventory')
{
	my @output;
    my $model = ESIA::SNMP::get($snmp,"$oid_model");    
    my $serial = ESIA::SNMP::get($snmp,"$oid_sn");
    my $version = ESIA::SNMP::get($snmp,"$oid_DSMVersion");
    my %hash = (
				'name' => "Synology $model",
				'description'=>'',
				'manufacturer'=>'Synology',
				'model' => "$model",
				'type'=>'CHASSIS',
				'serial' => $serial,
				'hardwareVersion' => '',
				'firmwareVersion' => $version,
				);
    push(@output, \%hash);
    my $disks = $snmp->get_table(Baseoid => $oid_diskID);
    if (defined($disks)) 
    {
	    foreach my $key ( sort keys %$disks) 
	    {	
		    if($key =~/6574\.2\.1\.1\.2\.([0-9]*)/)
		    {
			    my $id = "$1";
			    my $disk_name = $$disks{$key};		
                #my $disk_serial = ESIA::SNMP::get($snmp,"$oid_diskSerial.$id");
			    my $disk_model = ESIA::SNMP::get($snmp,"$oid_diskModel.$id");
			    $disk_model =~ s/\s+//g;
                my $manufacturer = 'unknown';
                if($disk_model =~/^WD.*/)
                {
                    $manufacturer ='Western Digital';
                }
                elsif($disk_model =~/^ST.*/)
                {
                    $manufacturer ='Seagate';
                }
                elsif($disk_model =~/^KINGSTON*/)
                {
                    $manufacturer ='KINGSTON';
                    $disk_model =~ s/$manufacturer//g;
                }
                elsif($disk_model =~/^Samsung*/)
                {
                    $manufacturer ='Samsung';
                    $disk_model =~ s/$manufacturer//g;
                }
                $disk_model =~ s/\s+//g;
                my %hash = (
				            'name' => "$disk_name",
				            'description'=>'',
				            'manufacturer'=>"$manufacturer",
				            'model' => "$disk_model",
				            'type'=>'HDD',
				            'serial' => '',
				            'hardwareVersion' => '',
				            'firmwareVersion' => '',
				            );
                push(@output, \%hash);
		    }
	    }
    }

	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
		my $size = @output;
		if($size != 0)
		{
			foreach my $line ( @output)
			{

				my $hash = \%$line;
				print "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
			}
			exit(0);
		}
	}
}
else
{
    print "UNKNOWN: test type not found\n";
    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});

}
exit($return);
