#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : Oct 28 2025
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib '/usr/local/esia/plugins/lib/perl';
use JSON;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::INVENTORY;

my $PROGNAME = 'check_snmp_tcpy.pl';
my $PROGDESC = 'Check SNMP TCP session established';
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS = %ESIA::PLUGIN::ERRORS;

my $return=0;
my ($opt_h,$opt_V,$json);
my $warning = 80;
my $critical = 90;


my $oid_tcp_established='.1.3.6.1.2.1.6.9.0';
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
if($opt_h)
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T (cpu|mem|client|inventory)');
	my @arr;
	push(@arr,["-w","--warning","warning threshold (default: $warning %)" ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical %)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if(!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
my $established = ESIA::SNMP::get($snmp, "$oid_tcp_established");
my $state='OK';
my $message ="< $warning";
if($established > $critical)
{
	$state='CRITICAL';
	$message ="< $critical%";
}
elsif($established > $warning)
{
	$state='WARNING';
	$message ="> $warning%";
}
print "$state : there are $established TCP connections $message|tcp=$established\n";
$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});



exit($return);
