#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;

use JSON;

my $PROGNAME='check_snmp_ubiquity_switch.pl';
my $PROGDESC = "Check Ubiquity switch CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;


my $return=0;
my ($opt_h,$opt_V,$json);
my $warning='';
my $critical='';
my $type = 'env';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#CPU
my $oid_ubiquity_cpu_no_parser 	= "1.3.6.1.4.1.4413.1.1.1.1.4.13.0"; #
my $oid_ubiquity_cpu_parser 	= "1.3.6.1.4.1.4413.1.1.1.1.4.9.0"; #
# MEM
my $oid_ubiquity_mem_total	    = "1.3.6.1.4.1.4413.1.1.1.1.4.2.0"; # Total KBytes
my $oid_ubiquity_mem_free		= "1.3.6.1.4.1.4413.1.1.1.1.4.1.0"; # Free KBytes

# FAN
my $oid_ubiquity_fan_state	    = "1.3.6.1.4.1.4413.1.1.43.1.6.1.3";
my %oid_ubiquity_fan_state_text = ( "0"  => "Unknown","1" => "notpresent","2"  => "operational","3"  => "failed","4"  => "powering","5"  => "nopower","6"  => "notpowering","7"  => "incompatible");
my %oid_ubiquity_fan_status = ( "0"  => "UNKNOWN","1"  => "UNKNOWN","2" => "OK","3"  => "CRITICAL","4"  => "WARNING","5"  => "CRITICAL","6"  => "WARNING","7"  => "WARNING");
#Power Supply
my $oid_ubiquity_power_state	= "1.3.6.1.4.1.4413.1.1.43.1.7.1.3";

#Temp
my $oid_ubiquity_temperature		= "1.3.6.1.4.1.4413.1.1.43.1.8.1.5";
my $oid_ubiquity_temperature_state	= "1.3.6.1.4.1.4413.1.1.43.1.8.1.4";
my %oid_ubiquity_temperature_state_text = ( "0"  => "low","1" => "normal","2"  => "warning","3"  => "critical","4"  => "shutdown","5"  => "notpresent","6"  => "notoperational");
my %oid_ubiquity_temperature_status = ( "0"  => "WARNING","1"  => "OK","2" => "WARNING","3"  => "CRITICAL","4"  => "WARNING","5"  => "OK","6"  => "WARNING");

my $oid_proc_load = "1.3.6.1.2.1.25.3.3.1.2"; # %time the proc was not idle over last minute
my $oid_ram_free	= "1.3.6.1.4.1.2021.4.6.0";  # Real memory free
my $oid_ram_total	= "1.3.6.1.4.1.2021.4.5.0";  # Real memory total
my $oid_ram_cache   = "1.3.6.1.4.1.2021.4.15.0"; # Real memory cached
my $oid_swap_free	= "1.3.6.1.4.1.2021.4.4.0";  # swap memory free
my $oid_swap_total	= "1.3.6.1.4.1.2021.4.3.0";  # Swap memory total
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"J" => \$json,"--json" => \$json,
);
# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,env] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
    else
    {
    	$warning=55;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
    else
    {
    	$critical=65;
    }
}
#
#Ubiquity switch
#
if($Sysoid_value =~/41112/)
{
	if($type eq 'cpu')
	{
		my $result = $snmp->get_table( $oid_proc_load);
		my $load=0;
		my $nbrcpu = 0;
		foreach my $key ( keys %$result)
		{
			$load=$load + $$result{$key};
			$nbrcpu++;
		}
		if($nbrcpu == 0)
		{
			print "UNKNOWN: No CPU information found\n";
			exit $ERRORS{'UNKNOWN'};
		}
		else
		{
			$load = $load / $nbrcpu;
			$load = sprintf("%.1f",$load);
			my $state = "OK";
			my $message = "< $warning%";
			if($load < $warning)
			{
				$state = "OK";
			}
			elsif($load < $critical)
			{
				$state = "WARNING";
				$message = "> $warning%";
			}
			else
			{
				$state = "CRITICAL";
				$message = "> $critical%";
			}
			printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$load,$message,$load);
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
	}
	elsif($type eq 'mem')
	{
		my $state = "OK";
		my $message = "< $warning%";
		my $ram_free = ESIA::SNMP::get($snmp,"$oid_ram_free");
		if(!defined($ram_free))
		{
			print "UNKNOWN: Host not SNMP response $snmperror\n";
			exit $ERRORS{'UNKNOWN'};
		}
		my $ram_cache = ESIA::SNMP::get($snmp,"$oid_ram_cache");
		my $ram_total = ESIA::SNMP::get($snmp,"$oid_ram_total");
		my $ram_used = undef;
		if(defined($ram_cache))
		{
			$ram_used = $ram_total-$ram_free - $ram_cache;
		}
		else
		{
			$ram_used = $ram_total-$ram_free;
		}
		my $percent_used = ($ram_used/$ram_total)*100;
		$percent_used = sprintf("%.0f",$percent_used);
		if($percent_used < $warning)
		{
				$state = "OK";
		}
		elsif($percent_used < $critical)
		{
			$state = "WARNING";
			$message = "> $warning%";
		}
		else
		{
			$state = "CRITICAL";
			$message = "> $critical%";
		}
		printf("%s :memory used %.0f%% %s|mem=%.0f",$state,$percent_used,$message,$percent_used);
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
	elsif($type eq 'env')
	{
		print "OK : No sensors avaible on this model\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
	}
	elsif($type eq 'inventory')
	{
		my @array;
		my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
		my $sysdescr = ESIA::SNMP::get($snmp,"$oid_sysDescr");
		my $version = '';
		if($sysdescr=~/EdgeOS\s+v([0-9]*\.[0-9]*\.[0-9]*)/)
		{
			$version=$1;
		}
		my %hash = ('name' => "EdgeOS",'manufacturer'=>'Ubiquiti Networks', 'model' =>'' , 'type'=>'OS','serial'=>'','hardwareVersion' =>'', 'firmwareVersion' =>$version);
		push(@array, \%hash);

		my $output;
		if($json)
		{
		   $output = encode_json(\@array);
		}
		else
		{
			foreach my $line(@array)
		    {
				my $hash = \%$line;
				$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
			}
		}
		print $output;
	}
}
else
{
	if($type eq 'cpu')
	{
		my $state = 'OK';
		my $threshold = ESIA::SNMP::get($snmp,"$oid_ubiquity_cpu_no_parser");
		if(!$threshold =~/[0-9]*/)
		{
			my $parser = ESIA::SNMP::get($snmp,"$oid_ubiquity_cpu_parser");
			if($parser =~/300\s+Secs\s+\(([\s0-9\.]*)[\%\s]*\)/)
			{
				$threshold=$1;
				$threshold =~ s/^\s+|\s+$//g;
			}
			else
			{
				$state='UNKNOWN';
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
				print "UNKNOWN: bad CPU value: $parser\n";
			}
		}
		my $message="(< $warning)";
		if($threshold >= $critical)
		{
			$state='CRITICAL';
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$message="(> $critical)";
		}
		elsif($threshold >= $warning)
		{
			$state='WARNING';
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$message="(> $warning)";
		}
		printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$threshold,$message,$threshold);
	}
	elsif($type eq 'mem')
	{
		my $state = 'OK';
		my $total = ESIA::SNMP::get($snmp,"$oid_ubiquity_mem_total");
		my $memory = ESIA::SNMP::get($snmp,"$oid_ubiquity_mem_free");
	 	my $used = 100 - ($memory / $total)*100;
		my $message="(< $warning)";
		if($used >= $critical)
		{
			$state='CRITICAL';
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$message="(> $critical)";
		}
		elsif($used >= $warning)
		{
			$state='WARNING';
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$message="(> $warning)";
		}
		printf("%s :memory used %.0f%% %s|mem=%.0f",$state,$used,$message,$used);
	}
	elsif($type eq 'env')
	{
		my $result = $snmp->get_table(Baseoid => $oid_ubiquity_fan_state);
		foreach my $key ( keys %$result)
		{
			#on récupère l'index
		   	if($key=~ /([0-9]*)$/ )
		   	{
		            my $index =$1;
				    my $status =$$result{"$key"};
		            my $state = $oid_ubiquity_fan_status{$status};
		            print "$state : Fan $index is ".$oid_ubiquity_fan_state_text{$status}."\n";
		            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			}

		}
		$result = $snmp->get_table(Baseoid => $oid_ubiquity_power_state);
		foreach my $key ( keys %$result)
		{
			#on récupère l'index
		   	if($key=~ /([0-9]*)$/ )
		   	{
		            my $index =$1;
				    my $status =$$result{"$key"};
		            my $state = $oid_ubiquity_fan_status{$status};
		            print "$state : Power supply $index is ".$oid_ubiquity_fan_state_text{$status}."\n";
		            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			}

		}
		$result = $snmp->get_table(Baseoid => $oid_ubiquity_temperature);
		my $total=0;
		my $i=0;
		foreach my $key ( sort keys %$result)
		{
			#on récupère l'index
		   	if($key=~ /([0-9]*\.[0-9]*)$/ )
		   	{
		            my $index =$1;
				    my $temperature =$$result{"$key"};
				    my $status = ESIA::SNMP::get($snmp,"$oid_ubiquity_temperature_state.$index");
		            my $state = $oid_ubiquity_temperature_status{$status};
		            print "$state : Sensor $index is ".$oid_ubiquity_temperature_state_text{$status}.", measured temperature: $temperature°C\n";
		            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		            $total=$total+$temperature;
		            $i++;
			}
		}
		my $average = sprintf('%.1f',$total/$i);
		my $state = 'OK';
		my $message="(< $warning)";
		if($average >= $critical)
		{
			$state='CRITICAL';
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$message="(> $critical)";
		}
		elsif($average >= $warning)
		{
			$state='WARNING';
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$message="(> $warning)";
		}
		print "$state: average temperature: $average°C $message|temp=$average\n";
	}
}
exit($return);
