#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;
use Net::SNMP qw(ticks_to_time);
my $PROGNAME='check_snmp_hp_switch.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $DEBUG;
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning=-1;
my $critical=-1;

my $oid_sysUpTime    = "1.3.6.1.2.1.1.3.0";
my $oid_hostUptime   = "1.3.6.1.2.1.25.1.1.0";

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password
my $snmp_domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4

# Convert a ticks string into integer format (reverse of ticks_to_time function)
sub convertTicksToTime {
	my ($ticks) = @_;

	# Get all numbers into an array (81 days, 06:29:20.67 => [81, 06, 29, 20, 67], 23.88 seconds => [23, 88])
	my @timeArr = ($ticks =~ /([0-9]+)\D*/g);
	# Reverse the array
	@timeArr = (reverse @timeArr);
	# Add missing element until days
	my $count = @timeArr;
	while($count < 5)
	{
		push(@timeArr, '00');
		$count++;
	}
	# Convert the array into time integer
	my $time = $timeArr[0]; # Seconds fraction
	$time += $timeArr[1] * 100; # Seconds
	$time += $timeArr[2] * 6000; # Minutes
	$time += $timeArr[3] * 360000; # Hours
	$time += $timeArr[4] * 8640000; # Days

	return $time;
}

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp traffic IP version.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning threshold in minutes\n";
	print " -c                  critical threshold in minutes\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}
sub Trace {
     my ($txt) = @_;
    if ($DEBUG)
    {
        print "$txt\n";
    }
}
my $vide="";
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "D"   => \$DEBUG,	"debug"       => \$DEBUG,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
    "o=s" => \$vide,"oid=s" => \$vide,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
if($hostname =~ /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/ )
{
    $snmp_domain ="UDP/IPV6";
}
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd))
{
    $snmp_version =3;
}
if ($snmp_version == 3)
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass))
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -domain     => $snmp_domain,
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );
    }
    else
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -domain     => $snmp_domain,
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
        -domain     => $snmp_domain,
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
        -domain     => $snmp_domain,
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations
$snmp->translate([-all => 0]);
my $result = $snmp->get_request("$oid_sysUpTime");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
            -domain     => $snmp_domain,
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysUpTime");
		if (!defined($result))
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
	    exit $ERRORS{"UNKNOWN"};
	}
}
my $uptime = $$result{"$oid_sysUpTime"};
my $types = $snmp->var_bind_types();

if($$types{"$oid_sysUpTime"} == 2) #Si le type est integer
{
    if( !($uptime =~/00$/)) #et qu'il ne fini pas par 00 c'est que le compteur est en seconde
    {
        $uptime = $uptime*100;
    }
}

$result = $snmp->get_request("$oid_hostUptime");
if ( defined($result))
{
    if( $$result{"$oid_hostUptime"} =~ /^\d+$/)
    {
        $uptime = $$result{"$oid_hostUptime"};
    }
}

# Ensure that $uptime is the time integer & $time is the ticks string
my $time = '';
# If uptime is an integer or a ticks string
if($uptime =~ /\D/)
{
	# Is a Ticks string
	$time = $uptime;
	$uptime = convertTicksToTime($uptime);
}
else
{
	# Is an Integer time
	$time = ticks_to_time($uptime);
}

my $status ="OK";
my $message = "node is started from $time";
if($uptime > 0)
{
	if(($critical * 6000) > $uptime)
	{
	    $status ='CRITICAL';
	    $message ="node has restarted. Uptime: $time < $critical minutes";
	}
	elsif(($warning * 6000) > $uptime)
	{
	    $status ='WARNING';
	    $message ="node has restarted. Uptime: $time < $warning minutes";
	}
	$uptime = int($uptime/100);
	print "$status : $message |iso.3.6.1.2.1.1.3.0=$uptime\n";
}
else
{
	$message ="bad format uptime $uptime";
    print "$status : $message |iso.3.6.1.2.1.1.3.0=$uptime\n";
}

exit($ERRORS{$status});
