#!/usr/bin/perl
use strict;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use JSON;
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $PROGNAME='check_snmp_win.pl';
my $PROGDESC = "Check Windows service by SNMP";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

# SNMP OID
my $oid_svSvcEntry = '.1.3.6.1.4.1.77.1.2.3.1';
my $oid_svSvcName  = '.1.3.6.1.4.1.77.1.2.3.1.1';
my $oid_svSvcInstalledState  = '.1.3.6.1.4.1.77.1.2.3.1.2';
my $oid_svSvcOperatingState  = '.1.3.6.1.4.1.77.1.2.3.1.3';
my $oid_svSvcUninstalled  = '.1.3.6.1.4.1.77.1.2.3.1.4';

my %label_svSvcInstalledState = ( 1 => 'uninstalled', 2=> 'install-pending', 3=> 'uninstall-pending', 4=> 'installed');
my %label_svSvcOperatingState = ( 1 => 'active', 2=> 'continue-pending', 3=> 'pause-pending', 4=> 'paused');
my %label_svSvcUninstalled = ( 1 => 'cannot-be-uninstalled', 2=> 'can-be-uninstalled');

my $o_host = 		undef; 	# hostname
my $o_timeout=  	undef;  # Timeout (Default 10)
my $o_help=		undef; 	# wan't some help ?
my $o_warn=             undef;  # WARNING alert if system has been up for < specified number of minutes
my $o_crit=             undef;  # CRITICAL alert if system has been up for < specified number of minutes
my $debug=undef;
my $o_descr = undef;
my $o_noreg = undef;
my $o_number = undef;
my $o_showall = undef;
my $o_brief = undef;
my $path='/tmp/winservice';

my @o_descrL = 	undef;
sub isnnum
{ # Return true if arg is not a number
	my $num = shift;
	if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
	return 1;
}

sub isnotnum
{ # Return true if arg is not a number
	my $num = shift;
  	if ( $num =~ /^-?(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  	return 1;
}
sub is_pattern_valid
{ 	# Test for things like "<I\s*[^>" or "+5-i"
 	my $pat = shift;
 	if (!defined($pat)) { $pat=" ";} # Just to get rid of compilation time warnings
 	return eval { "" =~ /$pat/; 1 } || 0;
}
sub getlist
{
    my ($session, $first_oid) = @_;
    my $result=undef;
    my $ret=undef;
    my $next=undef;
    my $key=undef;

    $result = $session->get_next_request(-varbindlist=>[ $first_oid ]);
    if (!defined($result))
    {
       printf("ERROR: Can not retrieve $oid_svSvcName table: %s.\n", $session->error);
       $session->close;
       exit $ERRORS{"UNKNOWN"};
    }

    #$key = (keys $result)[0];
    $key = (keys %{$result})[0];

    $next = $key;
    $key =~ s/$first_oid//g;

    while(defined($key))
    {
        $$ret{$key}=decode($$result{$next});
        $key=undef;

        $result = $session->get_next_request(-varbindlist=>[ $next ]);
        if(defined ($result))
        {
           # $key = (keys $result)[0];
            $key = (keys %{$result})[0];
            $next = $key;
            $key =~ s/$first_oid//g;
            if( $next eq $key )
            {
                $key=undef;
            }
        }
    }
    return $ret;
}
sub Trace
{
	 my($debug,$message) = @_;
	 if($debug)
	 {
	 	print "DEBUG: $message\n";
	 }
}
sub decode
{
	my $utfstr=shift;
  	if (substr($utfstr,0,2) ne "0x")
  	{
  		return $utfstr;
  	}
  	my @stringL=split(//,$utfstr);
  	my $newstring="";
  	for (my $i=2;$i<$#stringL;$i+=2)
  	{
		if ( ($stringL[$i] . $stringL[$i+1]) eq "c3")
		{
		    $i+=2;$newstring .= ".";
		}
		else
		{
		  $newstring .= chr(hex($stringL[$i] . $stringL[$i+1]));
		}
	}
	return $newstring;
}
sub print_usage()
{
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -n name -N number\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
sub print_help()
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-d","--directory","directory for tmp file (default: $path)" ]);
	push(@arr,["-n","--name","Comma separated names of services (perl regular expressions can be used for every one)." ]);
	push(@arr,["-r","--noregexp"," Do not use regexp to match NAME in service description." ]);
	push(@arr,["-N","--number","Compare matching services with <n> instead of the number of names provided" ]);
	push(@arr,["-b","--brief","Show only first line." ]);
	push(@arr,["-s","--showall","Show all services in the output, instead of only the non-active ones." ]);
	push(@arr,["-D","--debug","enable Debug mode" ]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}


&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions('h'     => \$o_help,    	'help'        	=> \$o_help,
		"D"   => \$debug,			"debug"     => \$debug,
        'n:s'   => \$o_descr,		'name:s'	    => \$o_descr,
        'r'     => \$o_noreg,       'noregexp'      => \$o_noreg,
        'N:i'   => \$o_number,      'number:i'      => \$o_number,
        'b'     => \$o_brief,  	    'brief'         => \$o_brief,
	    's'     => \$o_showall,  	'showall'       => \$o_showall,
	    "d=s" 	=> \$path,			"directory=s"=>\$path
	    );
if(defined ($o_help))
{
	print_help();
	exit $ERRORS{"UNKNOWN"};
};

if (!defined($o_descr) && !defined($o_showall) )
{
	print_help();
	exit $ERRORS{"UNKNOWN"};
};
@o_descrL=split(/,/,$o_descr);
foreach my $List (@o_descrL)
{
	if (!is_pattern_valid($List))
	{
		print "Invalid pattern ! ";
		print_usage();
		exit $ERRORS{"UNKNOWN"};
	}
}
if(defined ($o_number))
{
	if (isnotnum($o_number) || ($o_number<0) )
	{
		print "Invalid number of services!\n";
		print_usage();
		exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if (!defined($Sysoid_value))
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
my $listNames=undef;
my $all_str = '';
Trace($debug,"LOAD cache");
my $hostname = $snmp->hostname();
my $filecache = $hostname."_service.json";
my $cache = ESIA::CacheFile->new($path,$filecache,10);
if($cache->isValid() == 1)
{
	Trace($debug,"cache is valid");
    my $data = $cache->getData();
    $listNames = decode_json($data);
}
else
{
	Trace($debug,"create cache");
	$listNames = getlist($snmp,$oid_svSvcName);
	my $json = encode_json $listNames;
	$cache->writeData($json);
}
##For full stdout
if(defined($o_showall))
{
	$listNames = getlist($snmp,$oid_svSvcName);
	my $listInstallStates = getlist($snmp,$oid_svSvcInstalledState);
	my $listOperatingStates = getlist($snmp,$oid_svSvcOperatingState);
    foreach my $key ( keys %$listNames)
    {
        $all_str .= $label_svSvcInstalledState{$$listInstallStates{$key}} . " and ";
        $all_str .= $label_svSvcOperatingState{$$listOperatingStates{$key}} . " : ";
        $all_str .= $$listNames{$key}."\n";
    }
    if( !defined($o_descr))
    {
        print "UNKNOWN : NO STATUS. Show only\n";
        print "$all_str";
        exit $ERRORS{"UNKNOWN"};
    }
}
my $test=undef;
my $nb_found=0;
if( !defined($o_showall) && !defined($o_brief))
{
	foreach my $key ( keys %$listNames)
	{
		$test=undef;
		foreach my $List (@o_descrL)
		{
		    if (!($test))
		    {
	  	        $test = defined($o_noreg) ? $$listNames{$key} eq $List : $$listNames{$key} =~ /$List/i;
		    }
		}
		if ($test)
		{
			my $install = ESIA::SNMP::get($snmp,$oid_svSvcInstalledState.$key);
			my $operating = ESIA::SNMP::get($snmp,$oid_svSvcOperatingState.$key);
		    $all_str .= $label_svSvcInstalledState{$install} . " and ";
		    $all_str .= $label_svSvcOperatingState{$operating} . " : ";
		    $all_str .= $$listNames{$key}."\n";
		   	$nb_found++;
		}
	}
}
my $type;
if( $o_noreg )
{
    $type = 'equal to';
}
else
{
    $type = 'with pattern';
}
if( $nb_found == 0 )
{
    print "CRITICAL : Found $nb_found $type ($o_descr)\n$all_str";
    exit $ERRORS{"CRITICAL"};
}
elsif( defined($o_number) && $o_number != $nb_found )
{
    print "WARNING : Found $nb_found/$o_number $type ($o_descr)\n$all_str";
    exit $ERRORS{"WARNING"};
}
else
{
    print "OK : Found $nb_found $type ($o_descr)\n$all_str";
    exit $ERRORS{"OK"};
}
