#!/usr/bin/perl -w
############################## check_snmp_zyxel_load.pl ########
my $Version='1.0';
# Date : May 27 2024
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use JSON;


my $PROGNAME='check_snmp_zyxel.pl';
my $PROGDESC = "Check Zyxel USG by SNMP";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

my $return=0;
my ($opt_h,$opt_V, $json);


my $warning = 80;
my $critical = 90;
my $type = 'cpu';
my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
my $oid_cpu_load = "1.3.6.1.4.1.890.1.6.22.1.5.0";
my $oid_memory   = "1.3.6.1.4.1.890.1.6.22.1.2.0";
my $oid_flash  = "1.3.6.1.4.1.890.1.6.22.1.7.0";
my $oid_session  = "1.3.6.1.4.1.890.1.6.22.1.6.0";
my $oid_model = '1.3.6.1.4.1.890.1.15.3.1.11.0';
my $oid_version = '1.3.6.1.4.1.890.1.15.3.1.6.0';
my $oid_serial = '1.3.6.1.4.1.890.1.15.3.1.12.0';
my $oid_hw_version = '1.3.6.1.4.1.890.1.15.3.1.15.0';
my $oid_temp_name = '1.3.6.1.4.1.890.1.15.3.26.1.2.1.2';
my $oid_temp_value = '1.3.6.1.4.1.890.1.15.3.26.1.2.1.3';
my $oid_model_vdsl = '1.3.6.1.4.1.890.1.2.1.1.2.0';
my $oid_version_vdsl = '1.3.6.1.4.1.890.1.2.1.1.1.0';
my $oid_version_serial = '1.3.6.1.4.1.890.1.2.1.1.10.0';
my $oid_vdsl_name = '1.3.6.1.4.1.890.1.2.1.1.8.0';
sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -T <type> [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=s" => \$warning,"warning=s" => \$warning,
	"c=s" => \$critical,"critical=s" => \$critical,
	"J" => \$json,	 "json" => \$json,
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-w","--warning","warning threshold (default: $warning %)" ]);
	push(@arr,["-c","--critical","critical threshold (default: $critical %) "]);
	push(@arr,["-T","--type","Type of test (default: cpu)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}

#si l'host ne répond pas on blocke le script
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if ($Sysoid_value ==-1)
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
if($type eq 'cpu')
{
	my $value = ESIA::SNMP::get($snmp,"$oid_cpu_load");
	if($value =~ m/noSuch/i)
	{
		print "SNMP ERROR: bad cpu value returned : $value\n";
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
	my $state='OK';
	my $compare_value = "< $warning";
	if($value >= $critical)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$state ="CRITICAL";
		$compare_value = "> $critical";
	}
	elsif($value >= $warning)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$state ="WARNING";
		$compare_value = "> $warning";
	}
	print "$state : CPU used $value% $compare_value|cpu=$value \n";
}
elsif($type eq 'mem')
{
	my $value = ESIA::SNMP::get($snmp,"$oid_memory");
	if($value =~ m/noSuch/i)
	{
		print "SNMP ERROR: bad cpu value returned : $value\n";
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
	my $state='OK';
	my $compare_value = "< $warning";
	if($value >= $critical)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$state ="CRITICAL";
		$compare_value = "> $critical";
	}
	elsif($value >= $warning)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$state ="WARNING";
		$compare_value = "> $warning";
	}
	print "$state : RAM used $value% $compare_value|ram=$value \n";
}
elsif($type eq 'flash')
{
	my $value = ESIA::SNMP::get($snmp,"$oid_flash");
	if($value =~ m/noSuch/i)
	{
		print "SNMP ERROR: bad cpu value returned : $value\n";
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
	my $state='OK';
	my $compare_value = "< $warning";
	if($value >= $critical)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$state ="CRITICAL";
		$compare_value = "> $critical";
	}
	elsif($value >= $warning)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$state ="WARNING";
		$compare_value = "> $warning";
	}
	print "$state : Flash used $value% $compare_value|flash=$value \n";
}
elsif($type eq 'session')
{
	my $value = ESIA::SNMP::get($snmp,"$oid_session");
	if($value =~ m/noSuch/i)
	{
		print "SNMP ERROR: bad cpu value returned : $value\n";
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
	my $state='OK';
	my $compare_value = "< $warning";
	if($value >= $critical)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$state ="CRITICAL";
		$compare_value = "> $critical";
	}
	elsif($value >= $warning)
	{
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$state ="WARNING";
		$compare_value = "> $warning";
	}
	print "$state : Number of active session:  $value $compare_value|session=$value \n";
}
elsif($type eq 'temp')
{
	my $result = $snmp->get_table( $oid_temp_name);
    foreach my $key ( keys %$result)
    {
		#on récupère l'index
	   	if($key=~ /([0-9]*)$/ )
	   	{
	   		my $id = $1;
	   		my $name = $$result{$key};
	   		if($name eq 'CPU' || $name eq 'cpu')
	   		{
	   			my $state='OK';
				my $compare_value = "< $warning";
				my $value = ESIA::SNMP::get($snmp,"$oid_temp_value.$1");
				if($value =~ m/noSuch/i)
				{
					print "SNMP ERROR: bad temperature value returned : $value\n";
					$snmp->close;
					exit $ERRORS{"UNKNOWN"};
				}
				if($value >= $critical)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					$state ="CRITICAL";
					$compare_value = "> $critical";
				}
				elsif($value >= $warning)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
					$state ="WARNING";
					$compare_value = "> $warning";
				}
				print "$state : temperature is $value $compare_value|temp=$value \n";
				last;
	   		}
	   	}
	}
}
elsif( $type eq 'inventory' )
{
	my @array;
	my $sysdescr = ESIA::SNMP::get($snmp,"$oid_sysDescr");
	if($sysdescr=~/VDSL/)
	{
		my $name = ESIA::SNMP::get($snmp,"$oid_vdsl_name");
		my $model = ESIA::SNMP::get($snmp,"$oid_model_vdsl");
		my $firmware = ESIA::SNMP::get($snmp,"$oid_version_vdsl");
		my $serial = ESIA::SNMP::get($snmp,"$oid_version_serial");
		my %hash = ('name' => "$name $model",'manufacturer'=>'ZyXEL Communications Corp.', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
		push(@array, \%hash);
	}
	else
	{
		my $model = ESIA::SNMP::get($snmp,"$oid_model");
		my $firmware = ESIA::SNMP::get($snmp,"$oid_version");
		my $serial = ESIA::SNMP::get($snmp,"$oid_serial");
		my $hw = ESIA::SNMP::get($snmp,"$oid_hw_version");
		if($hw =~ m/noSuch/i)
		{
			$hw='';
		}
		my %hash = ('name' => "ZyXEL $model",'manufacturer'=>'ZyXEL Communications Corp.', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>$hw, 'firmwareVersion' => $firmware);
		push(@array, \%hash);
	}
	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
else
{
	print ("UNKNOWN: Incorrect or missed option -T, --type\n");
	$return = $ERRORS{'UNKNOWN'};
}

exit($return);
