#!/usr/bin/php
<?php
$DEFAULT_TIMEOUT = 50;
$TIME_SLEEP = 0.5;
$DATA_DIR = '/etc/esia/innovaphone/data';
$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

//Récupération et vérifications des paramètres.
$type_list = array('recordings','nb_records','first_record');

$params = new Parameters();
$params->addOpt('','h','help',NULL,'print this help');
$params->addOpt(':','u','uri','<url>','Powershell daemon url (http://127.0.0.1:2082 by default)');
$params->addOpt(':','H','host','<host>','Host IP ou FQDN');
$params->addOpt(':','U','user','<login>','Login or username');
$params->addOpt(':','P','password','<password/pathfile>','password or path for pubkey, sharedkey or special authentication options');
$params->addOpt(':','A','authfile','<filepath>','authentication file filepath form set default user, password parameters');
$params->addOpt(':','T','type','<test_type>','To specifie test type');
$params->addOpt(':','t','timeout','<second>','Timeout for script usage',$DEFAULT_TIMEOUT);
$params->addOpt(':','m','match','<regex>','To match specific value');
$params->addOpt(':','p','period','<seconds>','Interval of test between now and last <seconds> seconds');
$params->addOpt(':','w','warning','<treshold>','warning treshold');
$params->addOpt(':','c','critical','<treshold>','critical treshold');

$params->addOpt('','W','warning_ret','','if not OK, return WARNING');
$params->addOpt('','C','critical_ret','','if not OK, return CRITICAL');
$params->addOpt(':','f','first','<timestamp>','set Install date for type first_record');

$params->addUsage('-H <host> -U <user> -P <password -T <type> -w <treshold> -c <treshold>');
$params->addUsage('-H <host> [-U <user>] -A <authfile> -T <type> -w <treshold> -c <treshold>');

if( $params->getParam('help') )
{
    $params->print_help();
    exit($INT_STATUS['UNKNOWN']);
}

foreach(array('type') as $key)
{
    $param = $params->getParam($key);
    $params_opt_str = $params->getParamOptionsString($key);
    if( $param === NULL )
    {
        echo "Missing required option $params_opt_str\n\n";
        $params->print_usage();
        exit($INT_STATUS['UNKNOWN']);
    }
}

$o_type = $params->getParam('type');
if(!in_array($o_type,$type_list))
{
    $params_opt_str = $params->getParamOptionsString('type');
    $str = "Incorrect value $params_opt_str=$o_type.";
    $str.= " This value must be : ".implode(", ",$type_list);
    exit_state('UNKNOWN',$str);
}

if( $o_type != 'first_record' )
{
    foreach(array('warning','critical') as $key)
    {
        $param = $params->getParam($key);
        $params_opt_str = $params->getParamOptionsString($key);
        if( $param === NULL )
        {
            echo "Missing required option $params_opt_str\n\n";
            $params->print_usage();
            exit($INT_STATUS['UNKNOWN']);
        }
    }
}

$o_authfile_path = $params->getParam('authfile');
if(!empty($o_authfile_path))
{
    $params_opt_str = $params->getParamOptionsString('authfile');
    if( !file_exists($o_authfile_path) )
    {
        exit_state('UNKNOWN',"Unable to found $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = file_get_contents($o_authfile_path);
    if( empty($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to open $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = json_decode($auth_file_data,TRUE);
    if( !is_array($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to parse $params_opt_str=$o_authfile_path");
    }
}

$auth_params = array();
foreach(array('host','user','password') as $key)
{
    $auth_params[$key] = $params->getParam($key,@$auth_file_data[$key]);
    $params_opt_str = $params->getParamOptionsString($key);
    if( empty($auth_params[$key]))
    {
        exit_state('UNKNOWN',"Missing required option $params_opt_str or authfile parameter $key");
    }
}

//Authparam database
if( !isset($auth_file_data['database']))
{
    exit_state('UNKNOWN',"Missing required authfile parameter database");
}
else
{
    $auth_params['database'] = $auth_file_data['database'];
}

//Au cas ou un $IP serait passé pour le confile en plus du host
if( strpos('$IP',$auth_params['host']) !== FALSE && preg_match('/(\d{1,3}\.\d{1,3}.\d{1,3}\.\d{1,3})/',basename($o_authfile_path),$matches))
{
        if (filter_var($matches[1], FILTER_VALIDATE_IP))
        {
                $auth_params['host'] = str_replace('$IP', $matches[1], $auth_params['host']);
        }
}

//Armement timeout
declare(ticks = 1);
function sig_handler($signo)
{
        global $timeout;
        exit_state("UNKNOWN","Timeout after $timeout seconds");
}

$timeout = $params->getParam('timeout');
pcntl_alarm($timeout);
pcntl_signal(SIGALRM, "sig_handler");

// Actions en fonction du type
$user_auth_list[] = array($auth_params['user'],$auth_params['password']);
$user_auth_list[] = array('root',$auth_params['password']);
$user_auth_list[] = array('postgres','');

$first = array_shift($user_auth_list);
$host_user = $first[0];
$host_pass = $first[1];

try
{
    $shell = ssh_shell_open($auth_params['host'],$host_user,$host_pass);
}
catch(Exception $ex)
{
    exit_state('UNKNOWN',$ex->getMessage());
}

ssh_shell_multi_su($shell,$user_auth_list,$TIME_SLEEP);
ssh_shell_clear($shell);

$warning = $params->getParam('warning');
$critical = $params->getParam('critical');
$type = $params->getParam('type');

if( $type == 'recordings' )
{
    foreach(array('period') as $key)
    {
        $param = $params->getParam($key);
        $params_opt_str = $params->getParamOptionsString($key);
        if( $param === NULL )
        {
            echo "Missing required option $params_opt_str\n\n";
            $params->print_usage();
            exit($INT_STATUS['UNKNOWN']);
        }
    }
    try
    {
        $period = $params->getParam('period');
        $period = parseDate($period);
    }
    catch(Exception $ex)
    {
        exit_state('UNKNOWN',"Unable to parse date $period");
    }

    validate_treshold_calcul_min($warning,$critical,0);
    $date_format = 'Y-m-d H:i:s';
    //$ret = ssh_shell_oneline_cmd($shell,"id",1,$TIME_SLEEP); var_dump($ret);

    $match = $params->getParam('match');
    $period_ms = ($period*1000);

    $sql = "SELECT caller,caller_num,called,called_num,duration,name,size,created,modified,incomplete FROM recordings";
    $sql.= " INNER JOIN files ON(recordings.file = files.id)";
    $sql.= " WHERE files.modified > $period_ms";
    if(!empty($match))
    {
        $sql.= " AND (caller ~* '$match' OR called ~* '$match')";
    }
    $sql.= " ORDER BY files.modified DESC";

   // $output = ssh_shell_postgres($shell,'voipbeta2.rvoip.eu_Recordings',$sql,$TIME_SLEEP);

    $output = ssh_shell_postgres($shell,$auth_params['database'],$sql,$TIME_SLEEP);
    
    $nb = count($output);
    $since = date($date_format,$period);
    $now = date($date_format);

    $result = calcul_min_state($warning,$critical,$nb,' calls');
    $str = "Found $nb records between $since and now ($now)";
    if( !empty($match) )
    {
        $str.= " for users '$match'";
    }
    $str.= $result['reason']."\n";


    foreach(array_slice($output,0,5) as $row)
    {
        if( $row['incomplete'] == 'f' )
        {
            $completed = 'yes';
        }
        else
        {
            $completed = 'no';
        }

        $str.= sprintf("%s from %s(%s) to %s(%s) time:%s size:%s done:%s\n",
            date($date_format,intval($row['modified']/1000)),
            $row['caller'],
            $row['caller_num'],
            $row['called'],
            $row['called_num'],
            $row['duration'].'s',
            intval($row['size']/1024).'K',
            $completed
        );
    }
    exit_state($result['state'],$str);
}
elseif( $type == 'nb_records' )
{
    validate_treshold_calcul_max($warning,$critical,0);
    $date_format = 'Y-m-d H:i:s';
    //$ret = ssh_shell_oneline_cmd($shell,"id",1,$TIME_SLEEP); var_dump($ret);

    $match = $params->getParam('match');

    $sql = "SELECT count(*) FROM recordings";
    $sql.= " INNER JOIN files ON(recordings.file = files.id)";
    $from_host_data = ssh_shell_postgres($shell,$auth_params['database'],$sql,$TIME_SLEEP);
    $from_host_data = array_shift($from_host_data);
    $from_host_data['time'] = time();

    //Calcul du nom du fichier temporaire
    $uniq_ident =  md5(json_encode($auth_params)."$type");
    $filepath = sys_get_temp_dir() .'/'. basename(__FILE__) . $uniq_ident;

    //chargement des données sauvées sur le disque
    $from_disk_data = array();
    if( is_file($filepath) )
    {
        $from_disk_data = @file_get_contents($filepath);
        $from_disk_data = json_decode($from_disk_data,TRUE);
        if( $from_disk_data == FALSE )
        {
            $from_disk_data = array();
        }
    }

    //sauvegarde des données sur le disques
    if( !@file_put_contents($filepath,json_encode($from_host_data)))
    {
        exit_state('UNKNOWN',"Unable to save data into tmp file $filepath");
    }

    if(empty($from_disk_data))
    {
        $calculed_time = 0;
        $last_value = $from_host_data['count'];
        $current_value = $from_host_data['count'];
        $calculed_diff = 0;

        $str = "Found $current_value records ($calculed_diff/$calculed_time minutes). FIRST EXEC/NO DATA";
        $state = 'OK';
    }
    else
    {
        $calculed_time = round(($from_host_data['time'] - $from_disk_data['time'])/60,1);
        $last_value = $from_disk_data['count'];
        $current_value = $from_host_data['count'];
        $calculed_diff = $from_host_data['count'] - $from_disk_data['count'];
        if( $calculed_diff < 0 )
        {
            $calculed_diff = $from_disk_data['count'] - $from_host_data['count'];
            $result = calcul_max_state($warning,$critical,$calculed_diff);
            $str = "Found $current_value records (-$calculed_diff/$calculed_time minutes).".$result['reason'];
            $state = $result['state'];           
        }
        elseif( $calculed_diff > 0 )
        {
            $str = "Found $current_value records (+$calculed_diff/$calculed_time minutes). Number of records increases.";
            $state = 'OK';
        }
        else
        {
            $str = "Found $current_value records ($calculed_diff/$calculed_time minutes). Identical number of records";
            $state = 'OK';
        }
    }
    exit_state($state,$str);
}
elseif( $type == 'first_record' )
{
    if( $params->getParam('critical_ret') )
    {
        $error_state = 'CRITICAL';
    }
    elseif($params->getParam('warning_ret'))
    {
         $error_state = 'WARNING';
    }
    else
    {
        $params_opt_str = $params->getParamOptionsString('warning_ret');
        $params_opt_str.= ' or '; 
        $params_opt_str.= $params->getParamOptionsString('critical_ret');
        echo "Missing required option $params_opt_str\n";
        exit($INT_STATUS['UNKNOWN']);
    }

    if(empty($params->getParam('first')))
    {
        $install_date = FALSE;
    }
    else
    {
        $install_date = $params->getParam('first');
    }

    if(!is_dir($DATA_DIR))
    {
        $ret = @mkdir($DATA_DIR,TRUE);
        if( !$ret )
        {
            exit_state('UNKNOWN',"Unable to access or create directory $DATA_DIR");
        }
    }
    $filepath = "$DATA_DIR/".$auth_params['host'].'.json';
    
    $date_format = 'Y-m-d H:i:s';

    $sql = 'SELECT files.modified FROM recordings';
    $sql.= " INNER JOIN files ON(recordings.file = files.id)";
    $sql.= " ORDER BY files.modified ASC LIMIT 1";

    $from_host_data = ssh_shell_postgres($shell,$auth_params['database'],$sql,$TIME_SLEEP);
    $from_host_data = array_shift($from_host_data);
    if( empty($from_host_data) || empty($from_host_data['modified']) )
    {
	exit_state('UNKNOWN',"Unable to get data from database ".$auth_params['database']." please check database name into configfile and innovaphone credentials");
    }
    $from_host_data['modified'] = round($from_host_data['modified']/1000);

    $from_disk_data = FALSE;
    if( !$install_date && !is_file($filepath)) // FIRST EXEC
    {
        //sauvegarde des données sur le disques
        if( !@file_put_contents($filepath,json_encode($from_host_data)))
        {
            exit_state('UNKNOWN',"Unable to save data into tmp file $filepath");
        }
       
        exit_state('OK','First run, recording date : '.date($date_format,$from_host_data['modified']));
    }
    else
    {
        if( !$install_date )
        {
            $from_disk_data = @file_get_contents($filepath);
            $from_disk_data = json_decode($from_disk_data,TRUE);
            if( $from_disk_data == FALSE )
            {
                exit_state('UNKNOWN',"Unable to read or access file $filepath");
            }
        }
        else
        {
            $from_disk_data['modified'] = $install_date;
        }

        if ($from_disk_data['modified'] < $from_host_data['modified'])
        {
            $str_set_date = ""; //"\nUse -f, --first=".$from_host_data['modified']." to use another install date";
            exit_state($error_state,'Install date is '.date($date_format,$from_disk_data['modified']).". Current date into db too recent : ".date($date_format,$from_host_data['modified']).$str_set_date);
        }
        else
        {
            exit_state('OK','Install date is '.date($date_format,$from_disk_data['modified']).". Current date into DB is correct : ".date($date_format,$from_host_data['modified']));
        }
    }
}

ssh_shell_close($shell);

function validate_treshold_calcul_min($warning,$critical,$min_treshold=NULL)
{
    if( $warning < $critical )
    {
        exit_state('UNKNOWN',"Warning treshold $warning must be lower than critical treshold $critical");
    }
    if( $min_treshold != NULL && $warning < $min_treshold )
    {
        exit_state('UNKNOWN',"Tresholds must be lowest than $min_treshold");
    }
}

function validate_treshold_calcul_max($warning,$critical,$max_treshold=NULL)
{
    if( $warning > $critical )
    {
        exit_state('UNKNOWN',"Warning treshold $warning must be bigger than critical treshold $critical");
    }
    if( $max_treshold != NULL && $critical > $max_treshold )
    {
        exit_state('UNKNOWN',"Tresholds must be lowest than $max_treshold");
    }
}

function calcul_min_state($warning,$critical,$value,$unit='')
{
    global $INT_STATUS;

    $ret = array();
    if( $value < $critical )
    {
        $ret['state'] = 'CRITICAL';
        $ret['reason'] = " < $critical".$unit;
    }
    else if( $value < $warning )
    {
        $ret['state'] = 'WARNING';
        $ret['reason'] = " < $warning".$unit;
    }
    else
    {
        $ret['state'] = 'OK';
        $ret['reason'] = " >= $warning".$unit;
    }
    return $ret;
}

function calcul_max_state($warning,$critical,$value,$unit='')
{
    global $INT_STATUS;

    $ret = array();
    if( $value >= $critical )
    {
        $ret['state'] = 'CRITICAL';
        $ret['reason'] = " > $critical".$unit;
    }
    else if( $value >= $warning )
    {
        $ret['state'] = 'WARNING';
        $ret['reason'] = " > $warning".$unit;
    }
    else
    {
        $ret['state'] = 'OK';
        $ret['reason'] = " < $warning".$unit;
    }
    return $ret;
}

function parseDate($start)
{
    $start = strtolower($start);
    if(preg_match('/^[-]?(\d+)([hdwmy]?)$/',$start,$matches))
    {
        if(!empty($matches[2]))
        {
            $matches[2]=strtoupper($matches[2]);
            switch($matches[2])
            {
                case 'H' :
                    $since = time() - $matches[1] * 3600;
                    break;
                case 'D' :
                    $since = time() - $matches[1] * 86400;
                    break;
                case 'W' :
                    $since = time() - $matches[1] * 604800;
                    break;
                case 'M' :
                    $date = new DateTime();
                    $date->sub(new DateInterval('P'.$matches[1].$matches[2]));
                    $since=$date->format('U');
                    break;
                case 'Y' :
                    $date = new DateTime();
                    $date->sub(new DateInterval('P'.$matches[1].$matches[2]));
                    $since=$date->format('U');
                    break;
            }
        }
        else $since = time() - $matches[1];
    }
    else throw new Exception();
    return $since;
}

function secondsToPeriod($seconds)
{
    $seconds = intval($seconds);
    $dtF = new DateTime("@0");
    $dtT = new DateTime("@$seconds");
    return $dtF->diff($dtT)->format('%a days, %h hours, %i minutes and %s seconds');
}

function ssh_shell_open($host,$host_user,$host_pass)
{
    $port = 22;
    $exp = explode(':',$host);
    $addr = $exp[0];
    if(isset($exp[1]))
    {
        $port = $exp[1];
    }
    $con=@ssh2_connect($addr, $port);
    if( $con === FALSE )
    {
        throw new Exception("Unable to connect to $host:$port");
    }
    $ret = ssh2_auth_password($con, $host_user, $host_pass);
    if( $ret === FALSE )
    {
        throw new Exception("Unable to authenticate with user $host_user");
    }
    $shell=ssh2_shell($con, 'bash');
    if( $shell === FALSE )
    {
        throw new Exception("Unable to open interactive shell bash");
    }
    $ret = stream_set_blocking($shell, true);
    if( $ret === FALSE )
    {
        throw new Exception("Unable to set blocking stream");
    }
    return $shell;
}

function ssh_shell_multi_su($shell,$user_auth_list,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    foreach($user_auth_list as $auth)
    {
        $user = $auth[0];
        $pass = $auth[1];
        fwrite( $shell, "su $user".PHP_EOL);
        time_nanosleep($seconds,$nanoseconds);
        if(!empty($pass))
        {
            fwrite( $shell, $pass.PHP_EOL);
            time_nanosleep($seconds,$nanoseconds);
        }
    }

    fwrite( $shell, 'echo SU_END_SU_END > /dev/null'.PHP_EOL);
    while($row=fgets($shell,4096))
    {
        flush();
        if( preg_match('/^su:.*/',$row) )
        {
            throw new Exception("Connection error : $row");
        }
        if ( strpos("$row","echo SU_END_SU_END")!== FALSE)
        {
            break;
        }
    }
}

function ssh_shell_clear($shell)
{
    fwrite( $shell, 'echo CLEAR_CLEAR'.PHP_EOL);
    while($row=fgets($shell,4096))
    {
        if ( strpos("$row",'echo CLEAR_CLEAR') === FALSE && strpos("$row","CLEAR_CLEAR")!== FALSE)
        {
            break;
        }
    }
}

function ssh_shell_postgres($shell,$dbname,$sql,$sleep=0.1)
{
    $cmd = 'psql -A -d '.$dbname.' -c "'.$sql.'"';
    $output = ssh_shell_cmd($shell,$cmd,$sleep);
    $header = array_shift($output);
    $header_col = explode('|',$header);
    $ret = array();
    foreach($output as $row)
    {
        if( !empty($row) && ( count($header_col) == 1 || strpos($row,'|')))
        {
            $row_col = explode('|',$row);
            $ret_row = array();
            foreach($row_col as $idx => $col)
            {
                $ret_row[$header_col[$idx]] = trim($col);
            }
            $ret[] = $ret_row;
        }
    }
    if( count($header_col) == 1)
    {
        array_pop($ret);
    }
    return $ret;
}

function ssh_shell_oneline_cmd($shell,$cmd,$nb=1,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;
    time_nanosleep($seconds,$nanoseconds);
    fwrite( $shell, $cmd .PHP_EOL);
    flush();

    //On recherche l'appel a la ligne de commande ci-dessus
    do
    {
        $row=fgets($shell,4096);
    }while(strpos("$row","$cmd") === FALSE);
    $row = '';
    //On attends puis on récupére la sortie
    time_nanosleep($seconds,$nanoseconds);
    for($i=0;$i<$nb;$i++)
    {
        $row.= fgets($shell,4096);
    }
    return $row;
}

function ssh_shell_read_file($shell,$file,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    $row=trim(ssh_shell_oneline_cmd($shell,"wc -l $file",$sleep));
    $size = -1;
    if( preg_match('/^(\d+) \S+$/',$row,$matches))
    {
        $size = $matches[1];
    }
    if( $size != -1 )
    {
        fwrite( $shell,"cat $file" .PHP_EOL);
        time_nanosleep($seconds,$nanoseconds);
        flush();
        $output = '';
        for($i=0;$i<=$size;$i++)
        {
            $row=fgets($shell,4096);
            $output.=$row;
        }
        return $output;
    }
    else
    {
        return FALSE;
    }
}

function ssh_shell_cmd($shell,$cmd,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    $file_name = basename(__FILE__);
    $file_name = uniqid($file_name).'.tmp';
    $file_name = "/tmp/$file_name";
    fwrite( $shell, "echo START${file_name}START > $file_name" .PHP_EOL);
    fwrite( $shell, "$cmd >> $file_name" .PHP_EOL);
    time_nanosleep($seconds,$nanoseconds);
    fwrite( $shell, "echo END${file_name}END >> $file_name" .PHP_EOL);
    fwrite( $shell, "cat $file_name" .PHP_EOL);

    $output = array();
    while($row=fgets($shell,4096)){
        flush();
        if ( strpos("$row","END${file_name}END")!== FALSE)
        {
            break;
        }
        $output[] = trim($row);
        if ( strpos("$row","START${file_name}START")!== FALSE)
        {
            $output = array(); // DEBUT : On efface le contenu précédent et on récupére la suite
        }
    }
    time_nanosleep($seconds,$nanoseconds);
    fwrite( $shell, "rm $file_name" .PHP_EOL);
    $row=fgets($shell,4096);
    flush();
    time_nanosleep($seconds,$nanoseconds);
    return $output;
}

function ssh_shell_close($shell)
{
        ssh_shell_oneline_cmd($shell,"exit",1);
        ssh_shell_oneline_cmd($shell,"exit",1);
        ssh_shell_oneline_cmd($shell,"exit",1);

//    fclose($shell);
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null )
        {
          $msg=trim($msg);
          echo "$state : $msg\n";
        }
        if( $usage ) print_usage(true);
        exit($ret);
}

function calcul_state($a,$b=null){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];
        $PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

        if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
        if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

        foreach($PRIORITIES_STATES as $value){
                $value = $INT_STATUS[$value];

                if( $value == $a || $value == $b ) return $value;
        }
        return $INT_STATUS["UNKNOWN"];
}

class Parameters
{
    private $data = array();
    private $options = NULL;
    private $params = NULL;
    private $usage = array();

    function __construct()
    {
    }

    function addUsage($row)
    {
        $this->usage[] = $row;
    }

    static function getArrayBiggestWordSize($arr){
    	$biggest_size = 0;
            foreach($arr as $word){
                    $cur_size = strlen($word);
                    if( $biggest_size < $cur_size){
                            $biggest_size = $cur_size;
                    }
            }
    	return $biggest_size;
    }

    function print_usage($alone=TRUE)
    {
        $script = basename(__FILE__);
        $left = 'usage : ';
        $size_left = strlen($left);

        foreach($this->usage as $row)
        {
            echo str_pad($left,$size_left) . "$script $row [OPTIONS]\n";
            $left = '';
        }
        echo"\n";
        if($alone)
        {
            $row = "--help for more help";
            echo str_pad($left,$size_left) . "$script $row\n";
        }
    }

    function print_help()
    {
        $this->print_usage(FALSE);
        $opt = array();
        $desc = array();
        foreach($this->data as $key => $row)
        {
            $str = "\t-" . $this->getParamOptionsString($key);
            if( !empty($row['value']) )
            {
                $str.= '=<'.trim($row['value'],'<>').">";
            }
            $opt[] = $str;
            $desc[] = $row['descr'];
        }
        $size_left = self::getArrayBiggestWordSize($opt);
        foreach($opt as $idx => $left)
        {
            echo str_pad($left,$size_left) . "  ".$desc[$idx]."\n";
        }
    }

    public function getParamOptionsString($long_opt)
    {
        $row = $this->data[$long_opt];
        return $row['short'] . ", --" . $row['long'];
    }

    public function addOpt($optional,$short_opt,$long_opt,$value_desc,$descr,$default = NULL)
    {
        $row = array();
        $row['optional'] = $optional;
        $row['short'] = $short_opt;
        $row['long'] = $long_opt;
        $row['value'] = $value_desc;
        $row['descr'] = $descr;
        $row['default'] = $default;
        $this->data[$long_opt] = $row;
    }

    private function getopt()
    {
        if( $this->options != NULL )
        {
            return;
        }

        $short_opts = '';
        $long_opts = array();

        foreach($this->data as $row)
        {
            $short_opts.= $row['short'].$row['optional'];
            $long_opts[] = $row['long'].$row['optional'];
        }
        $this->options = getopt($short_opts, $long_opts);
        $this->params = array();
        foreach($this->data as $row)
        {
            $req_opt = $row['optional'];
            $long_opt = $row['long'];
            $short_opt =  $row['short'];
            $default_opt = $row['default'];
            $key_name = $row['long'];

            if( $req_opt == '' )
            {
                if( isset($this->options[$long_opt]) || isset($this->options[$short_opt] ) )
                {
                    $value = TRUE;
                }
                else
                {
                    $value = FALSE;
                }
            }
            elseif( isset($this->options[$long_opt]))
            {
                $value = $this->options[$long_opt];
            }
            elseif( isset($this->options[$short_opt] ))
            {
                $value = $this->options[$short_opt];
            }
            else
            {
                continue;
                $value = $default_opt;
            }
            $this->params[$key_name] = $value;
        }
    }

    public function getParam($name,$default=NULL)
    {
        $this->getopt();
        if( $default !== NULL && $default !== '0' && empty($this->params[$name]) )
        {
            return $default;
        }
        elseif( !isset($this->params[$name]))
        {
            return NULL;
        }
        elseif($this->params[$name] !== '0' && empty($this->params[$name]))
        {
            return FALSE;
        }
        return $this->params[$name];
    }
}
?>

