#!/usr/bin/perl -w
############################## check_qnap #################

# Date : Jan 21 2015
# Author  : Nicolas Biersart
# Rewrite : Jan 30 2023 by Benjamin
# Help : esia-sa.com
#################################################################

use strict;
use lib "/usr/local/esia/plugins/lib/perl";
use Net::SNMP;
use Getopt::Long;
use RRDs;
use Data::Dumper;
use ESIA::SNMP;
use ESIA::PLUGIN;
use JSON;

my $PROGNAME='check_qnap.pl';
my $PROGDESC='check QNAP.
Reading Temperature information, Raid information and volume used, ram and fan exploited from qnap';
my $PROGVERSION='1.0';
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
my $return=0;
my ($opt_h,$opt_V,$opt_u);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical= 90;
my $type = "temp";


#OID QNAP
my $oid_cputemp = "1.3.6.1.4.1.24681.1.2.5.0";
my $oid_systemtemp = "1.3.6.1.4.1.24681.1.2.6.0";
my $oid_model = "1.3.6.1.4.1.24681.1.2.12.0";
my $oid_hostname = "1.3.6.1.4.1.24681.1.2.13.0";
my $oid_disk_index="1.3.6.1.4.1.24681.1.2.11.1.1";
my $oid_disk_descr="1.3.6.1.4.1.24681.1.2.11.1.2";
my $oid_disk_temp="1.3.6.1.4.1.24681.1.2.11.1.3";
my $oid_disk_status="1.3.6.1.4.1.24681.1.2.11.1.4";
my $oid_disk_model="1.3.6.1.4.1.24681.1.2.11.1.5";
my $oid_disk_capacity="1.3.6.1.4.1.24681.1.2.11.1.6";
my $oid_disk_smart="1.3.6.1.4.1.24681.1.2.11.1.7";

my $oid_raid_index= my $oid_vol_index="1.3.6.1.4.1.24681.1.2.17.1.1";
my $oid_raid_descr= my $oid_vol_descr="1.3.6.1.4.1.24681.1.2.17.1.2";
my $oid_raid_fs= my $oid_vol_fs="1.3.6.1.4.1.24681.1.2.17.1.3";
my $oid_raid_status= my $oid_vol_status="1.3.6.1.4.1.24681.1.2.17.1.6";

my $oid_new_raid_index="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.2";
my $oid_new_raid_size="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.3";
my $oid_new_raid_level="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.7";
my $oid_new_raid_status="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.5";

my $oid_raid_alert_status="1.3.6.1.4.1.24681.1.4.1.1.1.2.3.2.1.5";
my $oid_raid_alert_name="1.3.6.1.4.1.24681.1.4.1.1.1.2.3.2.1.8";

my $oid_mem_total="1.3.6.1.4.1.24681.1.2.2.0";
my $oid_mem_free="1.3.6.1.4.1.24681.1.2.3.0";

my $oid_vol_size="1.3.6.1.4.1.24681.1.2.17.1.4";
my $oid_vol_free="1.3.6.1.4.1.24681.1.2.17.1.5";


my $oid_fan_index="1.3.6.1.4.1.24681.1.2.15.1.1";
my $oid_fan_desc="1.3.6.1.4.1.24681.1.2.15.1.2";
my $oid_fan_speed="1.3.6.1.4.1.24681.1.2.15.1.3";



my $HdStatus = { 0 => 'ready',-4=>'unknown', -5 => 'noDisk',-6 => 'invalid',-9 => 'rwError'  };
my $json;
my @jsonOutput;
my $match='.*';
&Getopt::Long::config('bundling');
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"u"   => \$opt_u,	"unknown"    => \$opt_u,
	"m=s" => \$match,"match=s" => \$match,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
    "T=s" => \$type,    "Type=s" => \$type,
    "J" => \$json,      "json" => \$json,
);

# -V & --version print version
if ($opt_V)
{
    print "$PROGNAME version $PROGVERSION\n";
    exit $ERRORS{'OK'};
}
# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T temp');
    my @arr;
    push(@arr,["-T","--type","type of test [temp,ram,raid,vol,fan,inventory] (default: temp)" ]);
    push(@arr,["-m","--match","Match volume" ]);
    push(@arr,["-w","--warning","warning threshold temperature in Celsius (default: 80) or warning threshold temperature in Celsius (default: 50)" ]);
    push(@arr,["-c","--critical","critical threshold temperature in Celsius (default: 90) or critical threshold temperature in Celsius (default: 70) "]);
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
    exit $ERRORS{'OK'};
}

sub octet_conversion
{
	my ($size,$unit) = @_;
	my $octet_size = 0;
	if($unit eq 'TB')
	{
		$octet_size = $size * 1024 * 1024 * 1024 *1024;
	}
	elsif($unit eq 'GB')
	{
		$octet_size = $size * 1024 * 1024 * 1024;
	}
	elsif($unit eq 'MB')
	{
		$octet_size = $size * 1024 * 1024;
	}
	elsif($unit eq 'KB')
	{
		$octet_size = $size * 1024;
	}
	else
	{
		$octet_size = $size;
	}
	return $octet_size;
}
#
#MAIN
#
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $output = "";
my $graphdata = "|";
if($type eq 'temp')
{
    #Récupération des températures
    my $systemtemp =  ESIA::SNMP::get($snmp,$oid_systemtemp);
    $systemtemp =~ s/\n+//g;
    #Réaffectation des valeurs de critical et warning
    if($warning eq 80)
    {
        $warning=50;
    }
    if($critical eq 90)
    {
        $critical=70;
    }
    #vérifaction de la température système.
    if($systemtemp =~ /([0-9]*)\s+C/)
    {
        $graphdata.="system:$1";
	    if($1 >= $critical)
	    {
		    $output .= "CRITICAL: System temperature: $systemtemp > $critical C\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	    }
	    elsif($1 >= $warning)
	    {
		    $output .= "WARNING: System temperature: $systemtemp > $warning C\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
	    }
	    else
	    {
		    $output .= "OK: System temperature: $systemtemp < $warning C\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
	    }
    }
    else
    {
	    $output .= "OK: System temperature: noInfo\n";
    }
    my $cputemp = ESIA::SNMP::get($snmp,$oid_cputemp);
    $cputemp =~ s/\n+//g;
    #vérifaction de la température CPU.
    if($cputemp =~ /([0-9]*)\s+C/)
    {
        $graphdata.=",cpu:$1";
	    if($1 >= $critical)
	    {
		    $output .= "CRITICAL: CPU temperature: $cputemp > $critical C\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	    }
	    elsif($1 >= $warning)
	    {
		    $output .= "WARNING: CPU temperature: $cputemp > $warning C\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
	    }
	    else
	    {
		    $output .= "OK: CPU temperature: $cputemp < $warning C\n";
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
	    }
    }
    else
    {
	    $output .= "OK: CPU temperature: noInfo\n";
    }
    # Recupération et vérification des températures disques.
    my $result = $snmp->get_request($oid_model);
    $result = $snmp->get_table(Baseoid => $oid_disk_descr);
	foreach my $key ( keys %$result)
	{
		if($key=~ /([0-9]*)$/ )
		{
		    my $id =$1;
            my $disk_descr = $$result{$key};
		    my $disk_temp = ESIA::SNMP::get($snmp,"$oid_disk_temp.$id");
			my $e = $disk_temp;
		    #nettoyage des variables
		    $disk_descr =~ s/\n+//g;
		   	$disk_temp =~ s/\n+//g;
            if($disk_temp !~ /--\s?[C]\/--\s?[F]/)
            {
                if($disk_temp =~ /([0-9]*)\s+C/)
    			{
    			    $graphdata.=",hdd:$1";
		            if($1 >= $critical)
		            {
		                $output .= "CRITICAL: Disk $disk_descr temperature: $disk_temp > $critical C\n";
		                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		            }
		            elsif($1 >= $warning)
		            {
		                $output .= "WARNING: Disk $disk_descr temperature: $disk_temp > $warning C\n";
		                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		            }
		            else
		            {
		                $output .= "OK: Disk $disk_descr temperature: $disk_temp < $warning C\n";
		                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		            }
		      	}
            }
		}
	}

}
elsif($type eq 'raid')
{
    #DISK INFORMATION
    my $result = $snmp->get_request($oid_model);
    $result = $snmp->get_table(Baseoid => $oid_disk_descr);
    if (!defined($result))
    {
       print "DISK: na\n";
    }
    else
    {
	    foreach my $key ( keys %$result)
	    {
		    if($key=~ /([0-9]*)$/ )
		    {
			    my $id =$1;
			    my $state = "OK";
                my $disk_descr = $$result{$key};
			    my $disk_temp = ESIA::SNMP::get($snmp,"$oid_disk_temp.$id");
			    my $disk_status = ESIA::SNMP::get($snmp,"$oid_disk_status.$id");
			    my $disk_model = ESIA::SNMP::get($snmp,"$oid_disk_model.$id");
			    my $disk_capacity =  ESIA::SNMP::get($snmp,"$oid_disk_capacity.$id");
			    my $disk_smart = ESIA::SNMP::get($snmp,"$oid_disk_smart.$id");
			    #nettoyage des variables
			    $disk_descr =~ s/\n+//g;
			    $disk_model =~ s/\n+//g;
			    $disk_smart =~ s/\n+//g;
			    $disk_temp =~ s/\n+//g;
			    $disk_capacity =~ s/\n+//g;
                #INFO SMART
			    if($disk_smart =~ m/noSuchInstance/i)
			    {
				    $disk_smart = 'noInfo';
			    }
                elsif($disk_smart =~ m/GOOD/i || $HdStatus->{$disk_status} eq "noDisk")
                {
                    $state ="OK";
                }
                else
			    {
				    $state ="WARNING";
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			    }
                #INFO DISK
			    if($disk_status == -4 && $state ne "WARNING")
			    {
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
				    $state ="UNKNOWN";
			    }
			    elsif($disk_status == -6 || $disk_status == -9)
			    {
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
				    $state ="WARNING";
			    }
			    if( $HdStatus->{$disk_status} eq "noDisk" ){
				    print "$state: DISK $id : $disk_descr (Missing)\n";
			    }
			    else{
			    $output .= "$state: DISK $id : $disk_descr (Model: $disk_model Capacity: $disk_capacity) status is: $HdStatus->{$disk_status} (SMART:$disk_smart) \n";
			    }
		    }
	    }
    }
    #RAID INFORMATION
    $result = $snmp->get_table(Baseoid => $oid_raid_status);
    if (!defined($result))
    {
        $result = $snmp->get_table(Baseoid => $oid_new_raid_size);
        if (!defined($result))
        {
            print "RAID: na\n";
        }
	    foreach my $key ( keys %$result)
	    {
		    if($key=~ /([0-9]*)$/ )
		    {
                my $id =$1;
			    my $state = "OK";
                my $raid_size =  $$result{"$oid_new_raid_size.$id"};
                my $raid_level = ESIA::SNMP::get($snmp,$oid_new_raid_level.$id);
                my $raid_status = ESIA::SNMP::get($snmp,$oid_new_raid_status.$id);
                $raid_size = int($raid_size/1000/1000/1000);
			    #nettoyage des variables
			    $raid_status =~ s/\n+//g;
			    if( $raid_status ne "Ready" )
			    {
			        $output .= "CRITICAL: raid $id (level: $raid_level, capacity $raid_size GB) is $raid_status.\n";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			    }
			    else
			    {
			        $output .= "OK: raid $id (level: $raid_level, capacity $raid_size GB) is $raid_status.\n";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
			    }
            }
        }
    }
    else
    {
	    my $result_name = $snmp->get_table(Baseoid => $oid_raid_descr);
	    my $result_aname;
	    my $result_astatus;
	    foreach my $key ( keys %$result)
	    {
		    if($key=~ /([0-9]*)$/ )
		    {
                my $id =$1;
			    my $state = "OK";
			    my $raid_status =  $$result{"$oid_raid_status.$id"};
			    my $raid_desc = $$result_name{"$oid_raid_descr.$id"};
                if( !($raid_desc =~ /Single\s+Disk/) )
                {
			        if ( $raid_desc =~ /([Vv]olume )?(\w+)/ ){
				        $raid_desc = $2;
			        }

			        #nettoyage des variables
			        $raid_status =~ s/\n+//g;
			        if( $raid_status eq "Ready" )
			        {
			            $output .= "OK: Volume $id ($raid_desc) software raid status is $raid_status.\n";
				        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
			        }
			        elsif( $raid_status =~ /Rebuilding/ )
			        {
			            $output .= "WARNING: Volume $id ($raid_desc) software raid status is $raid_status.\n";
				        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			        }
			        elsif( $raid_status =~ /^degraded.*$/ )
			        {
			            $output .= "CRITICAL: Volume $id ($raid_desc) software raid status is $raid_status.\n";
			           	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			        }
			        elsif( $raid_status =~ /[Uu]nknown/ ){
				        if( !defined($result_aname)){
				             	$result_aname = $snmp->get_table(Baseoid => $oid_raid_alert_name);
                				$result_astatus = $snmp->get_table(Baseoid => $oid_raid_alert_status);
				        }
				        my $astatus = 'Unknown';
				        foreach my $key_alert ( keys %$result_aname){
					        if($key_alert=~ /([0-9]*)$/ ){
						        my $id_alert = $1;
						        if( $$result_aname{"$oid_raid_alert_name.$id_alert"} eq "$raid_desc" ){
							        $astatus = $$result_astatus{"$oid_raid_alert_status.$id_alert"};
						        }
					        }
				        }
				        if( $astatus =~ /[Ww]arning/ ){
					        $output .= "WARNING: Volume $id ($raid_desc) software raid status $raid_status and disk space alert status is $astatus\n";
					        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
				        }
				        else{
					        if( $opt_u ){
						        $output .= "IGNORED: Volume $id ($raid_desc) software raid status is $raid_status and disk space alert status is $astatus\n";
                          		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
					        }
					        else{
						        $output .= "UNKNOWN: Volume $id ($raid_desc) software raid status is $raid_status and disk space alert status is $astatus\n";
                                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});

					        }
				        }
			        }
			        else
			        {
			            $output .= "UNKNOWN: Volume $id ($raid_desc) software raid status is $raid_status.\n";
				        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
			        }
                }
		    }
	    }
    }
}
elsif($type eq 'ram')
{
    #VOLUME MEMORY INFORMATION
    my $vol_size = ESIA::SNMP::get($snmp,$oid_mem_total);
    my $vol_free = ESIA::SNMP::get($snmp,$oid_mem_free);

    my $vol_mb_used=0;
    my $octet_size=0;
    my $octet_free=0;

    if($vol_size =~ /([0-9\.]*) ([A-Z]*)/)
    {
	    $octet_size = octet_conversion($1,$2);
    }
    if($vol_free =~ /([0-9\.]*) ([A-Z]*)/)
    {
	    $octet_free = octet_conversion($1,$2);
    }
    my $octet_used = $octet_size - $octet_free;
    my $percent = ($octet_used * 100)/ $octet_size;
    $percent = sprintf("%.2f", $percent);
    $graphdata.="percent:$percent,used:$octet_used,size:$octet_size";
    $vol_size = $octet_size/1024/1024; #convert to MB
    my $vol_used = $octet_used/1024/1024;
    if($percent > $critical)
    {
	    $output .= "CRITICAL: RAM : used $percent% ($vol_used MB/ $vol_size MB) > $critical\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
    }
    elsif($percent > $warning)
    {
	    $output .= "WARNING: RAM : used $percent% ($vol_used MB/ $vol_size MB) > $warning\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
    }
    else
    {
	    $output .= "OK: RAM : used $percent% ($vol_used MB/ $vol_size MB)\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
    }

}
#VOLUME DISK INFORMATION
elsif($type eq 'vol')
{
    my $result = $snmp->get_request($oid_model);
    $result = $snmp->get_table(Baseoid => $oid_vol_descr);
    if (!defined($result))
    {
	    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	    $snmp->close;
	    exit $ERRORS{"UNKNOWN"};
    }
    foreach my $key ( keys %$result)
    {
        if($key=~ /([0-9]*)$/ )
        {
		    my $id =$1;
		    my $state = "OK";
		    my $vol_descr =  $$result{$key};
		    if( $vol_descr =~/$match/)
		    {
				my $vol_fs =  ESIA::SNMP::get($snmp,"$oid_vol_fs.$id");
				my $vol_size =  ESIA::SNMP::get($snmp,"$oid_vol_size.$id");
				my $vol_free =  ESIA::SNMP::get($snmp,"$oid_vol_free.$id");
				my $vol_mb_used=0;
				my $octet_size=0;
				my $octet_free=0;
				if($vol_size =~ /([0-9\.]*) ([A-Z]*)/)
				{
					$octet_size = octet_conversion($1,$2);
				}
				if($vol_free =~ /([0-9\.]*) ([A-Z]*)/)
				{
					$octet_free = octet_conversion($1,$2);
				}
				if($octet_size !=0)
				{
				    #print "$octet_size $octet_free\n";
				    my $octet_used = $octet_size - $octet_free;
				    my $percent = ($octet_used * 100)/ $octet_size;
				    $percent = sprintf("%.2f", $percent);
				    $graphdata.="percent:$percent,used:$octet_used,size:$octet_size";
				    $vol_size = $octet_size/1024/1024/1024; #convert to GB
				    my $vol_used = $octet_used/1024/1024/1024;
				    if($percent > $critical)
				    {
					    $output .= "CRITICAL: Volume $id : $vol_descr ($vol_fs) used $percent% ($vol_used GB/ $vol_size GB) > $critical\n";
					    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
				    }
				    elsif($percent > $warning)
				    {
					    $output .= "WARNING: Volume $id : $vol_descr ($vol_fs) used $percent% ($vol_used GB/ $vol_size GB) > $warning\n";
					    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
				    }
				    else
				    {
					    $output .= "OK: Volume $id : $vol_descr ($vol_fs) used $percent% ($vol_used GB/ $vol_size GB) \n";
					    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
				    }
				}
			}
	    }
    }
    if( $output eq "")
    {
    	 $output .= "UNKNOWN: Volume $match not found \n";
    	 $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
    }
}
elsif($type eq 'fan')
{
    #Réaffectation des valeurs de critical et warning
    if($warning eq 80)
    {
        $warning=-1;
    }
    if($critical eq 90)
    {
        $critical=0;
    }
    my $result = $snmp->get_request($oid_model);
    $result = $snmp->get_table(Baseoid => $oid_fan_desc);
    if (!defined($result))
    {
	    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	    $snmp->close;
	    exit $ERRORS{"UNKNOWN"};
    }
    foreach my $key ( keys %$result)
	{
		if($key=~ /([0-9]*)$/ )
		{
            my $id =$1;
            my $fan_desc = $$result{"$oid_fan_desc.$id"};
            my $result_speed = $snmp->get_table(Baseoid => $oid_fan_speed);
            my $fan_speed = $$result_speed{"$oid_fan_speed.$id"};
            $fan_speed =~ s/\s[A-Z]*//gm;
            #vérifaction de la température système.

	        if($fan_speed <= $critical)
	        {
		        $output .= "CRITICAL: Name : $fan_desc Speed : $fan_speed RPM\n";
		        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	        }
	        elsif($fan_speed <= $warning)
	        {
		        $output .= "WARNING: Name : $fan_desc Speed : $fan_speed RPM\n";
		        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
	        }
	        else
	        {
		        $output .= "OK: Name : $fan_desc Speed : $fan_speed RPM\n";
		        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
	        }
        }
    }
}
elsif($type eq 'inventory')
{
    #On récupère le nom et le modèle
    my $model =  ESIA::SNMP::get($snmp,$oid_model);
    my $host = ESIA::SNMP::get($snmp,$oid_hostname);
    $host =~ s/\n+//g;
    $output .= "Model: $model Hostname: $host\n";
    my @hash = ('type' => 'NAS', 'name_component' => "$model" , 'description_component' => "$model", 'name' => "$host",
                                    'description' => "$host",'model' => "$model",'version' => "1",'serial_number' => "",
                                    'other'=>  "");
    push(@jsonOutput, \@hash);
    #On recupère le modèle du hdd
    my $result = $snmp->get_request($oid_model);
    $result = $snmp->get_table(Baseoid => $oid_disk_descr);

    if (defined($result))
    {
	    foreach my $key ( keys %$result)
	    {
		    if($key=~ /([0-9]*)$/ )
		    {
			    my $id =$1;
                my $disk_descr = $$result{$key};
			    my $disk_model = ESIA::SNMP::get($snmp,"$oid_disk_model.$id");
			    my $disk_capacity =  ESIA::SNMP::get($snmp,"$oid_disk_capacity.$id");
			    #nettoyage des variables
			    $disk_model =~ s/\n+//g;
			    $disk_capacity =~ s/\n+//g;
                if($disk_model ne "--")
                {
                    if(!$json)
                    {
                        $output .= "Model: $disk_model Descr : $disk_descr Capacity: $disk_capacity \n";
                    }
                    else
                    {
                        my %other = ('capacity' => "$disk_capacity");
                        my %hash = ('type' => 'HDD', 'name_component' => "$disk_model" , 'description_component' => "$disk_model", 'name' => "$disk_descr",
                                    'description' => "$disk_descr",'model' => "$disk_model",'version' => "1",'serial_number' => "",
                                    'other'=>  "");
                        $hash{other} =  \%other;
                        push(@jsonOutput, \%hash);
                    }
                }
             }
        }
    }
    if($json)
    {
       $output = encode_json(\@jsonOutput);
    }
}
else
{
	print ("UNKNOWN: Bad type parameter \n");
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC);
	exit $ERRORS{'UNKNOWN'};
}

if($output eq "")
{
    print "Status retrieval failed";
    exit $ERRORS{UNKNOWN};
}
else
{
    print $output.$graphdata;
    exit $return;
}
